@@ -64,57 +64,24 @@ Bvfs::Bvfs(JCR *j, B_DB *mdb) {
    attr = new_attr(jcr);
    list_entries = result_handler;
    user_data = this;
-   username = NULL;
 }
 
 Bvfs::~Bvfs() {
    free_pool_memory(jobids);
    free_pool_memory(pattern);
    free_pool_memory(prev_dir);
-   if (username) {
-      free(username);
-   }
    free_attr(attr);
    jcr->dec_use_count();
 }
 
-void Bvfs::filter_jobid()
-{
-   if (!username) {
-      return;
-   }
-
-   /* Query used by Bweb to filter clients, activated when using
-    * set_username()
-    */
-   POOL_MEM query;
-   Mmsg(query,
-      "SELECT DISTINCT JobId FROM Job JOIN Client USING (ClientId) "
-        "JOIN (SELECT ClientId FROM client_group_member "
-        "JOIN client_group USING (client_group_id) "
-        "JOIN bweb_client_group_acl USING (client_group_id) "
-        "JOIN bweb_user USING (userid) "
-       "WHERE bweb_user.username = '%s' "
-      ") AS filter USING (ClientId) "
-        " WHERE JobId IN (%s)",
-        username, jobids);
-
-   db_list_ctx ctx;
-   Dmsg1(dbglevel_sql, "q=%s\n", query.c_str());
-   db_sql_query(db, query.c_str(), db_list_handler, &ctx);
-   pm_strcpy(jobids, ctx.list);
-}
-
 void Bvfs::set_jobid(JobId_t id)
 {
    Mmsg(jobids, "%lld", (uint64_t)id);
-   filter_jobid();
 }
 
 void Bvfs::set_jobids(char *ids)
 {
    pm_strcpy(jobids, ids);
-   filter_jobid();
 }
 
 /*
@@ -410,10 +377,9 @@ static bool update_path_hierarchy_cache(JCR *jcr,
      "SELECT DISTINCT h.PPathId AS PathId "
        "FROM PathHierarchy AS h "
        "JOIN  PathVisibility AS p ON (h.PathId=p.PathId) "
-      "WHERE p.JobId=%s) AS a LEFT JOIN "
-       "(SELECT PathId "
-          "FROM PathVisibility "
-         "WHERE JobId=%s) AS b ON (a.PathId = b.PathId) "
+      "WHERE p.JobId=%s) AS a "
+     "LEFT JOIN PathVisibility AS b "
+       "ON (b.JobId=%s and a.PathId = b.PathId) "
    "WHERE b.PathId IS NULL",  jobid, jobid, jobid);
    }
 
@@ -453,45 +419,6 @@ void bvfs_update_cache(JCR *jcr, B_DB *mdb)
 
    db_lock(mdb);
 
-#ifdef xxx
-   /* TODO: Remove this code when updating make_bareos_table script */
-   Mmsg(mdb->cmd, "SELECT 1 FROM Job WHERE HasCache<>2 LIMIT 1");
-   if (!QUERY_DB(jcr, mdb, mdb->cmd)) {
-      Dmsg0(dbglevel, "Creating cache table\n");
-      Mmsg(mdb->cmd, "ALTER TABLE Job ADD HasCache int DEFAULT 0");
-      QUERY_DB(jcr, mdb, mdb->cmd);
-
-      Mmsg(mdb->cmd,
-           "CREATE TABLE PathHierarchy ( "
-           "PathId integer NOT NULL, "
-           "PPathId integer NOT NULL, "
-           "CONSTRAINT pathhierarchy_pkey "
-           "PRIMARY KEY (PathId))");
-      QUERY_DB(jcr, mdb, mdb->cmd);
-
-      Mmsg(mdb->cmd,
-           "CREATE INDEX pathhierarchy_ppathid "
-           "ON PathHierarchy (PPathId)");
-      QUERY_DB(jcr, mdb, mdb->cmd);
-
-      Mmsg(mdb->cmd,
-           "CREATE TABLE PathVisibility ("
-           "PathId integer NOT NULL, "
-           "JobId integer NOT NULL, "
-           "Size int8 DEFAULT 0, "
-           "Files int4 DEFAULT 0, "
-           "CONSTRAINT pathvisibility_pkey "
-           "PRIMARY KEY (JobId, PathId))");
-      QUERY_DB(jcr, mdb, mdb->cmd);
-
-      Mmsg(mdb->cmd,
-           "CREATE INDEX pathvisibility_jobid "
-           "ON PathVisibility (JobId)");
-      QUERY_DB(jcr, mdb, mdb->cmd);
-
-   }
-#endif
-
    Mmsg(mdb->cmd,
  "SELECT JobId from Job "
   "WHERE HasCache = 0 "
