@@ -187,7 +187,7 @@ int main (int argc, char *argv[])
 
    OSDependentInit();
 
-   if (configfile) {
+   if (configfile || (argc == 0)) {
       CATRES *catalog = NULL;
       int found = 0;
       if (argc > 0) {
@@ -251,11 +251,6 @@ int main (int argc, char *argv[])
          usage();
       }
 
-      if (argc < 1) {
-         Pmsg0(0, _("Working directory not supplied.\n"));
-         usage();
-      }
-
       /*
        * This is needed by SQLite to find the db
        */
@@ -314,10 +309,14 @@ int main (int argc, char *argv[])
                          password,
                          dbhost,
                          dbport,
-                         NULL);
+                         NULL,
+                         false,
+                         false,
+                         false,
+                         false);
    if (!db_open_database(NULL, db)) {
       Emsg1(M_FATAL, 0, "%s", db_strerror(db));
-          return 1;
+      return 1;
    }
 
    /*
@@ -372,7 +371,9 @@ static void print_catalog_details(CATRES *catalog, const char *working_dir)
                          catalog->db_port,
                          catalog->db_socket,
                          catalog->mult_db_connections,
-                         catalog->disable_batch_insert);
+                         catalog->disable_batch_insert,
+                         catalog->try_reconnect,
+                         catalog->exit_on_fatal);
    if (db) {
       printf("%sdb_type=%s\nworking_dir=%s\n", catalog->display(catalog_details),
              db->db_get_type(), working_directory);
@@ -531,7 +532,7 @@ static int get_name_handler(void *ctx, int num_fields, char **row)
    POOLMEM *name = (POOLMEM *)ctx;
 
    if (row[0]) {
-      pm_strcpy(&name, row[0]);
+      pm_strcpy(name, row[0]);
    }
    return 0;
 }
@@ -1334,7 +1335,7 @@ static void repair_bad_paths()
          /*
           * Add trailing slash
           */
-         len = pm_strcat(&name, "/");
+         len = pm_strcat(name, "/");
          db_escape_string(NULL, db, esc_name, name, len);
          bsnprintf(buf, sizeof(buf), "UPDATE Path SET Path='%s' WHERE PathId=%s",
             esc_name, edit_int64(id_list.Id[i], ed1));
