@@ -46,17 +46,47 @@ static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
  */
 static inline void register_callback_hooks(struct ndmlog *ixlog)
 {
+#ifdef HAVE_LMDB
+   NIS *nis = (NIS *)ixlog->ctx;
+
+   if (nis->jcr->res.client->ndmp_use_lmdb) {
+      ndmp_fhdb_lmdb_register(ixlog);
+   } else {
+      ndmp_fhdb_mem_register(ixlog);
+   }
+#else
    ndmp_fhdb_mem_register(ixlog);
+#endif
 }
 
 static inline void unregister_callback_hooks(struct ndmlog *ixlog)
 {
+#ifdef HAVE_LMDB
+   NIS *nis = (NIS *)ixlog->ctx;
+
+   if (nis->jcr->res.client->ndmp_use_lmdb) {
+      ndmp_fhdb_lmdb_unregister(ixlog);
+   } else {
+      ndmp_fhdb_mem_unregister(ixlog);
+   }
+#else
    ndmp_fhdb_mem_unregister(ixlog);
+#endif
 }
 
 static inline void process_fhdb(struct ndmlog *ixlog)
 {
+#ifdef HAVE_LMDB
+   NIS *nis = (NIS *)ixlog->ctx;
+
+   if (nis->jcr->res.client->ndmp_use_lmdb) {
+      ndmp_fhdb_lmdb_process_db(ixlog);
+   } else {
+      ndmp_fhdb_mem_process_db(ixlog);
+   }
+#else
    ndmp_fhdb_mem_process_db(ixlog);
+#endif
 }
 
 static inline int native_to_ndmp_level(JCR *jcr, char *filesystem)
@@ -237,13 +267,19 @@ static inline bool fill_backup_environment(JCR *jcr,
    }
 
    /*
-    * If we have a paired storage definition we put the storage daemon
-    * auth key and the filesystem into the tape device name of the
-    * NDMP session. This way the storage daemon can link the NDMP
-    * data and the normal save session together.
+    * If we have a paired storage definition we put the
+    * - Storage Daemon Auth Key
+    * - Filesystem
+    * - Dumplevel
+    * into the tape device name of the  NDMP session. This way the storage
+    * daemon can link the NDMP data and the normal save session together.
     */
    if (jcr->store_bsock) {
-      Mmsg(tape_device, "%s@%s", jcr->sd_auth_key, filesystem);
+      if (nbf_options && nbf_options->uses_level) {
+         Mmsg(tape_device, "%s@%s%%%d", jcr->sd_auth_key, filesystem, jcr->DumpLevel);
+      } else {
+         Mmsg(tape_device, "%s@%s", jcr->sd_auth_key, filesystem);
+      }
       job->tape_device = bstrdup(tape_device.c_str());
    }
 
@@ -349,7 +385,7 @@ bool do_ndmp_backup_init(JCR *jcr)
     */
    copy_wstorage(jcr, jcr->res.pool->storage, _("Pool resource"));
 
-   if (!jcr->wstorage) {
+   if (!jcr->res.wstorage) {
       Jmsg(jcr, M_FATAL, 0, _("No Storage specification found in Job or Pool.\n"));
       return false;
    }
@@ -443,7 +479,7 @@ bool do_ndmp_backup(JCR *jcr)
       /*
        * Now start a job with the Storage daemon
        */
-      if (!start_storage_daemon_job(jcr, NULL, jcr->wstorage)) {
+      if (!start_storage_daemon_job(jcr, NULL, jcr->res.wstorage)) {
          return false;
       }
 
@@ -527,22 +563,7 @@ bool do_ndmp_backup(JCR *jcr)
          nis->FileIndex = cnt + 1;
          nis->jcr = jcr;
          nis->save_filehist = jcr->res.job->SaveFileHist;
-
-         /*
-          * The full ndmp archive has a virtual filename, we need it to hardlink the individual
-          * file records to it. So we allocate it here once so its available during the whole
-          * NDMP session.
-          */
-         if (bstrcasecmp(jcr->backup_format, "dump")) {
-            Mmsg(virtual_filename, "/@NDMP%s%%%d", nis->filesystem, jcr->DumpLevel);
-         } else {
-            Mmsg(virtual_filename, "/@NDMP%s", nis->filesystem);
-         }
-
-         if (nis->virtual_filename) {
-            free(nis->virtual_filename);
-         }
-         nis->virtual_filename = bstrdup(virtual_filename.c_str());
+         nis->filehist_size = jcr->res.job->FileHistSize;
 
          ndmp_sess.param->log.ctx = nis;
          ndmp_sess.param->log_tag = bstrdup("DIR-NDMP");
@@ -559,6 +580,14 @@ bool do_ndmp_backup(JCR *jcr)
           * Copy the actual job to perform.
           */
          memcpy(&ndmp_sess.control_acb->job, &ndmp_job, sizeof(struct ndm_job_param));
+
+         /*
+          * We can use the same private pointer used in the logging with the JCR in
+          * the file index generation. We don't setup a index_log.deliver
+          * function as we catch the index information via callbacks.
+          */
+         ndmp_sess.control_acb->job.index_log.ctx = ndmp_sess.param->log.ctx;
+
          if (!fill_backup_environment(jcr,
                                       ie,
                                       nis->filesystem,
@@ -566,6 +595,22 @@ bool do_ndmp_backup(JCR *jcr)
             goto cleanup;
          }
 
+         /*
+          * The full ndmp archive has a virtual filename, we need it to hardlink the individual
+          * file records to it. So we allocate it here once so its available during the whole
+          * NDMP session.
+          */
+         if (bstrcasecmp(jcr->backup_format, "dump")) {
+            Mmsg(virtual_filename, "/@NDMP%s%%%d", nis->filesystem, jcr->DumpLevel);
+         } else {
+            Mmsg(virtual_filename, "/@NDMP%s", nis->filesystem);
+         }
+
+         if (nis->virtual_filename) {
+            free(nis->virtual_filename);
+         }
+         nis->virtual_filename = bstrdup(virtual_filename.c_str());
+
          ndma_job_auto_adjust(&ndmp_sess.control_acb->job);
          if (!ndmp_validate_job(jcr, &ndmp_sess.control_acb->job)) {
             goto cleanup;
@@ -588,12 +633,6 @@ bool do_ndmp_backup(JCR *jcr)
          ndmp_sess.conn_open = 1;
          ndmp_sess.conn_authorized = 1;
 
-         /*
-          * We can use the same private pointer used in the logging with the JCR in
-          * the file index generation. We don't setup a index_log.deliver
-          * function as we catch the index information via callbacks.
-          */
-         ndmp_sess.control_acb->job.index_log.ctx = ndmp_sess.param->log.ctx;
          register_callback_hooks(&ndmp_sess.control_acb->job.index_log);
 
          /*
