@@ -167,7 +167,7 @@ static inline bool do_native_restore_bootstrap(JCR *jcr)
       /*
        * Now start a job with the Storage daemon
        */
-      if (!start_storage_daemon_job(jcr, jcr->rstorage, NULL)) {
+      if (!start_storage_daemon_job(jcr, jcr->res.rstorage, NULL)) {
          goto bail_out;
       }
 
@@ -177,11 +177,17 @@ static inline bool do_native_restore_bootstrap(JCR *jcr)
           */
          jcr->setJobStatus(JS_WaitFD);
          jcr->keep_sd_auth_key = true; /* don't clear the sd_auth_key now */
-         if (!connect_to_file_daemon(jcr, 10, me->FDConnectTimeout, true, true)) {
+
+         if (!connect_to_file_daemon(jcr, 10, me->FDConnectTimeout, true)) {
             goto bail_out;
          }
+         send_job_info(jcr);
          fd = jcr->file_bsock;
 
+         if (!send_secure_erase_req_to_fd(jcr)) {
+            Dmsg1(500,"Unexpected %s secure erase\n","client");
+         }
+
          /*
           * Check if the file daemon supports passive client mode.
           */
@@ -236,8 +242,8 @@ static inline bool do_native_restore_bootstrap(JCR *jcr)
          /*
           * TLS Requirement
           */
-         if (store->tls_enable) {
-            if (store->tls_require) {
+         if (store->tls.enable) {
+            if (store->tls.require) {
                tls_need = BNET_TLS_REQUIRED;
             } else {
                tls_need = BNET_TLS_OK;
@@ -271,8 +277,8 @@ static inline bool do_native_restore_bootstrap(JCR *jcr)
           * TLS Requirement
           */
          tls_need = BNET_TLS_NONE;
-         if (client->tls_enable) {
-            if (client->tls_require) {
+         if (client->tls.enable) {
+            if (client->tls.require) {
                tls_need = BNET_TLS_REQUIRED;
             } else {
                tls_need = BNET_TLS_OK;
@@ -457,7 +463,7 @@ void native_restore_cleanup(JCR *jcr, int TermCode)
    update_job_end(jcr, TermCode);
 
    if (jcr->unlink_bsr && jcr->RestoreBootstrap) {
-      unlink(jcr->RestoreBootstrap);
+      secure_erase(jcr, jcr->RestoreBootstrap);
       jcr->unlink_bsr = false;
    }
 
@@ -520,6 +526,7 @@ void generate_restore_summary(JCR *jcr, int msg_type, const char *term_msg)
    char fd_term_msg[100], sd_term_msg[100];
    utime_t RunTime;
    double kbps;
+   POOL_MEM temp, secure_erase_status;
 
    bstrftimes(sdt, sizeof(sdt), jcr->jr.StartTime);
    bstrftimes(edt, sizeof(edt), jcr->jr.EndTime);
@@ -568,6 +575,19 @@ void generate_restore_summary(JCR *jcr, int msg_type, const char *term_msg)
            term_msg);
       break;
    default:
+      if (me->secure_erase_cmdline) {
+         Mmsg(temp,"  Dir Secure Erase Cmd:   %s\n", me->secure_erase_cmdline);
+         pm_strcat(secure_erase_status, temp.c_str());
+      }
+      if (!bstrcmp(jcr->FDSecureEraseCmd, "*None*")) {
+         Mmsg(temp,"  FD  Secure Erase Cmd:   %s\n", jcr->FDSecureEraseCmd);
+         pm_strcat(secure_erase_status, temp.c_str());
+      }
+      if (!bstrcmp(jcr->SDSecureEraseCmd, "*None*")) {
+         Mmsg(temp,"  SD  Secure Erase Cmd:   %s\n", jcr->SDSecureEraseCmd);
+         pm_strcat(secure_erase_status, temp.c_str());
+      }
+
       Jmsg(jcr, msg_type, 0, _("%s %s %s (%s):\n"
            "  Build OS:               %s %s %s\n"
            "  JobId:                  %d\n"
@@ -583,6 +603,7 @@ void generate_restore_summary(JCR *jcr, int msg_type, const char *term_msg)
            "  FD Errors:              %d\n"
            "  FD termination status:  %s\n"
            "  SD termination status:  %s\n"
+           "%s"
            "  Termination:            %s\n\n"),
            BAREOS, my_name, VERSION, LSMDATE,
            HOST_OS, DISTNAME, DISTVER,
@@ -599,6 +620,7 @@ void generate_restore_summary(JCR *jcr, int msg_type, const char *term_msg)
            jcr->JobErrors,
            fd_term_msg,
            sd_term_msg,
+           secure_erase_status.c_str(),
            term_msg);
       break;
    }
