@@ -32,7 +32,6 @@
 #include "dird.h"
 
 /* Imported variables */
-extern struct s_jt jobtypes[];
 
 /* Imported functions */
 
@@ -98,7 +97,7 @@ int file_delete_handler(void *ctx, int num_fields, char **row)
  * prune stats
  * prune directory=xxx [client=xxx] [recursive]
  */
-int prune_cmd(UAContext *ua, const char *cmd)
+bool prune_cmd(UAContext *ua, const char *cmd)
 {
    CLIENTRES *client;
    POOLRES *pool;
@@ -160,8 +159,7 @@ int prune_cmd(UAContext *ua, const char *cmd)
 
       return true;
    case 1: { /* prune jobs */
-      int i;
-      char jobtype[MAX_NAME_LENGTH];
+      int jobtype = -1;
 
       if (!(client = get_client_resource(ua))) {
          return false;
@@ -176,28 +174,8 @@ int prune_cmd(UAContext *ua, const char *cmd)
       /*
        * Ask what jobtype to prune.
        */
-      if ((i = find_arg_with_value(ua, NT_("jobtype"))) >= 0) {
-         bstrncpy(jobtype, ua->argv[i], sizeof(jobtype));
-      } else {
-         start_prompt(ua, _("Jobtype to prune:\n"));
-         for (i = 0; jobtypes[i].type_name; i++) {
-            add_prompt(ua, jobtypes[i].type_name);
-         }
-
-         if (do_prompt(ua, _("JobType"),  _("Select Job Type"), jobtype, sizeof(jobtype)) < 0) {
-            return true;
-         }
-      }
-
-      for (i = 0; jobtypes[i].type_name; i++) {
-         if (bstrcasecmp(jobtypes[i].type_name, jobtype)) {
-            break;
-         }
-      }
-
-      if (!jobtypes[i].type_name) {
-         ua->warning_msg(_("Illegal jobtype %s.\n"), jobtype);
-         return false;
+      if (!get_user_job_type_selection(ua, &jobtype) || jobtype == -1) {
+         return (jobtype == -1) ? true : false;
       }
 
       /*
@@ -211,11 +189,7 @@ int prune_cmd(UAContext *ua, const char *cmd)
          return false;
       }
 
-      if (jobtypes[i].type_name) {
-         return prune_jobs(ua, client, pool, jobtypes[i].job_type);
-      }
-
-      return false;
+      return prune_jobs(ua, client, pool, jobtype);
    }
    case 2: /* prune volume */
       if (!select_pool_and_media_dbr(ua, &pr, &mr)) {
@@ -279,8 +253,7 @@ static bool prune_directory(UAContext *ua, CLIENTRES *client)
     */
    if (!client) {
       if (!get_yesno(ua, _("No client restriction given really remove "
-                           "directory for all clients (yes/no): ")) ||
-          ua->pint32_val == 0) {
+                           "directory for all clients (yes/no): ")) || !ua->pint32_val) {
          if (!(client = get_client_resource(ua))) {
             return false;
          }
@@ -364,8 +337,7 @@ static bool prune_directory(UAContext *ua, CLIENTRES *client)
     * them anymore.
     */
    if (!client) {
-      if (!get_yesno(ua, _("Cleanup orphaned path records (yes/no):")) ||
-          ua->pint32_val == 0) {
+      if (!get_yesno(ua, _("Cleanup orphaned path records (yes/no):")) || !ua->pint32_val) {
          retval = true;
          goto bail_out;
       }
@@ -565,10 +537,7 @@ bail_out:
 
 static void drop_temp_tables(UAContext *ua)
 {
-   int i;
-   for (i=0; drop_deltabs[i]; i++) {
-      db_sql_query(ua->db, drop_deltabs[i]);
-   }
+   db_sql_query(ua->db, drop_deltabs[db_get_type_index(ua->db)]);
 }
 
 static bool create_temp_tables(UAContext *ua)
@@ -607,33 +576,39 @@ struct accurate_check_ctx {
    DBId_t FileSetId;                  /* Id of FileSet */
 };
 
-/* row: Job.Name, FileSet, Client.Name, FileSetId, ClientId, Type */
+/*
+ * row: Job.Name, FileSet, Client.Name, FileSetId, ClientId, Type
+ */
 static int job_select_handler(void *ctx, int num_fields, char **row)
 {
    alist *lst = (alist *)ctx;
    struct accurate_check_ctx *res;
    ASSERT(num_fields == 6);
 
-   /* Quick fix for #5507, avoid locking res_head after db_lock() */
-
-#ifdef bug5507
-   /* If this job doesn't exist anymore in the configuration, delete it */
-   if (GetResWithName(R_JOB, row[0]) == NULL) {
+   /*
+    * If this job doesn't exist anymore in the configuration, delete it.
+    */
+   if (GetResWithName(R_JOB, row[0], false) == NULL) {
       return 0;
    }
 
-   /* If this fileset doesn't exist anymore in the configuration, delete it */
-   if (GetResWithName(R_FILESET, row[1]) == NULL) {
+   /*
+    * If this fileset doesn't exist anymore in the configuration, delete it.
+    */
+   if (GetResWithName(R_FILESET, row[1], false) == NULL) {
       return 0;
    }
 
-   /* If this client doesn't exist anymore in the configuration, delete it */
-   if (GetResWithName(R_CLIENT, row[2]) == NULL) {
+   /*
+    * If this client doesn't exist anymore in the configuration, delete it.
+    */
+   if (GetResWithName(R_CLIENT, row[2], false) == NULL) {
       return 0;
    }
-#endif
 
-   /* Don't compute accurate things for Verify jobs */
+   /*
+    * Don't compute accurate things for Verify jobs
+    */
    if (*row[5] == 'V') {
       return 0;
    }
@@ -703,7 +678,7 @@ static bool prune_backup_jobs(UAContext *ua, CLIENTRES *client, POOLRES *pool)
 
    /*
     * Select all files that are older than the JobRetention period
-    *  and add them into the "DeletionCandidates" table.
+    * and add them into the "DeletionCandidates" table.
     */
    Mmsg(query,
         "INSERT INTO DelCandidates "
@@ -721,7 +696,8 @@ static bool prune_backup_jobs(UAContext *ua, CLIENTRES *client, POOLRES *pool)
       goto bail_out;
    }
 
-   /* Now, for the selection, we discard some of them in order to be always
+   /*
+    * Now, for the selection, we discard some of them in order to be always
     * able to restore files. (ie, last full, last diff, last incrs)
     * Note: The DISTINCT could be more useful if we don't get FileSetId
     */
@@ -737,20 +713,25 @@ static bool prune_backup_jobs(UAContext *ua, CLIENTRES *client, POOLRES *pool)
    "AND Job.JobStatus IN ('T', 'W') "     /* Look only useful jobs */
       );
 
-   /* The job_select_handler will skip jobs or filesets that are no longer
+   /*
+    * The job_select_handler will skip jobs or filesets that are no longer
     * in the configuration file. Interesting ClientId/FileSetId will be
-    * added to jobids_check (currently disabled in 6.0.7b)
+    * added to jobids_check.
     */
    if (!db_sql_query(ua->db, query.c_str(), job_select_handler, jobids_check)) {
       ua->error_msg("%s", db_strerror(ua->db));
    }
 
-   /* For this selection, we exclude current jobs used for restore or
+   /*
+    * For this selection, we exclude current jobs used for restore or
     * accurate. This will prevent to prune the last full backup used for
     * current backup & restore
     */
    memset(&jr, 0, sizeof(jr));
-   /* To find useful jobs, we do like an incremental */
+
+   /*
+    * To find useful jobs, we do like an incremental
+    */
    jr.JobLevel = L_INCREMENTAL;
    foreach_alist(elt, jobids_check) {
       jr.ClientId = elt->ClientId;   /* should be always the same */
@@ -759,7 +740,8 @@ static bool prune_backup_jobs(UAContext *ua, CLIENTRES *client, POOLRES *pool)
       jobids.add(tempids);
    }
 
-   /* Discard latest Verify level=InitCatalog job
+   /*
+    * Discard latest Verify level=InitCatalog job
     * TODO: can have multiple fileset
     */
    Mmsg(query,
