@@ -36,18 +36,36 @@ extern void *start_heap;
 static void list_scheduled_jobs(UAContext *ua);
 static void list_running_jobs(UAContext *ua);
 static void list_terminated_jobs(UAContext *ua);
+static void list_connected_clients(UAContext *ua);
 static void do_director_status(UAContext *ua);
 static void do_scheduler_status(UAContext *ua);
 static bool do_subscription_status(UAContext *ua);
 static void do_all_status(UAContext *ua);
 static void status_slots(UAContext *ua, STORERES *store);
-static void status_content(UAContext *ua, STORERES *store);
+static void status_content_api(UAContext *ua, STORERES *store);
+static void status_content_json(UAContext *ua, STORERES *store);
 
-static char OKqstatus[] =
+static char OKdotstatus[] =
    "1000 OK .status\n";
 static char DotStatusJob[] =
    "JobId=%s JobStatus=%c JobErrors=%d\n";
 
+static void client_status(UAContext *ua, CLIENTRES *client, char *cmd)
+{
+   switch (client->Protocol) {
+   case APT_NATIVE:
+      do_native_client_status(ua, client, cmd);
+      break;
+   case APT_NDMPV2:
+   case APT_NDMPV3:
+   case APT_NDMPV4:
+      do_ndmp_client_status(ua, client, cmd);
+      break;
+   default:
+      break;
+   }
+}
+
 /*
  * .status command
  */
@@ -58,28 +76,29 @@ bool dot_status_cmd(UAContext *ua, const char *cmd)
    JCR* njcr = NULL;
    s_last_job* job;
    char ed1[50];
+   char *statuscmd = NULL;
 
    Dmsg2(20, "status=\"%s\" argc=%d\n", cmd, ua->argc);
 
-   if (ua->argc < 3) {
+   if (ua->argc < 2) {
       ua->send_msg("1900 Bad .status command, missing arguments.\n");
       return false;
    }
 
    if (bstrcasecmp(ua->argk[1], "dir")) {
       if (bstrcasecmp(ua->argk[2], "current")) {
-         ua->send_msg(OKqstatus, ua->argk[2]);
+         ua->send_msg(OKdotstatus, ua->argk[2]);
          foreach_jcr(njcr) {
-            if (njcr->JobId != 0 && acl_access_ok(ua, Job_ACL, njcr->res.job->name())) {
+            if (njcr->JobId != 0 && ua->acl_access_ok(Job_ACL, njcr->res.job->name())) {
                ua->send_msg(DotStatusJob, edit_int64(njcr->JobId, ed1), njcr->JobStatus, njcr->JobErrors);
             }
          }
          endeach_jcr(njcr);
       } else if (bstrcasecmp(ua->argk[2], "last")) {
-         ua->send_msg(OKqstatus, ua->argk[2]);
+         ua->send_msg(OKdotstatus, ua->argk[2]);
          if ((last_jobs) && (last_jobs->size() > 0)) {
             job = (s_last_job*)last_jobs->last();
-            if (acl_access_ok(ua, Job_ACL, job->Job)) {
+            if (ua->acl_access_ok(Job_ACL, job->Job)) {
                ua->send_msg(DotStatusJob, edit_int64(job->JobId, ed1), job->JobStatus, job->Errors);
             }
          }
@@ -98,31 +117,19 @@ bool dot_status_cmd(UAContext *ua, const char *cmd)
    } else if (bstrcasecmp(ua->argk[1], "client")) {
       client = get_client_resource(ua);
       if (client) {
-         Dmsg2(200, "Client=%s arg=%s\n", client->name(), NPRT(ua->argk[2]));
-         switch (client->Protocol) {
-         case APT_NDMPV2:
-         case APT_NDMPV3:
-         case APT_NDMPV4:
-            do_ndmp_client_status(ua, client, ua->argk[2]);
-            break;
-         default:
-            do_native_client_status(ua, client, ua->argk[2]);
-            break;
+         if (ua->argc == 3) {
+            statuscmd = ua->argk[2];
          }
+         Dmsg2(200, "Client=%s arg=%s\n", client->name(), NPRT(statuscmd));
+         client_status(ua, client, statuscmd);
       }
    } else if (bstrcasecmp(ua->argk[1], "storage")) {
       store = get_storage_resource(ua);
       if (store) {
-         switch (store->Protocol) {
-         case APT_NDMPV2:
-         case APT_NDMPV3:
-         case APT_NDMPV4:
-            do_ndmp_storage_status(ua, store, ua->argk[2]);
-            break;
-         default:
-            do_native_storage_status(ua, store, ua->argk[2]);
-            break;
+         if (ua->argc == 3) {
+            statuscmd = ua->argk[2];
          }
+         storage_status(ua, store, statuscmd);
       }
    } else {
       ua->send_msg("1900 Bad .status command, wrong argument.\n");
@@ -132,19 +139,10 @@ bool dot_status_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
-/* This is the *old* command handler, so we must return
- *  1 or it closes the connection
- */
-int qstatus_cmd(UAContext *ua, const char *cmd)
-{
-   dot_status_cmd(ua, cmd);
-   return 1;
-}
-
 /*
  * status command
  */
-int status_cmd(UAContext *ua, const char *cmd)
+bool status_cmd(UAContext *ua, const char *cmd)
 {
    STORERES *store;
    CLIENTRES *client;
@@ -156,33 +154,24 @@ int status_cmd(UAContext *ua, const char *cmd)
    for (i = 1; i < ua->argc; i++) {
       if (bstrcasecmp(ua->argk[i], NT_("all"))) {
          do_all_status(ua);
-         return 1;
+         return true;
       } else if (bstrncasecmp(ua->argk[i], NT_("dir"), 3)) {
          do_director_status(ua);
-         return 1;
+         return true;
       } else if (bstrcasecmp(ua->argk[i], NT_("client"))) {
          client = get_client_resource(ua);
          if (client) {
-            switch (client->Protocol) {
-            case APT_NDMPV2:
-            case APT_NDMPV3:
-            case APT_NDMPV4:
-               do_ndmp_client_status(ua, client, NULL);
-               break;
-            default:
-               do_native_client_status(ua, client, NULL);
-               break;
-            }
+            client_status(ua, client, NULL);
          }
-         return 1;
+         return true;
       } else if (bstrncasecmp(ua->argk[i], NT_("sched"), 5)) {
          do_scheduler_status(ua);
-         return 1;
+         return true;
       } else if (bstrncasecmp(ua->argk[i], NT_("sub"), 3)) {
          if (do_subscription_status(ua)) {
-            return 1;
+            return true;
          } else {
-            return 0;
+            return false;
          }
       } else {
          /*
@@ -193,29 +182,23 @@ int status_cmd(UAContext *ua, const char *cmd)
 
          if (store) {
             if (find_arg(ua, NT_("slots")) > 0) {
-               switch (store->Protocol) {
-               case APT_NDMPV2:
-               case APT_NDMPV3:
-               case APT_NDMPV4:
-                  break;
-               default:
+               switch (ua->api) {
+               case API_MODE_OFF:
                   status_slots(ua, store);
                   break;
+               case API_MODE_ON:
+                  status_content_api(ua, store);
+                  break;
+               case API_MODE_JSON:
+                  status_content_json(ua, store);
+                  break;
                }
             } else {
-               switch (store->Protocol) {
-               case APT_NDMPV2:
-               case APT_NDMPV3:
-               case APT_NDMPV4:
-                  do_ndmp_storage_status(ua, store, NULL);
-                  break;
-               default:
-                  do_native_storage_status(ua, store, NULL);
-                  break;
-               }
+               storage_status(ua, store, NULL);
             }
          }
-         return 1;
+
+         return true;
       }
    }
    /* If no args, ask for status type */
@@ -230,7 +213,7 @@ int status_cmd(UAContext *ua, const char *cmd)
       add_prompt(ua, NT_("All"));
       Dmsg0(20, "do_prompt: select daemon\n");
       if ((item=do_prompt(ua, "",  _("Select daemon type for status"), prmt, sizeof(prmt))) < 0) {
-         return 1;
+         return true;
       }
       Dmsg1(20, "item=%d\n", item);
       switch (item) {
@@ -240,31 +223,13 @@ int status_cmd(UAContext *ua, const char *cmd)
       case 1:
          store = select_storage_resource(ua);
          if (store) {
-            switch (store->Protocol) {
-            case APT_NDMPV2:
-            case APT_NDMPV3:
-            case APT_NDMPV4:
-               do_ndmp_storage_status(ua, store, NULL);
-               break;
-            default:
-               do_native_storage_status(ua, store, NULL);
-               break;
-            }
+            storage_status(ua, store, NULL);
          }
          break;
       case 2:
          client = select_client_resource(ua);
          if (client) {
-            switch (client->Protocol) {
-            case APT_NDMPV2:
-            case APT_NDMPV3:
-            case APT_NDMPV4:
-               do_ndmp_client_status(ua, client, NULL);
-               break;
-            default:
-               do_native_client_status(ua, client, NULL);
-               break;
-            }
+            client_status(ua, client, NULL);
          }
          break;
       case 3:
@@ -277,7 +242,7 @@ int status_cmd(UAContext *ua, const char *cmd)
          break;
       }
    }
-   return 1;
+   return true;
 }
 
 static void do_all_status(UAContext *ua)
@@ -300,7 +265,7 @@ static void do_all_status(UAContext *ua)
    i = 0;
    foreach_res(store, R_STORAGE) {
       found = false;
-      if (!acl_access_ok(ua, Storage_ACL, store->name())) {
+      if (!ua->acl_access_ok(Storage_ACL, store->name())) {
          continue;
       }
       for (j = 0; j < i; j++) {
@@ -319,16 +284,7 @@ static void do_all_status(UAContext *ua)
 
    /* Call each unique Storage daemon */
    for (j = 0; j < i; j++) {
-      switch (unique_store[j]->Protocol) {
-      case APT_NDMPV2:
-      case APT_NDMPV3:
-      case APT_NDMPV4:
-         do_ndmp_storage_status(ua, unique_store[j], NULL);
-         break;
-      default:
-         do_native_storage_status(ua, unique_store[j], NULL);
-         break;
-      }
+      storage_status(ua, unique_store[j], NULL);
    }
    free(unique_store);
 
@@ -343,7 +299,7 @@ static void do_all_status(UAContext *ua)
    i = 0;
    foreach_res(client, R_CLIENT) {
       found = false;
-      if (!acl_access_ok(ua, Client_ACL, client->name())) {
+      if (!ua->acl_access_ok(Client_ACL, client->name())) {
          continue;
       }
       for (j = 0; j < i; j++) {
@@ -362,16 +318,7 @@ static void do_all_status(UAContext *ua)
 
    /* Call each unique File daemon */
    for (j = 0; j < i; j++) {
-      switch (unique_client[j]->Protocol) {
-      case APT_NDMPV2:
-      case APT_NDMPV3:
-      case APT_NDMPV4:
-         do_ndmp_client_status(ua, unique_client[j], NULL);
-         break;
-      default:
-         do_native_client_status(ua, unique_client[j], NULL);
-         break;
-      }
+      client_status(ua, unique_client[j], NULL);
    }
    free(unique_client);
 
@@ -379,22 +326,37 @@ static void do_all_status(UAContext *ua)
 
 void list_dir_status_header(UAContext *ua)
 {
-   int len;
+   int len, cnt;
+   CATRES *catalog;
    char dt[MAX_TIME_LENGTH];
    char b1[35], b2[35], b3[35], b4[35], b5[35];
-   POOL_MEM msg(PM_FNAME);
+   POOL_MEM msg(PM_FNAME),
+            dbdrivers(PM_FNAME);
 
+   cnt = 0;
+   foreach_res(catalog, R_CATALOG) {
+      if (cnt) {
+         dbdrivers.strcat(" ");
+      }
+      dbdrivers.strcat(catalog->db_driver);
+      cnt++;
+   }
    ua->send_msg(_("%s Version: %s (%s) %s %s %s\n"), my_name, VERSION, BDATE,
                 HOST_OS, DISTNAME, DISTVER);
    bstrftime_nc(dt, sizeof(dt), daemon_start_time);
-   ua->send_msg(_("Daemon started %s. Jobs: run=%d, running=%d mode=%d\n"),
-                dt, num_jobs_run, job_count(), (int)DEVELOPER_MODE);
+   ua->send_msg(_("Daemon started %s. Jobs: run=%d, running=%d mode=%d db=%s\n"),
+                dt, num_jobs_run, job_count(), (int)DEVELOPER_MODE, dbdrivers.c_str() );
    ua->send_msg(_(" Heap: heap=%s smbytes=%s max_bytes=%s bufs=%s max_bufs=%s\n"),
                 edit_uint64_with_commas((char *)sbrk(0)-(char *)start_heap, b1),
                 edit_uint64_with_commas(sm_bytes, b2),
                 edit_uint64_with_commas(sm_max_bytes, b3),
                 edit_uint64_with_commas(sm_buffers, b4),
                 edit_uint64_with_commas(sm_max_buffers, b5));
+
+   if (me->secure_erase_cmdline) {
+      ua->send_msg(_(" secure erase command='%s'\n"), me->secure_erase_cmdline);
+   }
+
    len = list_dir_plugins(msg);
    if (len > 0) {
       ua->send_msg("%s\n", msg.c_str());
@@ -626,7 +588,7 @@ static void do_scheduler_status(UAContext *ua)
     */
    i = find_arg_with_value(ua, NT_("job"));
    if (i >= 0) {
-      job = GetJobResWithName(ua->argv[i]);
+      job = ua->GetJobResWithName(ua->argv[i]);
 
       /*
        * If a bogus jobname was given ask for it interactively.
@@ -648,7 +610,7 @@ static void do_scheduler_status(UAContext *ua)
          continue;
       }
 
-      if (!acl_access_ok(ua, Schedule_ACL, sched->hdr.name)) {
+      if (!ua->acl_access_ok(Schedule_ACL, sched->hdr.name)) {
          continue;
       }
 
@@ -667,7 +629,7 @@ static void do_scheduler_status(UAContext *ua)
          }
       } else {
          foreach_res(job, R_JOB) {
-            if (!acl_access_ok(ua, Job_ACL, job->hdr.name)) {
+            if (!ua->acl_access_ok(Job_ACL, job->hdr.name)) {
                continue;
             }
 
@@ -725,7 +687,7 @@ start_again:
          } else {
             LockRes();
             foreach_res(job, R_JOB) {
-               if (!acl_access_ok(ua, Job_ACL, job->hdr.name)) {
+               if (!ua->acl_access_ok(Job_ACL, job->hdr.name)) {
                   continue;
                }
 
@@ -747,7 +709,11 @@ start_again:
           */
          LockRes();
          foreach_res(sched, R_SCHEDULE) {
-            if (!acl_access_ok(ua, Schedule_ACL, sched->hdr.name)) {
+            if (!schedulegiven && !sched->enabled) {
+               continue;
+            }
+
+            if (!ua->acl_access_ok(Schedule_ACL, sched->hdr.name)) {
                continue;
             }
 
@@ -795,6 +761,9 @@ static void do_director_status(UAContext *ua)
     * List terminated jobs
     */
    list_terminated_jobs(ua);
+
+   list_connected_clients(ua);
+
    ua->send_msg("====\n");
 }
 
@@ -844,7 +813,7 @@ static void prt_runtime(UAContext *ua, sched_pkt *sp)
          set_storageid_in_mr(jcr->res.wstore, &mr);
          Dmsg0(250, "call find_next_volume_for_append\n");
          /* no need to set ScratchPoolId, since we use fnv_no_create_vol */
-         ok = find_next_volume_for_append(jcr, &mr, 1, fnv_no_create_vol, fnv_no_prune);
+         ok = find_next_volume_for_append(jcr, &mr, 1, NULL, fnv_no_create_vol, fnv_no_prune);
       }
       if (!ok) {
          bstrncpy(mr.VolumeName, "*unknown*", sizeof(mr.VolumeName));
@@ -853,6 +822,7 @@ static void prt_runtime(UAContext *ua, sched_pkt *sp)
    bstrftime_nc(dt, sizeof(dt), sp->runtime);
    switch (sp->job->JobType) {
    case JT_ADMIN:
+   case JT_ARCHIVE:
    case JT_RESTORE:
       level_ptr = " ";
       break;
@@ -931,7 +901,7 @@ static void list_scheduled_jobs(UAContext *ua)
     */
    LockRes();
    foreach_res(job, R_JOB) {
-      if (!acl_access_ok(ua, Job_ACL, job->name()) ||
+      if (!ua->acl_access_ok(Job_ACL, job->name()) ||
           !job->enabled ||
           (job->client && !job->client->enabled)) {
          continue;
@@ -1013,7 +983,7 @@ static void list_running_jobs(UAContext *ua)
       ua->send_msg(_("======================================================================\n"));
    }
    foreach_jcr(jcr) {
-      if (jcr->JobId == 0 || !acl_access_ok(ua, Job_ACL, jcr->res.job->name())) {
+      if (jcr->JobId == 0 || !ua->acl_access_ok(Job_ACL, jcr->res.job->name())) {
          continue;
       }
       njobs++;
@@ -1140,7 +1110,12 @@ static void list_running_jobs(UAContext *ua)
             emsg = (char *)get_pool_memory(PM_FNAME);
             pool_mem = true;
          }
-         if (!jcr->res.client || !jcr->res.wstore) {
+         if (!jcr->file_bsock) {
+            /*
+             * client initiated connection
+             */
+            Mmsg(emsg, _("is waiting for Client to connect (Client Initiated Connection)"));
+         } else if (!jcr->res.client || !jcr->res.wstore) {
             Mmsg(emsg, _("is waiting for Client to connect to Storage daemon"));
          } else {
             Mmsg(emsg, _("is waiting for Client %s to connect to Storage %s"),
@@ -1163,6 +1138,7 @@ static void list_running_jobs(UAContext *ua)
       }
       switch (jcr->getJobType()) {
       case JT_ADMIN:
+      case JT_ARCHIVE:
       case JT_RESTORE:
          bstrncpy(level, "      ", sizeof(level));
          break;
@@ -1225,13 +1201,14 @@ static void list_terminated_jobs(UAContext *ua)
          }
       }
 
-      if (!acl_access_ok(ua, Job_ACL, JobName)) {
+      if (!ua->acl_access_ok(Job_ACL, JobName)) {
          continue;
       }
 
       bstrftime_nc(dt, sizeof(dt), je->end_time);
       switch (je->JobType) {
       case JT_ADMIN:
+      case JT_ARCHIVE:
       case JT_RESTORE:
          bstrncpy(level, "    ", sizeof(level));
          break;
@@ -1286,13 +1263,38 @@ static void list_terminated_jobs(UAContext *ua)
    unlock_last_jobs_list();
 }
 
-static void content_send_info(UAContext *ua, char type, int Slot, char *vol_name)
+
+static void list_connected_clients(UAContext *ua)
+{
+   CONNECTION *connection = NULL;
+   alist *connections = NULL;
+   const char *separator = "====================";
+   char dt[MAX_TIME_LENGTH];
+
+   ua->send->decoration("\n");
+   ua->send->decoration("Client Initiated Connections (waiting for jobs):\n");
+   connections = get_client_connections()->get_as_alist();
+   ua->send->decoration("%-20s%-20s%-20s%-40s\n", "Connect time", "Protocol", "Authenticated", "Name");
+   ua->send->decoration("%-20s%-20s%-20s%-20s%-20s\n", separator, separator, separator, separator, separator);
+   foreach_alist(connection, connections) {
+      ua->send->object_start("client-connection");
+      bstrftime_nc(dt, sizeof(dt), connection->connect_time());
+      ua->send->object_key_value("connect_time", dt, "%-20s");
+      ua->send->object_key_value("protocol_version", connection->protocol_version(), "%-20d");
+      ua->send->object_key_value("authenticated", connection->authenticated(), "%-20d");
+      ua->send->object_key_value("name", connection->name(), "%-40s");
+      ua->send->object_end("client-connection");
+      ua->send->decoration("\n");
+   }
+}
+
+static void content_send_info_api(UAContext *ua, char type, int Slot, char *vol_name)
 {
    char ed1[50], ed2[50], ed3[50];
    POOL_DBR pr;
    MEDIA_DBR mr;
    /* Type|Slot|RealSlot|Volume|Bytes|Status|MediaType|Pool|LastW|Expire */
-   const char *slot_api_full_format="%c|%d|%d|%s|%s|%s|%s|%s|%s|%s\n";
+   const char *slot_api_full_format="%c|%hd|%hd|%s|%s|%s|%s|%s|%s|%s\n";
 
    memset(&pr, 0, sizeof(pr));
    memset(&mr, 0, sizeof(mr));
@@ -1308,13 +1310,56 @@ static void content_send_info(UAContext *ua, char type, int Slot, char *vol_name
                    edit_uint64(mr.VolBytes, ed1),
                    mr.VolStatus, mr.MediaType, pr.Name,
                    edit_uint64(mr.LastWritten, ed2),
-                   edit_uint64(mr.LastWritten+mr.VolRetention, ed3));
-
+                   edit_uint64(mr.LastWritten + mr.VolRetention, ed3));
    } else {                  /* Media unknown */
       ua->send_msg(slot_api_full_format,
                    type, Slot, 0, mr.VolumeName, "?", "?", "?", "?",
                    "0", "0");
+   }
+}
 
+static void content_send_info_json(UAContext *ua, const char *type, int Slot, char *vol_name)
+{
+   POOL_DBR pr;
+   MEDIA_DBR mr;
+
+   memset(&pr, 0, sizeof(pr));
+   memset(&mr, 0, sizeof(mr));
+
+   bstrncpy(mr.VolumeName, vol_name, sizeof(mr.VolumeName));
+   if (db_get_media_record(ua->jcr, ua->db, &mr)) {
+      pr.PoolId = mr.PoolId;
+      if (!db_get_pool_record(ua->jcr, ua->db, &pr)) {
+         strcpy(pr.Name, "?");
+      }
+
+      ua->send->object_start();
+      ua->send->object_key_value("type", type, "%s\n");
+      ua->send->object_key_value("slotnr", Slot, "%hd\n");
+      ua->send->object_key_value("content", "full", "%s\n");
+      ua->send->object_key_value("mr_slotnr", mr.Slot, "%lld\n");
+      ua->send->object_key_value("mr_volname", mr.VolumeName, "%s\n");
+      ua->send->object_key_value("mr_volbytes", mr.VolBytes, "%lld\n");
+      ua->send->object_key_value("mr_volstatus", mr.VolStatus, "%s\n");
+      ua->send->object_key_value("mr_mediatype", mr.MediaType, "%s\n");
+      ua->send->object_key_value("pr_name", pr.Name, "%s\n");
+      ua->send->object_key_value("mr_lastwritten", mr.LastWritten, "%lld\n");
+      ua->send->object_key_value("mr_expire", mr.LastWritten + mr.VolRetention, "%lld\n");
+      ua->send->object_end();
+   } else {                  /* Media unknown */
+      ua->send->object_start();
+      ua->send->object_key_value("type", type, "%s\n");
+      ua->send->object_key_value("slotnr", Slot, "%hd\n");
+      ua->send->object_key_value("content", "full", "%s\n");
+      ua->send->object_key_value("mr_slotnr", (uint64_t)0, "%lld\n");
+      ua->send->object_key_value("mr_volname", mr.VolumeName, "%s\n");
+      ua->send->object_key_value("mr_volbytes", "?", "%s\n");
+      ua->send->object_key_value("mr_volstatus", "?", "%s\n");
+      ua->send->object_key_value("mr_mediatype", "?", "%s\n");
+      ua->send->object_key_value("pr_name", "?", "%s\n");
+      ua->send->object_key_value("mr_lastwritten", (uint64_t)0, "%lld\n");
+      ua->send->object_key_value("mr_expire", (uint64_t)0, "%lld\n");
+      ua->send->object_end();
    }
 }
 
@@ -1350,28 +1395,26 @@ static void content_send_info(UAContext *ua, char type, int Slot, char *vol_name
  * S|1|1|vol1|31417344|Full|LTO1-ANSI|Inc|1250858902|1282394902
  * S|2||||||||
  * S|3|3|vol4|15869952|Append|LTO1-ANSI|Inc|1250858907|1282394907
- *
  */
-static void status_content(UAContext *ua, STORERES *store)
+static void status_content_api(UAContext *ua, STORERES *store)
 {
-   bool found;
    vol_list_t *vl1, *vl2;
-   dlist *vol_list = NULL;
-   const char *slot_api_drive_full_format="%c|%d|%d|%s\n";
-   const char *slot_api_drive_empty_format="%c|%d||\n";
-   const char *slot_api_slot_empty_format="%c|%d||||||||\n";
+   changer_vol_list_t *vol_list = NULL;
+   const char *slot_api_drive_full_format="%c|%hd|%hd|%s\n";
+   const char *slot_api_drive_empty_format="%c|%hd||\n";
+   const char *slot_api_slot_empty_format="%c|%hd||||||||\n";
 
    if (!open_client_db(ua)) {
       return;
    }
 
-   vol_list = get_vol_list_from_SD(ua, store, true /* listall */ , true /* want to see all slots */);
+   vol_list = get_vol_list_from_storage(ua, store, true /* listall */ , true /* want to see all slots */);
    if (!vol_list) {
       ua->warning_msg(_("No Volumes found, or no barcodes.\n"));
       goto bail_out;
    }
 
-   foreach_dlist(vl1, vol_list) {
+   foreach_dlist(vl1, vol_list->contents) {
       switch (vl1->Type) {
       case slot_type_drive:
          switch (vl1->Content) {
@@ -1391,10 +1434,10 @@ static void status_content(UAContext *ua, STORERES *store)
          case slot_content_full:
             switch (vl1->Type) {
             case slot_type_normal:
-               content_send_info(ua, 'S', vl1->Slot, vl1->VolName);
+               content_send_info_api(ua, 'S', vl1->Slot, vl1->VolName);
                break;
             case slot_type_import:
-               content_send_info(ua, 'I', vl1->Slot, vl1->VolName);
+               content_send_info_api(ua, 'I', vl1->Slot, vl1->VolName);
                break;
             default:
                break;
@@ -1405,57 +1448,36 @@ static void status_content(UAContext *ua, STORERES *store)
              * See if this empty slot is empty because the volume is loaded
              * in one of the drives.
              */
-            found = false;
-            vl2 = (vol_list_t *)vol_list->first();
-            while (!found && vl2) {
-               switch (vl2->Type) {
-               case slot_type_drive:
-                  if (vl1->Slot == vl2->Loaded) {
-                     switch (vl1->Type) {
-                     case slot_type_normal:
-                        content_send_info(ua, 'S', vl1->Slot, vl2->VolName);
-                        break;
-                     case slot_type_import:
-                        content_send_info(ua, 'I', vl1->Slot, vl2->VolName);
-                        break;
-                     default:
-                        break;
-                     }
-                     found = true;
-
-                     /*
-                      * When we used this entry we can remove it from
-                      * the list so we limit the search time next round.
-                      */
-                     vol_list->remove(vl2);
-                     if (vl2->VolName) {
-                        free(vl2->VolName);
-                     }
-                     free(vl2);
-                     continue;
-                  }
-                  break;
-               default:
-                  break;
-               }
-               vl2 = (vol_list_t *)vol_list->next((void *)vl2);
-            }
-            if (!found) {
+            vl2 = vol_is_loaded_in_drive(store, vol_list, vl1->Slot);
+            if (vl2) {
                switch (vl1->Type) {
                case slot_type_normal:
-                  ua->send_msg(slot_api_slot_empty_format, 'S', vl1->Slot);
+                  content_send_info_api(ua, 'S', vl1->Slot, vl2->VolName);
                   break;
                case slot_type_import:
-                  ua->send_msg(slot_api_slot_empty_format, 'I', vl1->Slot);
+                  content_send_info_api(ua, 'I', vl1->Slot, vl2->VolName);
                   break;
                default:
                   break;
                }
+               continue;
+            }
+
+            switch (vl1->Type) {
+            case slot_type_normal:
+               ua->send_msg(slot_api_slot_empty_format, 'S', vl1->Slot);
+               break;
+            case slot_type_import:
+               ua->send_msg(slot_api_slot_empty_format, 'I', vl1->Slot);
+               break;
+            default:
+               break;
             }
             break;
          default:
             break;
          }
+         break;
       default:
          break;
       }
@@ -1463,7 +1485,116 @@ static void status_content(UAContext *ua, STORERES *store)
 
 bail_out:
    if (vol_list) {
-      free_vol_list(vol_list);
+      storage_release_vol_list(store, vol_list);
+   }
+   close_sd_bsock(ua);
+
+   return;
+}
+
+static void status_content_json(UAContext *ua, STORERES *store)
+{
+   vol_list_t *vl1, *vl2;
+   changer_vol_list_t *vol_list = NULL;
+
+   if (!open_client_db(ua)) {
+      return;
+   }
+
+   vol_list = get_vol_list_from_storage(ua, store, true /* listall */ , true /* want to see all slots */);
+   if (!vol_list) {
+      ua->warning_msg(_("No Volumes found, or no barcodes.\n"));
+      goto bail_out;
+   }
+
+   ua->send->array_start("contents");
+   foreach_dlist(vl1, vol_list->contents) {
+      switch (vl1->Type) {
+      case slot_type_drive:
+         ua->send->object_start();
+         ua->send->object_key_value("type", "drive", "%s\n");
+         ua->send->object_key_value("slotnr", vl1->Slot, "%hd\n");
+         switch (vl1->Content) {
+         case slot_content_full:
+            ua->send->object_key_value("content", "full", "%s\n");
+            ua->send->object_key_value("loaded", vl1->Loaded, "%hd\n");
+            ua->send->object_key_value("volname", vl1->VolName, "%s\n");
+            break;
+         case slot_content_empty:
+            ua->send->object_key_value("content", "empty", "%s\n");
+            break;
+         default:
+            break;
+         }
+         ua->send->object_end();
+         break;
+      case slot_type_normal:
+      case slot_type_import:
+         switch (vl1->Content) {
+         case slot_content_full:
+            switch (vl1->Type) {
+            case slot_type_normal:
+               content_send_info_json(ua, "slot", vl1->Slot, vl1->VolName);
+               break;
+            case slot_type_import:
+               content_send_info_json(ua, "import_slot", vl1->Slot, vl1->VolName);
+               break;
+            default:
+               break;
+            }
+            break;
+         case slot_content_empty:
+            /*
+             * See if this empty slot is empty because the volume is loaded
+             * in one of the drives.
+             */
+            vl2 = vol_is_loaded_in_drive(store, vol_list, vl1->Slot);
+            if (vl2) {
+               switch (vl1->Type) {
+               case slot_type_normal:
+                  content_send_info_json(ua, "slot", vl1->Slot, vl2->VolName);
+                  break;
+               case slot_type_import:
+                  content_send_info_json(ua, "import_slot", vl1->Slot, vl2->VolName);
+                  break;
+               default:
+                  break;
+               }
+               continue;
+            }
+
+            switch (vl1->Type) {
+            case slot_type_normal:
+               ua->send->object_start();
+               ua->send->object_key_value("type", "slot", "%s\n");
+               ua->send->object_key_value("slotnr", vl1->Slot, "%hd\n");
+               ua->send->object_key_value("content", "empty", "%s\n");
+               ua->send->object_end();
+               break;
+            case slot_type_import:
+               ua->send->object_start();
+               ua->send->object_key_value("type", "import_slot", "%s\n");
+               ua->send->object_key_value("slotnr", vl1->Slot, "%hd\n");
+               ua->send->object_key_value("content", "empty", "%s\n");
+               ua->send->object_end();
+               break;
+            default:
+               break;
+            }
+            break;
+         default:
+            break;
+         }
+         break;
+      default:
+         break;
+      }
+   }
+   ua->send->array_end("contents");
+
+bail_out:
+   if (vol_list) {
+      storage_release_vol_list(store, vol_list);
    }
    close_sd_bsock(ua);
 
@@ -1473,37 +1604,27 @@ bail_out:
 /*
  * Print slots from AutoChanger
  */
-static void status_slots(UAContext *ua, STORERES *store_r)
+static void status_slots(UAContext *ua, STORERES *store)
 {
-   USTORERES store;
    POOL_DBR pr;
-   vol_list_t *vl1, *vl2;
-   dlist *vol_list = NULL;
    MEDIA_DBR mr;
    char *slot_list;
-   int max_slots;
-   bool is_loaded_in_drive;
+   vol_list_t *vl1, *vl2;
+   slot_number_t max_slots;
+   changer_vol_list_t *vol_list = NULL;
+
    /*
     * Slot | Volume | Status | MediaType | Pool
     */
    const char *slot_hformat=" %4i%c| %16s | %9s | %14s | %24s |\n";
 
-   if (ua->api) {
-      status_content(ua, store_r);
-      return;
-   }
-
    if (!open_client_db(ua)) {
       return;
    }
 
    memset(&mr, 0, sizeof(mr));
 
-   store.store = store_r;
-   pm_strcpy(store.store_source, _("command line"));
-   set_wstorage(ua->jcr, &store);
-
-   max_slots = get_num_slots_from_SD(ua);
+   max_slots = get_num_slots(ua, store);
    if (max_slots <= 0) {
       ua->warning_msg(_("No slots in changer to scan.\n"));
       return;
@@ -1516,7 +1637,7 @@ static void status_slots(UAContext *ua, STORERES *store_r)
       return;
    }
 
-   vol_list = get_vol_list_from_SD(ua, store.store, true /* listall */ , true /* want to see all slots */);
+   vol_list = get_vol_list_from_storage(ua, store, true /* listall */ , true /* want to see all slots */);
    if (!vol_list) {
       ua->warning_msg(_("No Volumes found, or no barcodes.\n"));
       goto bail_out;
@@ -1528,7 +1649,8 @@ static void status_slots(UAContext *ua, STORERES *store_r)
     * Walk through the list getting the media records
     * Slots start numbering at 1.
     */
-   foreach_dlist(vl1, vol_list) {
+   foreach_dlist(vl1, vol_list->contents) {
+      vl2 = NULL;
       switch (vl1->Type) {
       case slot_type_drive:
          /*
@@ -1538,7 +1660,7 @@ static void status_slots(UAContext *ua, STORERES *store_r)
       case slot_type_normal:
       case slot_type_import:
          if (vl1->Slot > max_slots) {
-            ua->warning_msg(_("Slot %d greater than max %d ignored.\n"),
+            ua->warning_msg(_("Slot %hd greater than max %hd ignored.\n"),
                             vl1->Slot, max_slots);
             continue;
          }
@@ -1546,12 +1668,10 @@ static void status_slots(UAContext *ua, STORERES *store_r)
           * Check if user wants us to look at this slot
           */
          if (!bit_is_set(vl1->Slot - 1, slot_list)) {
-            Dmsg1(100, "Skipping slot=%d\n", vl1->Slot);
+            Dmsg1(100, "Skipping slot=%hd\n", vl1->Slot);
             continue;
          }
 
-         vl2 = NULL;
-         is_loaded_in_drive = false;
          switch (vl1->Content) {
          case slot_content_empty:
             if (vl1->Type == slot_type_normal) {
@@ -1559,21 +1679,8 @@ static void status_slots(UAContext *ua, STORERES *store_r)
                 * See if this empty slot is empty because the volume is loaded
                 * in one of the drives.
                 */
-               vl2 = (vol_list_t *)vol_list->first();
-               while (!is_loaded_in_drive && vl2) {
-                  switch (vl2->Type) {
-                  case slot_type_drive:
-                     if (vl1->Slot == vl2->Loaded) {
-                        is_loaded_in_drive = true;
-                        continue;
-                     }
-                     break;
-                  default:
-                     break;
-                  }
-                  vl2 = (vol_list_t *)vol_list->next((void *)vl2);
-               }
-               if (!is_loaded_in_drive) {
+               vl2 = vol_is_loaded_in_drive(store, vol_list, vl1->Slot);
+               if (!vl2) {
                   ua->send_msg(slot_hformat,
                                vl1->Slot, '*',
                                "?", "?", "?", "?");
@@ -1595,7 +1702,7 @@ static void status_slots(UAContext *ua, STORERES *store_r)
              */
             if (vl1->Content == slot_content_full) {
                if (!vl1->VolName) {
-                  Dmsg1(100, "No VolName for Slot=%d.\n", vl1->Slot);
+                  Dmsg1(100, "No VolName for Slot=%hd.\n", vl1->Slot);
                   ua->send_msg(slot_hformat,
                                vl1->Slot,
                               (vl1->Type == slot_type_import) ? '@' : '*',
@@ -1607,7 +1714,7 @@ static void status_slots(UAContext *ua, STORERES *store_r)
                bstrncpy(mr.VolumeName, vl1->VolName, sizeof(mr.VolumeName));
             } else {
                if (!vl2 || !vl2->VolName) {
-                  Dmsg1(100, "No VolName for Slot=%d.\n", vl1->Slot);
+                  Dmsg1(100, "No VolName for Slot=%hd.\n", vl1->Slot);
                   ua->send_msg(slot_hformat,
                                vl1->Slot,
                               (vl1->Type == slot_type_import) ? '@' : '*',
@@ -1636,7 +1743,7 @@ static void status_slots(UAContext *ua, STORERES *store_r)
                } else {
                   ua->send_msg(slot_hformat,
                                vl1->Slot,
-                               ((vl1->Slot == mr.Slot) ? (is_loaded_in_drive ? '%' : ' ') : '*'),
+                               ((vl1->Slot == mr.Slot) ? (vl2 ? '%' : ' ') : '*'),
                                mr.VolumeName, mr.VolStatus, mr.MediaType, pr.Name);
                }
             } else {
@@ -1657,7 +1764,7 @@ static void status_slots(UAContext *ua, STORERES *store_r)
 
 bail_out:
    if (vol_list) {
-      free_vol_list(vol_list);
+      storage_release_vol_list(store, vol_list);
    }
    free(slot_list);
    close_sd_bsock(ua);
