@@ -3,7 +3,7 @@
 
    Copyright (C) 2000-2010 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2014 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -33,6 +33,7 @@
 /* Imported Functions */
 extern void *handle_connection_request(void *dir_sock);
 extern bool parse_fd_config(CONFIG *config, const char *configfile, int exit_code);
+extern void prtmsg(void *sock, const char *fmt, ...);
 
 /* Forward referenced functions */
 static bool check_resources();
@@ -46,33 +47,30 @@ bool backup_only_mode = false;
 bool restore_only_mode = false;
 void *start_heap;
 
-#define CONFIG_FILE "bareos-fd.conf"  /* default config file */
-
 char *configfile = NULL;
 static bool foreground = false;
-static workq_t dir_workq;             /* queue of work from Director */
-static alist *sock_fds;
-static pthread_t tcp_server_tid;
 
 static void usage()
 {
    fprintf(stderr, _(
 PROG_COPYRIGHT
 "\nVersion: %s (%s)\n\n"
-"Usage: bareos-fd [-f -s] [-c config_file] [-d debug_level]\n"
+"Usage: bareos-fd [options]\n"
 "        -b          backup only mode\n"
-"        -c <file>   use <file> as configuration file\n"
+"        -c <path>   use <path> as configuration file or directory\n"
 "        -d <nn>     set debug level to <nn>\n"
-"        -dt         print a timestamp in debug output\n"
+"        -dt         print timestamp in debug output\n"
 "        -f          run in foreground (for debugging)\n"
-"        -g          groupid\n"
+"        -g <group>  run as group <group>\n"
 "        -k          keep readall capabilities\n"
 "        -m          print kaboom output (for debugging)\n"
 "        -r          restore only mode\n"
 "        -s          no signals (for debugging)\n"
 "        -t          test configuration file and exit\n"
-"        -u          userid\n"
+"        -u <user>   run as user <user>\n"
 "        -v          verbose user messages\n"
+"        -xc         print configuration and exit\n"
+"        -xs         print configuration file schema in JSON format and exit\n"
 "        -?          print this message.\n"
 "\n"), 2000, VERSION, BDATE);
 
@@ -93,6 +91,8 @@ int main (int argc, char *argv[])
 {
    int ch;
    bool test_config = false;
+   bool export_config = false;
+   bool export_config_schema = false;
    bool keep_readall_caps = false;
    char *uid = NULL;
    char *gid = NULL;
@@ -107,7 +107,7 @@ int main (int argc, char *argv[])
    init_msg(NULL, NULL);
    daemon_start_time = time(NULL);
 
-   while ((ch = getopt(argc, argv, "bc:d:fg:kmrstu:v?")) != -1) {
+   while ((ch = getopt(argc, argv, "bc:d:fg:kmrstu:vx:?")) != -1) {
       switch (ch) {
       case 'b':
          backup_only_mode = true;
@@ -167,10 +167,20 @@ int main (int argc, char *argv[])
          verbose++;
          break;
 
+      case 'x':                    /* export configuration/schema and exit */
+         if (*optarg == 's') {
+            export_config_schema = true;
+         } else if (*optarg == 'c') {
+            export_config = true;
+         } else {
+            usage();
+         }
+         break;
+
       case '?':
       default:
          usage();
-
+         break;
       }
    }
    argc -= optind;
@@ -198,28 +208,45 @@ int main (int argc, char *argv[])
       drop(uid, gid, keep_readall_caps);
    }
 
-   tcp_server_tid = pthread_self();
    if (!no_signals) {
       init_signals(terminate_filed);
    } else {
-      /* This reduces the number of signals facilitating debugging */
+      /*
+       * This reduces the number of signals facilitating debugging
+       */
       watchdog_sleep_time = 120;      /* long timeout for debugging */
    }
 
-   if (configfile == NULL) {
-      configfile = bstrdup(CONFIG_FILE);
+   if (export_config_schema) {
+      POOL_MEM buffer;
+
+      my_config = new_config_parser();
+      init_fd_config(my_config, configfile, M_ERROR_TERM);
+      print_config_schema_json(buffer);
+      printf("%s\n", buffer.c_str());
+      goto bail_out;
    }
 
    my_config = new_config_parser();
    parse_fd_config(my_config, configfile, M_ERROR_TERM);
 
+   if (export_config) {
+      my_config->dump_resources(prtmsg, NULL);
+      goto bail_out;
+   }
+
+   if (!foreground && !test_config) {
+      daemon_start();
+      init_stack_dump();              /* set new pid */
+   }
+
    if (init_crypto() != 0) {
       Emsg0(M_ERROR, 0, _("Cryptography library initialization failed.\n"));
       terminate_filed(1);
    }
 
    if (!check_resources()) {
-      Emsg1(M_ERROR, 0, _("Please correct configuration file: %s\n"), configfile);
+      Emsg1(M_ERROR, 0, _("Please correct configuration file: %s\n"), my_config->get_base_config_path());
       terminate_filed(1);
    }
 
@@ -237,12 +264,7 @@ int main (int argc, char *argv[])
       terminate_filed(0);
    }
 
-   if (!foreground) {
-      daemon_start();
-      init_stack_dump();              /* set new pid */
-   }
-
-   set_thread_concurrency(me->MaxConcurrentJobs + 10);
+   set_thread_concurrency(me->MaxConcurrentJobs * 2 + 10);
    lmgr_init_thread(); /* initialize the lockmanager stack */
 
    /* Maximum 1 daemon at a time */
@@ -259,25 +281,21 @@ int main (int argc, char *argv[])
          init_jcr_subsystem(me->jcr_watchdog_time); /* start JCR watchdogs etc. */
       }
    }
-   tcp_server_tid = pthread_self();
 
-   /* Become server, and handle requests */
-   IPADDR *p;
-   foreach_dlist(p, me->FDaddrs) {
-      Dmsg1(10, "filed: listening on port %d\n", p->get_port_host_order());
-   }
+   /*
+    * if configured, start threads and connect to Director.
+    */
+   start_connect_to_director_threads();
 
-   sock_fds = New(alist(10, not_owned_by_alist));
-   bnet_thread_server_tcp(me->FDaddrs,
-                          me->MaxConcurrentJobs,
-                          sock_fds,
-                          &dir_workq,
-                          me->nokeepalive,
-                          handle_connection_request);
+   /*
+    * start socket server to listen for new connections.
+    */
+   start_socket_server(me->FDaddrs);
 
    terminate_filed(0);
 
-   exit(0);                           /* should never get here */
+bail_out:
+   exit(0);
 }
 
 void terminate_filed(int sig)
@@ -292,10 +310,8 @@ void terminate_filed(int sig)
    debug_level = 0;                   /* turn off debug */
    stop_watchdog();
 
-   bnet_stop_thread_server_tcp(tcp_server_tid);
-   cleanup_bnet_thread_server_tcp(sock_fds, &dir_workq);
-   delete sock_fds;
-   sock_fds = NULL;
+   stop_connect_to_director_threads(true);
+   stop_socket_server(true);
 
    unload_fd_plugins();
    flush_mntent_cache();
@@ -318,7 +334,7 @@ void terminate_filed(int sig)
    cleanup_crypto();
    close_memory_pool();               /* release free memory in pool */
    lmgr_cleanup_main();
-   sm_dump(false);                    /* dump orphaned buffers */
+   sm_dump(false, false);             /* dump orphaned buffers */
    exit(sig);
 }
 
@@ -331,6 +347,7 @@ static bool check_resources()
    bool OK = true;
    DIRRES *director;
    bool need_tls;
+   const char *configfile = my_config->get_base_config_path();
 
    LockRes();
 
@@ -340,12 +357,19 @@ static bool check_resources()
             "Without that I don't know who I am :-(\n"), configfile);
       OK = false;
    } else {
+      /*
+       * Sanity check.
+       */
+      if (me->MaxConnections < (2 * me->MaxConcurrentJobs)) {
+         me->MaxConnections = (2 * me->MaxConcurrentJobs) + 2;
+      }
+
       if (GetNextRes(R_CLIENT, (RES *) me) != NULL) {
          Emsg1(M_FATAL, 0, _("Only one Client resource permitted in %s\n"),
               configfile);
          OK = false;
       }
-      my_name_is(0, NULL, me->hdr.name);
+      my_name_is(0, NULL, me->name());
       if (!me->messages) {
          me->messages = (MSGSRES *)GetNextRes(R_MSGS, NULL);
          if (!me->messages) {
@@ -354,17 +378,17 @@ static bool check_resources()
          }
       }
       /* tls_require implies tls_enable */
-      if (me->tls_require) {
+      if (me->tls.require) {
 #ifndef HAVE_TLS
          Jmsg(NULL, M_FATAL, 0, _("TLS required but not configured in Bareos.\n"));
          OK = false;
 #else
-         me->tls_enable = true;
+         me->tls.enable = true;
 #endif
       }
-      need_tls = me->tls_enable || me->tls_authenticate;
+      need_tls = me->tls.enable || me->tls.authenticate;
 
-      if ((!me->tls_ca_certfile && !me->tls_ca_certdir) && need_tls) {
+      if ((!me->tls.ca_certfile && !me->tls.ca_certdir) && need_tls) {
          Emsg1(M_FATAL, 0, _("Neither \"TLS CA Certificate\""
             " or \"TLS CA Certificate Dir\" are defined for File daemon in %s.\n"),
                             configfile);
@@ -372,28 +396,29 @@ static bool check_resources()
       }
 
       /* If everything is well, attempt to initialize our per-resource TLS context */
-      if (OK && (need_tls || me->tls_require)) {
+      if (OK && (need_tls || me->tls.require)) {
          /* Initialize TLS context:
           * Args: CA certfile, CA certdir, Certfile, Keyfile,
           * Keyfile PEM Callback, Keyfile CB Userdata, DHfile, Verify Peer */
-         me->tls_ctx = new_tls_context(me->tls_ca_certfile,
-                                       me->tls_ca_certdir,
-                                       me->tls_crlfile,
-                                       me->tls_certfile,
-                                       me->tls_keyfile,
+         me->tls.ctx = new_tls_context(me->tls.ca_certfile,
+                                       me->tls.ca_certdir,
+                                       me->tls.crlfile,
+                                       me->tls.certfile,
+                                       me->tls.keyfile,
                                        NULL,
                                        NULL,
                                        NULL,
-                                       me->tls_verify_peer);
+                                       me->tls.cipherlist,
+                                       me->tls.verify_peer);
 
-         if (!me->tls_ctx) {
+         if (!me->tls.ctx) {
             Emsg2(M_FATAL, 0, _("Failed to initialize TLS context for File daemon \"%s\" in %s.\n"),
-                                me->hdr.name, configfile);
+                                me->name(), configfile);
             OK = false;
          }
 
-         set_tls_enable(me->tls_ctx, need_tls);
-         set_tls_require(me->tls_ctx, me->tls_require);
+         set_tls_enable(me->tls.ctx, need_tls);
+         set_tls_require(me->tls.ctx, me->tls.require);
       }
 
       if (me->pki_encrypt || me->pki_sign) {
@@ -411,7 +436,7 @@ static bool check_resources()
       if ((me->pki_encrypt || me->pki_sign) && !me->pki_keypair_file) {
          Emsg2(M_FATAL, 0, _("\"PKI Key Pair\" must be defined for File"
             " daemon \"%s\" in %s if either \"PKI Sign\" or"
-            " \"PKI Encrypt\" are enabled.\n"), me->hdr.name, configfile);
+            " \"PKI Encrypt\" are enabled.\n"), me->name(), configfile);
          OK = false;
       }
 
@@ -426,13 +451,13 @@ static bool check_resources()
          } else {
             if (!crypto_keypair_load_cert(me->pki_keypair, me->pki_keypair_file)) {
                Emsg2(M_FATAL, 0, _("Failed to load public certificate for File"
-                     " daemon \"%s\" in %s.\n"), me->hdr.name, configfile);
+                     " daemon \"%s\" in %s.\n"), me->name(), configfile);
                OK = false;
             }
 
             if (!crypto_keypair_load_key(me->pki_keypair, me->pki_keypair_file, NULL, NULL)) {
                Emsg2(M_FATAL, 0, _("Failed to load private key for File"
-                     " daemon \"%s\" in %s.\n"), me->hdr.name, configfile);
+                     " daemon \"%s\" in %s.\n"), me->name(), configfile);
                OK = false;
             }
          }
@@ -462,14 +487,14 @@ static bool check_resources()
                      if (crypto_keypair_has_key(filepath)) {
                         if (!crypto_keypair_load_key(keypair, filepath, NULL, NULL)) {
                            Emsg3(M_FATAL, 0, _("Failed to load private key from file %s for File"
-                              " daemon \"%s\" in %s.\n"), filepath, me->hdr.name, configfile);
+                              " daemon \"%s\" in %s.\n"), filepath, me->name(), configfile);
                            OK = false;
                         }
                      }
 
                   } else {
                      Emsg3(M_FATAL, 0, _("Failed to load trusted signer certificate"
-                        " from file %s for File daemon \"%s\" in %s.\n"), filepath, me->hdr.name, configfile);
+                        " from file %s for File daemon \"%s\" in %s.\n"), filepath, me->name(), configfile);
                      OK = false;
                   }
                }
@@ -500,7 +525,7 @@ static bool check_resources()
                      me->pki_recipients->append(keypair);
                   } else {
                      Emsg3(M_FATAL, 0, _("Failed to load master key certificate"
-                        " from file %s for File daemon \"%s\" in %s.\n"), filepath, me->hdr.name, configfile);
+                        " from file %s for File daemon \"%s\" in %s.\n"), filepath, me->name(), configfile);
                      OK = false;
                   }
                }
@@ -522,61 +547,62 @@ static bool check_resources()
 
    foreach_res(director, R_DIRECTOR) {
       /* tls_require implies tls_enable */
-      if (director->tls_require) {
+      if (director->tls.require) {
 #ifndef HAVE_TLS
          Jmsg(NULL, M_FATAL, 0, _("TLS required but not configured in Bareos.\n"));
          OK = false;
          continue;
 #else
-         director->tls_enable = true;
+         director->tls.enable = true;
 #endif
       }
-      need_tls = director->tls_enable || director->tls_authenticate;
+      need_tls = director->tls.enable || director->tls.authenticate;
 
-      if (!director->tls_certfile && need_tls) {
+      if (!director->tls.certfile && need_tls) {
          Emsg2(M_FATAL, 0, _("\"TLS Certificate\" file not defined for Director \"%s\" in %s.\n"),
-               director->hdr.name, configfile);
+               director->name(), configfile);
          OK = false;
       }
 
-      if (!director->tls_keyfile && need_tls) {
+      if (!director->tls.keyfile && need_tls) {
          Emsg2(M_FATAL, 0, _("\"TLS Key\" file not defined for Director \"%s\" in %s.\n"),
-               director->hdr.name, configfile);
+               director->name(), configfile);
          OK = false;
       }
 
-      if ((!director->tls_ca_certfile && !director->tls_ca_certdir) && need_tls && director->tls_verify_peer) {
+      if ((!director->tls.ca_certfile && !director->tls.ca_certdir) && need_tls && director->tls.verify_peer) {
          Emsg2(M_FATAL, 0, _("Neither \"TLS CA Certificate\""
                              " or \"TLS CA Certificate Dir\" are defined for Director \"%s\" in %s."
                              " At least one CA certificate store is required"
                              " when using \"TLS Verify Peer\".\n"),
-                             director->hdr.name, configfile);
+                             director->name(), configfile);
          OK = false;
       }
 
       /* If everything is well, attempt to initialize our per-resource TLS context */
-      if (OK && (need_tls || director->tls_require)) {
+      if (OK && (need_tls || director->tls.require)) {
          /* Initialize TLS context:
           * Args: CA certfile, CA certdir, Certfile, Keyfile,
           * Keyfile PEM Callback, Keyfile CB Userdata, DHfile, Verify Peer */
-         director->tls_ctx = new_tls_context(director->tls_ca_certfile,
-                                             director->tls_ca_certdir,
-                                             director->tls_crlfile,
-                                             director->tls_certfile,
-                                             director->tls_keyfile,
+         director->tls.ctx = new_tls_context(director->tls.ca_certfile,
+                                             director->tls.ca_certdir,
+                                             director->tls.crlfile,
+                                             director->tls.certfile,
+                                             director->tls.keyfile,
                                              NULL,
                                              NULL,
-                                             director->tls_dhfile,
-                                             director->tls_verify_peer);
+                                             director->tls.dhfile,
+                                             director->tls.cipherlist,
+                                             director->tls.verify_peer);
 
-         if (!director->tls_ctx) {
+         if (!director->tls.ctx) {
             Emsg2(M_FATAL, 0, _("Failed to initialize TLS context for Director \"%s\" in %s.\n"),
-                                director->hdr.name, configfile);
+                                director->name(), configfile);
             OK = false;
          }
 
-         set_tls_enable(director->tls_ctx, need_tls);
-         set_tls_require(director->tls_ctx, director->tls_require);
+         set_tls_enable(director->tls.ctx, need_tls);
+         set_tls_require(director->tls.ctx, director->tls.require);
       }
    }
 
@@ -585,6 +611,12 @@ static bool check_resources()
    if (OK) {
       close_msg(NULL);                /* close temp message handler */
       init_msg(NULL, me->messages);   /* open user specified message handler */
+      if (me->secure_erase_cmdline) {
+         set_secure_erase_cmdline(me->secure_erase_cmdline);
+      }
+      if (me->log_timestamp_format) {
+         set_log_timestamp_format(me->log_timestamp_format);
+      }
    }
 
    return OK;
