@@ -29,18 +29,18 @@
 
 /*
  * Various BAREOS Utility subroutines
- *
  */
 
 /*
- *  Escape special characters in bareos configuration strings
- *  needed for dumping config strings
+ * Escape special characters in bareos configuration strings
+ * needed for dumping config strings
  */
-void escape_string(char *snew, char *old, int len)
+void escape_string(POOL_MEM &snew, char *old, int len)
 {
    char *n, *o;
 
-   n = snew;
+   snew.check_size(len * 2);
+   n = snew.c_str();
    o = old;
    while (len--) {
       switch (*o) {
@@ -49,6 +49,11 @@ void escape_string(char *snew, char *old, int len)
          *n++ = '\'';
          o++;
          break;
+      case '\\':
+         *n++ = '\\';
+         *n++ = '\\';
+         o++;
+         break;
       case 0:
          *n++ = '\\';
          *n++ = 0;
@@ -70,7 +75,9 @@ void escape_string(char *snew, char *old, int len)
    *n = 0;
 }
 
-/* Return true of buffer has all zero bytes */
+/*
+ * Return true of buffer has all zero bytes
+ */
 bool is_buf_zero(char *buf, int len)
 {
    uint64_t *ip;
@@ -81,7 +88,10 @@ bool is_buf_zero(char *buf, int len)
       return false;
    }
    ip = (uint64_t *)buf;
-   /* Optimize by checking uint64_t for zero */
+
+   /*
+    * Optimize by checking uint64_t for zero
+    */
    len64 = len / sizeof(uint64_t);
    for (i=0; i < len64; i++) {
       if (ip[i] != 0) {
@@ -100,7 +110,9 @@ bool is_buf_zero(char *buf, int len)
 }
 
 
-/* Convert a string in place to lower case */
+/*
+ * Convert a string in place to lower case
+ */
 void lcase(char *str)
 {
    while (*str) {
@@ -111,11 +123,11 @@ void lcase(char *str)
    }
 }
 
-/* Convert spaces to non-space character.
+/*
+ * Convert spaces to non-space character.
  * This makes scanf of fields containing spaces easier.
  */
-void
-bash_spaces(char *str)
+void bash_spaces(char *str)
 {
    while (*str) {
       if (*str == ' ')
@@ -124,11 +136,11 @@ bash_spaces(char *str)
    }
 }
 
-/* Convert spaces to non-space character.
+/*
+ * Convert spaces to non-space character.
  * This makes scanf of fields containing spaces easier.
  */
-void
-bash_spaces(POOL_MEM &pm)
+void bash_spaces(POOL_MEM &pm)
 {
    char *str = pm.c_str();
    while (*str) {
@@ -139,9 +151,10 @@ bash_spaces(POOL_MEM &pm)
 }
 
 
-/* Convert non-space characters (0x1) back into spaces */
-void
-unbash_spaces(char *str)
+/*
+ * Convert non-space characters (0x1) back into spaces
+ */
+void unbash_spaces(char *str)
 {
    while (*str) {
      if (*str == 0x1)
@@ -150,9 +163,10 @@ unbash_spaces(char *str)
    }
 }
 
-/* Convert non-space characters (0x1) back into spaces */
-void
-unbash_spaces(POOL_MEM &pm)
+/*
+ * Convert non-space characters (0x1) back into spaces
+ */
+void unbash_spaces(POOL_MEM &pm)
 {
    char *str = pm.c_str();
    while (*str) {
@@ -193,7 +207,17 @@ char *encode_time(utime_t utime, char *buf)
    return buf+n;
 }
 
+bool convert_timeout_to_timespec(timespec &timeout, int timeout_in_seconds)
+{
+   struct timeval tv;
+   struct timezone tz;
 
+   gettimeofday(&tv, &tz);
+   timeout.tv_nsec = tv.tv_usec * 1000;
+   timeout.tv_sec = tv.tv_sec + timeout_in_seconds;
+
+   return true;
+}
 
 /*
  * Convert a JobStatus code into a human readable form
@@ -340,7 +364,6 @@ void jobstatus_to_ascii_gui(int JobStatus, char *msg, int maxlen)
    }
 }
 
-
 /*
  * Convert Job Termination Status into a string
  */
@@ -375,7 +398,6 @@ const char *job_status_to_str(int stat)
    return str;
 }
 
-
 /*
  * Convert Job Type into a string
  */
@@ -420,6 +442,9 @@ const char *job_type_to_str(int type)
    case JT_SCAN:
       str = _("Scan");
       break;
+   case JT_CONSOLIDATE:
+      str = _("Consolidate");
+      break;
    }
    if (!str) {
       str = _("Unknown Type");
@@ -427,7 +452,8 @@ const char *job_type_to_str(int type)
    return str;
 }
 
-/* Convert ActionOnPurge to string (Truncate, Erase, Destroy)
+/*
+ * Convert ActionOnPurge to string (Truncate, Erase, Destroy)
  */
 char *action_on_purge_to_string(int aop, POOL_MEM &ret)
 {
@@ -519,10 +545,9 @@ const char *volume_status_to_str(const char *status)
 }
 
 
-/***********************************************************************
+/*
  * Encode the mode bits into a 10 character string like LS does
- ***********************************************************************/
-
+ */
 char *encode_mode(mode_t mode, char *buf)
 {
   char *cp = buf;
@@ -570,7 +595,9 @@ int do_shell_expansion(char *name, int name_len)
    BPIPE *bpipe;
    const char *shellcmd;
 
-   /* Check if any meta characters are present */
+   /*
+    * Check if any meta characters are present
+    */
    len = strlen(meta);
    for (i = 0; i < len; i++) {
       if (strchr(name, meta[i])) {
@@ -581,14 +608,16 @@ int do_shell_expansion(char *name, int name_len)
    if (found) {
       cmd = get_pool_memory(PM_FNAME);
       line = get_pool_memory(PM_FNAME);
-      /* look for shell */
+      /*
+       * Look for shell
+       */
       if ((shellcmd = getenv("SHELL")) == NULL) {
          shellcmd = "/bin/sh";
       }
-      pm_strcpy(&cmd, shellcmd);
-      pm_strcat(&cmd, " -c \"echo ");
-      pm_strcat(&cmd, name);
-      pm_strcat(&cmd, "\"");
+      pm_strcpy(cmd, shellcmd);
+      pm_strcat(cmd, " -c \"echo ");
+      pm_strcat(cmd, name);
+      pm_strcat(cmd, "\"");
       Dmsg1(400, "Send: %s\n", cmd);
       if ((bpipe = open_bpipe(cmd, 0, "r"))) {
          bfgets(line, bpipe->rfd);
@@ -608,14 +637,14 @@ int do_shell_expansion(char *name, int name_len)
 }
 #endif
 
-
-/*  MAKESESSIONKEY  --  Generate session key with optional start
-                        key.  If mode is TRUE, the key will be
-                        translated to a string, otherwise it is
-                        returned as 16 binary bytes.
-
-    from SpeakFreely by John Walker */
-
+/*
+ * MAKESESSIONKEY  --  Generate session key with optional start
+ *                     key.  If mode is TRUE, the key will be
+ *                     translated to a string, otherwise it is
+ *                     returned as 16 binary bytes.
+ *
+ *  from SpeakFreely by John Walker
+ */
 void make_session_key(char *key, char *seed, int mode)
 {
    int j, k;
@@ -630,13 +659,14 @@ void make_session_key(char *key, char *seed, int mode)
      bstrncat(s, seed, sizeof(s));
    }
 
-   /* The following creates a seed for the session key generator
-     based on a collection of volatile and environment-specific
-     information unlikely to be vulnerable (as a whole) to an
-     exhaustive search attack.  If one of these items isn't
-     available on your machine, replace it with something
-     equivalent or, if you like, just delete it. */
-
+   /*
+    * The following creates a seed for the session key generator
+    * based on a collection of volatile and environment-specific
+    * information unlikely to be vulnerable (as a whole) to an
+    * exhaustive search attack.  If one of these items isn't
+    * available on your machine, replace it with something
+    * equivalent or, if you like, just delete it.
+    */
 #if defined(HAVE_WIN32)
    {
       LARGE_INTEGER     li;
@@ -749,9 +779,9 @@ void decode_session_key(char *decode, char *session, char *key, int maxlen)
  *  %j = Unique Job id
  *  %l = job level
  *  %n = Unadorned Job name
+ *  %r = Recipients
  *  %s = Since time
  *  %t = Job type (Backup, ...)
- *  %r = Recipients
  *  %v = Volume name(s)
  *
  *  omsg = edited output message
@@ -762,6 +792,7 @@ POOLMEM *edit_job_codes(JCR *jcr, char *omsg, char *imsg, const char *to, job_co
 {
    char *p, *q;
    const char *str;
+   char ed1[50];
    char add[50];
    char name[MAX_NAME_LENGTH];
    int i;
@@ -775,58 +806,59 @@ POOLMEM *edit_job_codes(JCR *jcr, char *omsg, char *imsg, const char *to, job_co
             str = "%";
             break;
          case 'B':                    /* Job Bytes in human readable format */
-            str = edit_uint64_with_suffix(jcr->JobBytes, add);
+            bsnprintf(add, sizeof(add), "%sB", edit_uint64_with_suffix(jcr->JobBytes, ed1));
+            str = add;
             break;
          case 'F':                    /* Job Files */
             str = edit_uint64(jcr->JobFiles, add);
             break;
-         case 'P':
+         case 'P':                    /* Process Id */
             bsnprintf(add, sizeof(add), "%lu", (uint32_t)getpid());
             str = add;
             break;
          case 'b':                    /* Job Bytes */
             str = edit_uint64(jcr->JobBytes, add);
             break;
-         case 'c':
+         case 'c':                    /* Client's name */
             if (jcr) {
                str = jcr->client_name;
             } else {
-               str = _("*none*");
+               str = _("*None*");
             }
             break;
-         case 'd':
-            str = my_name;            /* Director's name */
+         case 'd':                    /* Director's name */
+            str = my_name;
             break;
-         case 'e':
+         case 'e':                    /* Job Exit code */
             if (jcr) {
                str = job_status_to_str(jcr->JobStatus);
             } else {
-               str = _("*none*");
+               str = _("*None*");
             }
             break;
-         case 'i':
+         case 'i':                    /* JobId */
             if (jcr) {
                bsnprintf(add, sizeof(add), "%d", jcr->JobId);
                str = add;
             } else {
-               str = _("*none*");
+               str = _("*None*");
             }
             break;
          case 'j':                    /* Job name */
             if (jcr) {
                str = jcr->Job;
             } else {
-               str = _("*none*");
+               str = _("*None*");
             }
             break;
-         case 'l':
+         case 'l':                    /* Job level */
             if (jcr) {
                str = job_level_to_str(jcr->getJobLevel());
             } else {
-               str = _("*none*");
+               str = _("*None*");
             }
             break;
-         case 'n':
+         case 'n':                    /* Unadorned Job name */
              if (jcr) {
                 bstrncpy(name, jcr->Job, sizeof(name));
                 /*
@@ -839,35 +871,35 @@ POOLMEM *edit_job_codes(JCR *jcr, char *omsg, char *imsg, const char *to, job_co
                 }
                 str = name;
              } else {
-                str = _("*none*");
+                str = _("*None*");
              }
              break;
-         case 'r':
+         case 'r':                    /* Recipients */
             str = to;
             break;
          case 's':                    /* Since time */
             if (jcr && jcr->stime) {
                str = jcr->stime;
             } else {
-               str = _("*none*");
+               str = _("*None*");
             }
             break;
-         case 't':
+         case 't':                    /* Job type */
             if (jcr) {
                str = job_type_to_str(jcr->getJobType());
             } else {
-               str = _("*none*");
+               str = _("*None*");
             }
             break;
-         case 'v':
+         case 'v':                    /* Volume name(s) */
             if (jcr) {
                if (jcr->VolumeName) {
                   str = jcr->VolumeName;
                } else {
-                  str = _("*none*");
+                  str = _("*None*");
                }
             } else {
-               str = _("*none*");
+               str = _("*None*");
             }
             break;
          default:
@@ -890,9 +922,10 @@ POOLMEM *edit_job_codes(JCR *jcr, char *omsg, char *imsg, const char *to, job_co
          str = add;
       }
       Dmsg1(1200, "add_str %s\n", str);
-      pm_strcat(&omsg, str);
+      pm_strcat(omsg, str);
       Dmsg1(1200, "omsg=%s\n", omsg);
    }
+
    return omsg;
 }
 
