@@ -1,6 +1,7 @@
 /*
    BAREOS® - Backup Archiving REcovery Open Sourced
 
+   Copyright (C) 2013-2014 Planets Communications B.V.
    Copyright (C) 2013-2014 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
@@ -285,19 +286,19 @@ static bRC handleJobEnd(bpContext *ctx)
    }
 
    if (p_ctx->inflate_bytes_in) {
-      Dmsg(ctx, dbglvl, "autoxflate-sd.c: inflate ratio: %lld/%lld = %0.2f%%\n" ,
+      Dmsg(ctx, dbglvl, "autoxflate-sd: inflate ratio: %lld/%lld = %0.2f%%\n" ,
            p_ctx->inflate_bytes_out, p_ctx->inflate_bytes_in,
           (p_ctx->inflate_bytes_out * 100.0 / p_ctx->inflate_bytes_in));
-      Jmsg(ctx, M_INFO, _("autoxflate-sd.c: inflate ratio: %0.2f%%\n"),
+      Jmsg(ctx, M_INFO, _("autoxflate-sd: inflate ratio: %0.2f%%\n"),
            (p_ctx->inflate_bytes_out * 100.0 / p_ctx->inflate_bytes_in));
    }
 
    if (p_ctx->deflate_bytes_in) {
-      Dmsg(ctx, dbglvl, "autoxflate-sd.c: deflate ratio: %lld/%lld =  %0.2f%%\n",
+      Dmsg(ctx, dbglvl, "autoxflate-sd: deflate ratio: %lld/%lld =  %0.2f%%\n",
            p_ctx->deflate_bytes_out,
            p_ctx->deflate_bytes_in,
           (p_ctx->deflate_bytes_out * 100.0 / p_ctx->deflate_bytes_in));
-      Jmsg(ctx, M_INFO, _("autoxflate-sd.c: deflate ratio: %0.2f%%\n"),
+      Jmsg(ctx, M_INFO, _("autoxflate-sd: deflate ratio: %0.2f%%\n"),
            (p_ctx->deflate_bytes_out * 100.0 / p_ctx->deflate_bytes_in));
    }
 
@@ -343,7 +344,7 @@ static bRC setup_record_translation(bpContext *ctx, void *value)
       deflate_out = SETTING_YES;
       break;
    default:
-      Jmsg(ctx, M_ERROR, _("Unexpected autodeflate setting on %s"), dcr->dev_name);
+      Jmsg(ctx, M_ERROR, _("autoxflate-sd: Unexpected autodeflate setting on %s"), dcr->dev_name);
       break;
    }
 
@@ -365,7 +366,7 @@ static bRC setup_record_translation(bpContext *ctx, void *value)
       inflate_out = SETTING_YES;
       break;
    default:
-      Jmsg(ctx, M_ERROR, _("Unexpected autoinflate setting on %s"), dcr->dev_name);
+      Jmsg(ctx, M_ERROR, _("autoxflate-sd: Unexpected autoinflate setting on %s"), dcr->dev_name);
       break;
    }
 
@@ -402,7 +403,7 @@ static bRC setup_record_translation(bpContext *ctx, void *value)
 
    if (did_setup) {
       Jmsg(ctx, M_INFO,
-            _("autoxflate-sd.c: %s OUT:[SD->inflate=%s->deflate=%s->DEV] IN:[DEV->inflate=%s->deflate=%s->SD]\n"),
+            _("autoxflate-sd: %s OUT:[SD->inflate=%s->deflate=%s->DEV] IN:[DEV->inflate=%s->deflate=%s->SD]\n"),
            dcr->dev_name, inflate_out, deflate_out, inflate_in, deflate_in);
    }
 
@@ -529,7 +530,7 @@ static bool setup_auto_deflation(bpContext *ctx, DCR *dcr)
 
       pZlibStream = (z_stream *)jcr->compress.workset.pZLIB;
       if ((zstat = deflateParams(pZlibStream, dcr->device->autodeflate_level, Z_DEFAULT_STRATEGY)) != Z_OK) {
-         Jmsg(ctx, M_FATAL, _("Compression deflateParams error: %d\n"), zstat);
+         Jmsg(ctx, M_FATAL, _("autoxflate-sd: Compression deflateParams error: %d\n"), zstat);
          jcr->setJobStatus(JS_ErrorTerminated);
          goto bail_out;
       }
@@ -561,7 +562,7 @@ static bool setup_auto_deflation(bpContext *ctx, DCR *dcr)
 
       pZfastStream = (zfast_stream *)jcr->compress.workset.pZFAST;
       if ((zstat = fastlzlibSetCompressor(pZfastStream, compressor)) != Z_OK) {
-         Jmsg(ctx, M_FATAL, _("Compression fastlzlibSetCompressor error: %d\n"), zstat);
+         Jmsg(ctx, M_FATAL, _("autoxflate-sd: Compression fastlzlibSetCompressor error: %d\n"), zstat);
          jcr->setJobStatus(JS_ErrorTerminated);
          goto bail_out;
       }
@@ -572,7 +573,7 @@ static bool setup_auto_deflation(bpContext *ctx, DCR *dcr)
       break;
    }
 
-   Jmsg(ctx, M_INFO,  _("autodeflation: Compressor on device %s is %s\n"), dcr->dev_name, compressorname);
+   Jmsg(ctx, M_INFO,  _("autoxflate-sd: Compressor on device %s is %s\n"), dcr->dev_name, compressorname);
    retval = true;
 
 bail_out:
@@ -633,7 +634,7 @@ static bool auto_deflate_record(bpContext *ctx, DCR *dcr)
 
    /*
     * See what our starting point is. When dcr->after_rec is set we already have
-    * a translated record by an other SD plugin. Then we use that translated record
+    * a translated record by another SD plugin. Then we use that translated record
     * as the starting point otherwise we start at dcr->before_rec. When an earlier
     * translation already happened we can free that record when we have a success
     * full translation here as that record is of no use anymore.
@@ -785,7 +786,7 @@ static bool auto_inflate_record(bpContext *ctx, DCR *dcr)
 
    /*
     * See what our starting point is. When dcr->after_rec is set we already have
-    * a translated record by an other SD plugin. Then we use that translated record
+    * a translated record by another SD plugin. Then we use that translated record
     * as the starting point otherwise we start at dcr->before_rec. When an earlier
     * translation already happened we can free that record when we have a success
     * full translation here as that record is of no use anymore.
