@@ -1,7 +1,7 @@
 /*
    BAREOS® - Backup Archiving REcovery Open Sourced
 
-   Copyright (C) 2011-2012 Planets Communications B.V.
+   Copyright (C) 2011-2014 Planets Communications B.V.
    Copyright (C) 2013-2014 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
@@ -52,7 +52,7 @@ static bRC freePlugin(bpContext *ctx);
 static bRC getPluginValue(bpContext *ctx, psdVariable var, void *value);
 static bRC setPluginValue(bpContext *ctx, psdVariable var, void *value);
 static bRC handlePluginEvent(bpContext *ctx, bsdEvent *event, void *value);
-static bRC parse_plugin_definition(bpContext *ctx, void *value);
+static bRC parse_plugin_definition(bpContext *ctx, void *value, POOL_MEM &plugin_options);
 
 static void PyErrorHandler(bpContext *ctx, int msgtype);
 static bRC PyLoadModule(bpContext *ctx, void *value);
@@ -93,14 +93,16 @@ static psdFuncs pluginFuncs = {
  * Plugin private context
  */
 struct plugin_ctx {
-   PyThreadState *interpreter;
-   int64_t instance;
-   bool python_loaded;
-   char *module_path;
-   char *module_name;
-   PyObject *pModule;
-   PyObject *pDict;
-   PyObject *bpContext;
+   int64_t instance;                  /* Instance number of plugin */
+   bool python_loaded;                /* Plugin has python module loaded ? */
+   bool python_path_set;              /* Python plugin search path is set ? */
+   char *module_path;                 /* Plugin Module Path */
+   char *module_name;                 /* Plugin Module Name */
+   PyThreadState *interpreter;        /* Python interpreter for this instance of the plugin */
+   PyObject *pInstance;               /* Python Module instance */
+   PyObject *pModule;                 /* Python Module entry point */
+   PyObject *pDict;                   /* Python Dictionary */
+   PyObject *bpContext;               /* Python representation of plugin context */
 };
 
 #include "python-sd.h"
@@ -270,9 +272,10 @@ static bRC setPluginValue(bpContext *ctx, psdVariable var, void *value)
  */
 static bRC handlePluginEvent(bpContext *ctx, bsdEvent *event, void *value)
 {
-   bool event_dispatched = false;
-   plugin_ctx *p_ctx = (plugin_ctx *)ctx->pContext;
    bRC retval = bRC_Error;
+   bool event_dispatched = false;
+   POOL_MEM plugin_options(PM_FNAME);
+   plugin_ctx *p_ctx = (plugin_ctx *)ctx->pContext;
 
    if (!p_ctx) {
       goto bail_out;
@@ -285,7 +288,7 @@ static bRC handlePluginEvent(bpContext *ctx, bsdEvent *event, void *value)
    switch (event->eventType) {
    case bsdEventNewPluginOptions:
       event_dispatched = true;
-      retval = parse_plugin_definition(ctx, value);
+      retval = parse_plugin_definition(ctx, value, plugin_options);
       break;
    default:
       break;
@@ -310,14 +313,14 @@ static bRC handlePluginEvent(bpContext *ctx, bsdEvent *event, void *value)
           * See if we already loaded the Python modules.
           */
          if (!p_ctx->python_loaded) {
-            retval = PyLoadModule(ctx, value);
+            retval = PyLoadModule(ctx, plugin_options.c_str());
          }
 
          /*
           * Only try to call when the loading succeeded.
           */
          if (retval == bRC_OK) {
-            retval = PyParsePluginDefinition(ctx, value);
+            retval = PyParsePluginDefinition(ctx, plugin_options.c_str());
          }
          break;
       default:
@@ -403,10 +406,12 @@ static inline void set_string(char **destination, char *value)
  *
  * python:module_path=<path>:module_name=<python_module_name>:...
  */
-static bRC parse_plugin_definition(bpContext *ctx, void *value)
+static bRC parse_plugin_definition(bpContext *ctx, void *value, POOL_MEM &plugin_options)
 {
-   int i;
-   char *plugin_definition, *bp, *argument, *argument_value;
+   bool found;
+   int i, cnt;
+   POOL_MEM plugin_definition(PM_FNAME);
+   char *bp, *argument, *argument_value;
    plugin_ctx *p_ctx = (plugin_ctx *)ctx->pContext;
 
    if (!value) {
@@ -417,12 +422,12 @@ static bRC parse_plugin_definition(bpContext *ctx, void *value)
     * Parse the plugin definition.
     * Make a private copy of the whole string.
     */
-   plugin_definition = bstrdup((char *)value);
+   pm_strcpy(plugin_definition, (char *)value);
 
-   bp = strchr(plugin_definition, ':');
+   bp = strchr(plugin_definition.c_str(), ':');
    if (!bp) {
-      Jmsg(ctx, M_FATAL, "Illegal plugin definition %s\n", plugin_definition);
-      Dmsg(ctx, dbglvl, "Illegal plugin definition %s\n", plugin_definition);
+      Jmsg(ctx, M_FATAL, "python-sd: Illegal plugin definition %s\n", plugin_definition.c_str());
+      Dmsg(ctx, dbglvl, "python-sd: Illegal plugin definition %s\n", plugin_definition.c_str());
       goto bail_out;
    }
 
@@ -430,6 +435,8 @@ static bRC parse_plugin_definition(bpContext *ctx, void *value)
     * Skip the first ':'
     */
    bp++;
+
+   cnt = 0;
    while (bp) {
       if (strlen(bp) == 0) {
          break;
@@ -445,8 +452,8 @@ static bRC parse_plugin_definition(bpContext *ctx, void *value)
       argument = bp;
       argument_value = strchr(bp, '=');
       if (!argument_value) {
-         Jmsg(ctx, M_FATAL, "Illegal argument %s without value\n", argument);
-         Dmsg(ctx, dbglvl, "Illegal argument %s without value\n", argument);
+         Jmsg(ctx, M_FATAL, "python-sd: Illegal argument %s without value\n", argument);
+         Dmsg(ctx, dbglvl, "python-sd: Illegal argument %s without value\n", argument);
          goto bail_out;
       }
       *argument_value++ = '\0';
@@ -467,6 +474,7 @@ static bRC parse_plugin_definition(bpContext *ctx, void *value)
          }
       } while (bp);
 
+      found = false;
       for (i = 0; plugin_arguments[i].name; i++) {
          if (bstrcasecmp(argument, plugin_arguments[i].name)) {
             int64_t *int_destination = NULL;
@@ -502,18 +510,38 @@ static bRC parse_plugin_definition(bpContext *ctx, void *value)
             /*
              * When we have a match break the loop.
              */
+            found = true;
             break;
          }
       }
+
+      /*
+       * If we didn't consume this parameter we add it to the plugin_options list.
+       */
+      if (!found) {
+         POOL_MEM option(PM_FNAME);
+
+         if (cnt) {
+            Mmsg(option, ":%s=%s", argument, argument_value);
+            pm_strcat(plugin_options, option.c_str());
+         } else {
+            Mmsg(option, "%s=%s", argument, argument_value);
+            pm_strcat(plugin_options, option.c_str());
+         }
+         cnt++;
+      }
+   }
+
+   if (cnt > 0) {
+      pm_strcat(plugin_options, ":");
    }
 
-   free(plugin_definition);
    return bRC_OK;
 
 bail_out:
-   free(plugin_definition);
    return bRC_Error;
 }
+
 /*
  * Work around API changes in Python versions.
  * These function abstract the storage and retrieval of the bpContext
@@ -521,9 +549,9 @@ bail_out:
  * back and which allow the callback function to understand what bpContext
  * its talking about.
  */
-#if ((PY_VERSION_HEX <  0x02070000) || \
+#if ((PY_VERSION_HEX < 0x02070000) || \
      ((PY_VERSION_HEX >= 0x03000000) && \
-      (PY_VERSION_HEX <  0x03010000)))
+      (PY_VERSION_HEX < 0x03010000)))
 /*
  * Python version before 2.7 and 3.0.
  */
@@ -562,10 +590,15 @@ static bpContext *PyGetbpContext(PyObject *pyCtx)
  */
 static inline bRC conv_python_retval(PyObject *pRetVal)
 {
-   int retval;
+   return (bRC)PyInt_AsLong(pRetVal);
+}
 
-   retval = PyInt_AsLong(pRetVal);
-   return (bRC)retval;
+/*
+ * Convert a return value from bRC enum value into Python Object.
+ */
+static inline PyObject *conv_retval_python(bRC retval)
+{
+   return (PyObject *)PyInt_FromLong((int)retval);
 }
 
 /*
@@ -615,9 +648,9 @@ static void PyErrorHandler(bpContext *ctx, int msgtype)
    Py_XDECREF(value);
    Py_XDECREF(traceback);
 
-   Dmsg(ctx, dbglvl, "%s\n", error_string);
+   Dmsg(ctx, dbglvl, "python-sd: %s\n", error_string);
    if (msgtype) {
-      Jmsg(ctx, msgtype, "%s\n", error_string);
+      Jmsg(ctx, msgtype, "python-sd: %s\n", error_string);
    }
 
    free(error_string);
@@ -637,39 +670,49 @@ static bRC PyLoadModule(bpContext *ctx, void *value)
    PyObject *sysPath,
             *mPath,
             *pName,
-            *pFunc,
-            *module;
+            *pFunc;
 
    /*
-    * Extend the Python search path with the given module_path.
+    * See if we already setup the python search path.
     */
-   if (p_ctx->module_path) {
-      sysPath = PySys_GetObject((char *)"path");
-      mPath = PyString_FromString(p_ctx->module_path);
-      PyList_Append(sysPath, mPath);
-      Py_DECREF(mPath);
+   if (!p_ctx->python_path_set) {
+      /*
+       * Extend the Python search path with the given module_path.
+       */
+      if (p_ctx->module_path) {
+         sysPath = PySys_GetObject((char *)"path");
+         mPath = PyString_FromString(p_ctx->module_path);
+         PyList_Append(sysPath, mPath);
+         Py_DECREF(mPath);
+         p_ctx->python_path_set = true;
+      }
    }
 
    /*
-    * Make our callback methods available for Python.
+    * See if we already setup the module structure.
     */
-   module = Py_InitModule("bareossd", BareosSDMethods);
+   if (!p_ctx->pInstance) {
+      /*
+       * Make our callback methods available for Python.
+       */
+      p_ctx->pInstance = Py_InitModule("bareossd", BareosSDMethods);
+   }
 
    /*
     * Try to load the Python module by name.
     */
    if (p_ctx->module_name) {
-      Dmsg(ctx, dbglvl, "Trying to load module with name %s\n", p_ctx->module_name);
+      Dmsg(ctx, dbglvl, "python-sd: Trying to load module with name %s\n", p_ctx->module_name);
       pName = PyString_FromString(p_ctx->module_name);
       p_ctx->pModule = PyImport_Import(pName);
       Py_DECREF(pName);
 
       if (!p_ctx->pModule) {
-         Dmsg(ctx, dbglvl, "Failed to load module with name %s\n", p_ctx->module_name);
+         Dmsg(ctx, dbglvl, "python-sd: Failed to load module with name %s\n", p_ctx->module_name);
          goto bail_out;
       }
 
-      Dmsg(ctx, dbglvl, "Sucessfully loaded module with name %s\n", p_ctx->module_name);
+      Dmsg(ctx, dbglvl, "python-sd: Successfully loaded module with name %s\n", p_ctx->module_name);
 
       /*
        * Get the Python dictionary for lookups in the Python namespace.
@@ -704,15 +747,15 @@ static bRC PyLoadModule(bpContext *ctx, void *value)
             Py_DECREF(pRetVal);
          }
       } else {
-         Dmsg(ctx, dbglvl, "Failed to find function named load_bareos_plugins()\n");
+         Dmsg(ctx, dbglvl, "python-sd: Failed to find function named load_bareos_plugins()\n");
          goto bail_out;
       }
-   }
 
-   /*
-    * Keep track we successfully loaded.
-    */
-   p_ctx->python_loaded = true;
+      /*
+       * Keep track we successfully loaded.
+       */
+      p_ctx->python_loaded = true;
+   }
 
    return retval;
 
@@ -760,7 +803,7 @@ static bRC PyParsePluginDefinition(bpContext *ctx, void *value)
 
       return retval;
    } else {
-      Dmsg(ctx, dbglvl, "Failed to find function named parse_plugin_definition()\n");
+      Dmsg(ctx, dbglvl, "python-sd: Failed to find function named parse_plugin_definition()\n");
       return bRC_Error;
    }
 
@@ -808,7 +851,7 @@ static bRC PyHandlePluginEvent(bpContext *ctx, bsdEvent *event, void *value)
          Py_DECREF(pRetVal);
       }
    } else {
-      Dmsg(ctx, dbglvl, "Failed to find function named handle_plugin_event()\n");
+      Dmsg(ctx, dbglvl, "python-sd: Failed to find function named handle_plugin_event()\n");
    }
 
    return retval;
@@ -868,11 +911,13 @@ static PyObject *PyBareosGetValue(PyObject *self, PyObject *args)
    case bsdVarStorage:
    case bsdVarMediaType:
    case bsdVarVolumeName: {
-      char *value;
+      char *value = NULL;
 
       ctx = PyGetbpContext(pyCtx);
       if (bfuncs->getBareosValue(ctx, (bsdrVariable)var, &value) == bRC_OK) {
-         pRetVal = PyString_FromString(value);
+         if (value) {
+            pRetVal = PyString_FromString(value);
+         }
       }
       break;
    }
@@ -888,16 +933,18 @@ static PyObject *PyBareosGetValue(PyObject *self, PyObject *args)
       break;
    }
    case bsdVarPluginDir: {
-      char *value;
+      char *value = NULL;
 
       if (bfuncs->getBareosValue(NULL, (bsdrVariable)var, &value) == bRC_OK) {
-         pRetVal = PyString_FromString(value);
+         if (value) {
+            pRetVal = PyString_FromString(value);
+         }
       }
       break;
    }
    default:
       ctx = PyGetbpContext(pyCtx);
-      Dmsg(ctx, dbglvl, "PyBareosGetValue: Unknown variable requested %d\n", var);
+      Dmsg(ctx, dbglvl, "python-sd: PyBareosGetValue unknown variable requested %d\n", var);
       break;
    }
 
@@ -917,10 +964,11 @@ static PyObject *PyBareosSetValue(PyObject *self, PyObject *args)
 {
    int var;
    bpContext *ctx = NULL;
+   bRC retval = bRC_Error;
    PyObject *pyCtx, *pyValue;
 
    if (!PyArg_ParseTuple(args, "OiO:BareosSetValue", &pyCtx, &var, &pyValue)) {
-      return NULL;
+      goto bail_out;
    }
 
    switch (var) {
@@ -942,18 +990,18 @@ static PyObject *PyBareosSetValue(PyObject *self, PyObject *args)
       ctx = PyGetbpContext(pyCtx);
       value = PyInt_AsLong(pyValue);
       if (value >= 0) {
-         bfuncs->setBareosValue(ctx, (bsdwVariable)var, &value);
+         retval = bfuncs->setBareosValue(ctx, (bsdwVariable)var, &value);
       }
       break;
    }
    default:
       ctx = PyGetbpContext(pyCtx);
-      Dmsg(ctx, dbglvl, "PyBareosSetValue: Unknown variable requested %d\n", var);
+      Dmsg(ctx, dbglvl, "python-sd: PyBareosSetValue unknown variable requested %d\n", var);
       break;
    }
 
-   Py_INCREF(Py_None);
-   return Py_None;
+bail_out:
+   return conv_retval_python(retval);
 }
 
 /*
@@ -973,7 +1021,7 @@ static PyObject *PyBareosDebugMessage(PyObject *self, PyObject *args)
 
    if (dbgmsg) {
       ctx = PyGetbpContext(pyCtx);
-      Dmsg(ctx, level, dbgmsg);
+      Dmsg(ctx, level, "python-sd: %s", dbgmsg);
    }
 
    Py_INCREF(Py_None);
@@ -997,7 +1045,7 @@ static PyObject *PyBareosJobMessage(PyObject *self, PyObject *args)
 
    if (jobmsg) {
       ctx = PyGetbpContext(pyCtx);
-      Jmsg(ctx, type, jobmsg);
+      Jmsg(ctx, type, "python-sd: %s", jobmsg);
    }
 
    Py_INCREF(Py_None);
@@ -1013,15 +1061,16 @@ static PyObject *PyBareosRegisterEvents(PyObject *self, PyObject *args)
 {
    int len, event;
    bpContext *ctx;
+   bRC retval = bRC_Error;
    PyObject *pyCtx, *pyEvents, *pySeq, *pyEvent;
 
    if (!PyArg_ParseTuple(args, "OO:BareosRegisterEvents", &pyCtx, &pyEvents)) {
-      return NULL;
+      goto bail_out;
    }
 
    pySeq = PySequence_Fast(pyEvents, "Expected a sequence of events");
    if (!pySeq) {
-      return NULL;
+      goto bail_out;
    }
 
    len = PySequence_Fast_GET_SIZE(pySeq);
@@ -1032,13 +1081,90 @@ static PyObject *PyBareosRegisterEvents(PyObject *self, PyObject *args)
       event = PyInt_AsLong(pyEvent);
 
       if (event >= bsdEventJobStart && event <= bsdEventWriteRecordTranslation) {
-         Dmsg(ctx, dbglvl, "PyBareosRegisterEvents: registering event %d\n", event);
-         bfuncs->registerBareosEvents(ctx, 1, event);
+         Dmsg(ctx, dbglvl, "python-sd: PyBareosRegisterEvents registering event %d\n", event);
+         retval = bfuncs->registerBareosEvents(ctx, 1, event);
+
+         if (retval != bRC_OK) {
+            break;
+         }
       }
    }
 
    Py_DECREF(pySeq);
 
-   Py_INCREF(Py_None);
-   return Py_None;
+bail_out:
+   return conv_retval_python(retval);
+}
+
+/*
+ * Callback function which is exposed as a part of the additional methods which allow
+ * a Python plugin to issue an Unregister Event to unregister events it doesn't want to
+ * receive anymore.
+ */
+static PyObject *PyBareosUnRegisterEvents(PyObject *self, PyObject *args)
+{
+   int len, event;
+   bpContext *ctx;
+   bRC retval = bRC_Error;
+   PyObject *pyCtx, *pyEvents, *pySeq, *pyEvent;
+
+   if (!PyArg_ParseTuple(args, "OO:BareosUnRegisterEvents", &pyCtx, &pyEvents)) {
+      goto bail_out;
+   }
+
+   pySeq = PySequence_Fast(pyEvents, "Expected a sequence of events");
+   if (!pySeq) {
+      goto bail_out;
+   }
+
+   len = PySequence_Fast_GET_SIZE(pySeq);
+
+   ctx = PyGetbpContext(pyCtx);
+   for (int i = 0; i < len; i++) {
+      pyEvent = PySequence_Fast_GET_ITEM(pySeq, i);
+      event = PyInt_AsLong(pyEvent);
+
+      if (event >= bsdEventJobStart && event <= bsdEventWriteRecordTranslation) {
+         Dmsg(ctx, dbglvl, "PyBareosUnRegisterEvents: registering event %d\n", event);
+         retval = bfuncs->unregisterBareosEvents(ctx, 1, event);
+
+         if (retval != bRC_OK) {
+            break;
+         }
+      }
+   }
+
+   Py_DECREF(pySeq);
+
+bail_out:
+   return conv_retval_python(retval);
+}
+
+/*
+ * Callback function which is exposed as a part of the additional methods which allow
+ * a Python plugin to issue a GetInstanceCount to retrieve the number of instances of
+ * the current plugin being loaded into the daemon.
+ */
+static PyObject *PyBareosGetInstanceCount(PyObject *self, PyObject *args)
+{
+   int value;
+   bpContext *ctx = NULL;
+   PyObject *pyCtx;
+   PyObject *pRetVal = NULL;
+
+   if (!PyArg_ParseTuple(args, "O:BareosGetInstanceCount", &pyCtx)) {
+      return NULL;
+   }
+
+   ctx = PyGetbpContext(pyCtx);
+   if (bfuncs->getInstanceCount(ctx, &value) == bRC_OK) {
+      pRetVal = PyInt_FromLong(value);
+   }
+
+   if (!pRetVal) {
+      Py_INCREF(Py_None);
+      pRetVal = Py_None;
+   }
+
+   return pRetVal;
 }
