@@ -33,9 +33,13 @@ struct cl_opts
 {
    char *configfile;
    bool test_config_only;
+   bool export_config;
+   bool export_config_schema;
    cl_opts () {
       configfile = (char*)0;
       test_config_only = false;
+      export_config = false;
+      export_config_schema = false;
    }
 };
 
