@@ -31,12 +31,12 @@
 #include "stored.h"                   /* pull in Storage Deamon headers */
 #include "lib/crypto_cache.h"
 
-static const int dbglvl = 200;
+static const int dbglvl = 50;
 static pthread_mutex_t vol_info_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 /* Requests sent to the Director */
 static char Find_media[] =
-   "CatReq Job=%s FindMedia=%d pool_name=%s media_type=%s\n";
+   "CatReq Job=%s FindMedia=%d pool_name=%s media_type=%s unwanted_volumes=%s\n";
 static char Get_Vol_Info[] =
    "CatReq Job=%s GetVolInfo VolName=%s write=%d\n";
 static char Update_media[] =
@@ -83,7 +83,7 @@ bool SD_DCR::dir_update_device(JCR *jcr, DEVICE *dev)
    DEVRES *device = dev->device;
    bool ok;
 
-   pm_strcpy(dev_name, device->hdr.name);
+   pm_strcpy(dev_name, device->name());
    bash_spaces(dev_name);
    if (dev->is_labeled()) {
       pm_strcpy(VolumeName, dev->VolHdr.VolumeName);
@@ -94,22 +94,23 @@ bool SD_DCR::dir_update_device(JCR *jcr, DEVICE *dev)
    pm_strcpy(MediaType, device->media_type);
    bash_spaces(MediaType);
    if (device->changer_res) {
-      pm_strcpy(ChangerName, device->changer_res->hdr.name);
+      pm_strcpy(ChangerName, device->changer_res->name());
       bash_spaces(ChangerName);
    } else {
       pm_strcpy(ChangerName, "*");
    }
    ok = dir->fsend(Device_update,
-      jcr->Job,
-      dev_name.c_str(),
-      dev->can_append()!=0,
-      dev->can_read()!=0, dev->num_writers,
-      dev->is_open()!=0, dev->is_labeled()!=0,
-      dev->is_offline()!=0, dev->reserved_device,
-      dev->is_tape()?100000:1,
-      dev->autoselect, 0,
-      ChangerName.c_str(), MediaType.c_str(), VolumeName.c_str());
+                   jcr->Job,
+                   dev_name.c_str(),
+                   dev->can_append() != 0,
+                   dev->can_read() != 0, dev->num_writers,
+                   dev->is_open() != 0, dev->is_labeled() != 0,
+                   dev->is_offline() != 0, dev->reserved_device,
+                   dev->is_tape() ? 100000 : 1,
+                   dev->autoselect, 0,
+                   ChangerName.c_str(), MediaType.c_str(), VolumeName.c_str());
    Dmsg1(dbglvl, ">dird: %s", dir->msg);
+
    return ok;
 }
 
@@ -120,25 +121,26 @@ bool SD_DCR::dir_update_changer(JCR *jcr, AUTOCHANGER *changer)
    DEVRES *device;
    bool ok;
 
-   pm_strcpy(dev_name, changer->hdr.name);
+   pm_strcpy(dev_name, changer->name());
    bash_spaces(dev_name);
    device = (DEVRES *)changer->device->first();
    pm_strcpy(MediaType, device->media_type);
    bash_spaces(MediaType);
    /* This is mostly to indicate that we are here */
    ok = dir->fsend(Device_update,
-      jcr->Job,
-      dev_name.c_str(),         /* Changer name */
-      0, 0, 0,                  /* append, read, num_writers */
-      0, 0, 0,                  /* is_open, is_labeled, offline */
-      0, 0,                     /* reserved, max_writers */
-      0,                        /* Autoselect */
-      changer->device->size(),  /* Number of devices */
-      "0",                      /* PoolId */
-      "*",                      /* ChangerName */
-      MediaType.c_str(),        /* MediaType */
-      "*");                     /* VolName */
+                   jcr->Job,
+                   dev_name.c_str(),         /* Changer name */
+                   0, 0, 0,                  /* append, read, num_writers */
+                   0, 0, 0,                  /* is_open, is_labeled, offline */
+                   0, 0,                     /* reserved, max_writers */
+                   0,                        /* Autoselect */
+                   changer->device->size(),  /* Number of devices */
+                   "0",                      /* PoolId */
+                   "*",                      /* ChangerName */
+                   MediaType.c_str(),        /* MediaType */
+                   "*");                     /* VolName */
    Dmsg1(dbglvl, ">dird: %s", dir->msg);
+
    return ok;
 }
 #endif
@@ -231,7 +233,7 @@ static bool do_get_volume_info(DCR *dcr)
  *                              VolStatus==Recycle)
  *          false on failure
  *
- *          Volume information returned in dcr->VolCatInfo
+ * Volume information returned in dcr->VolCatInfo
  */
 bool SD_DCR::dir_get_volume_info(enum get_vol_info_rw writing)
 {
@@ -260,41 +262,44 @@ bool SD_DCR::dir_get_volume_info(enum get_vol_info_rw writing)
  *                also sets dcr->found_in_use if at least one
  *                in use volume was found.
  *
- *          Volume information returned in dcr
+ * Volume information returned in dcr
  */
 bool SD_DCR::dir_find_next_appendable_volume()
 {
+    bool retval;
     BSOCK *dir = jcr->dir_bsock;
-    bool rtn;
-    char lastVolume[MAX_NAME_LENGTH];
+    POOL_MEM unwanted_volumes(PM_MESSAGE);
 
     Dmsg2(dbglvl, "dir_find_next_appendable_volume: reserved=%d Vol=%s\n", is_reserved(), VolumeName);
 
     /*
-     * Try the twenty oldest or most available volumes.  Note,
+     * Try the twenty oldest or most available volumes. Note,
      * the most available could already be mounted on another
      * drive, so we continue looking for a not in use Volume.
      */
     lock_volumes();
     P(vol_info_mutex);
     clear_found_in_use();
-    lastVolume[0] = 0;
-    for (int vol_index=1;  vol_index < 20; vol_index++) {
+
+    pm_strcpy(unwanted_volumes, "");
+    for (int vol_index = 1; vol_index < 20; vol_index++) {
        bash_spaces(media_type);
        bash_spaces(pool_name);
-       dir->fsend(Find_media, jcr->Job, vol_index, pool_name, media_type);
+       bash_spaces(unwanted_volumes.c_str());
+       dir->fsend(Find_media, jcr->Job, vol_index, pool_name, media_type, unwanted_volumes.c_str());
        unbash_spaces(media_type);
        unbash_spaces(pool_name);
+       unbash_spaces(unwanted_volumes.c_str());
        Dmsg1(dbglvl, ">dird %s", dir->msg);
+
        if (do_get_volume_info(this)) {
-          /*
-           * Give up if we get the same volume name twice
-           */
-          if (lastVolume[0] && bstrcmp(lastVolume, VolumeName)) {
-             Dmsg1(dbglvl, "Got same vol = %s\n", lastVolume);
-             break;
+          if (vol_index == 1) {
+             pm_strcpy(unwanted_volumes, VolumeName);
+          } else {
+             pm_strcat(unwanted_volumes, ",");
+             pm_strcat(unwanted_volumes, VolumeName);
           }
-          bstrncpy(lastVolume, VolumeName, sizeof(lastVolume));
+
           if (can_i_write_volume()) {
              Dmsg1(dbglvl, "Call reserve_volume for write. Vol=%s\n", VolumeName);
              if (reserve_volume(this, VolumeName) == NULL) {
@@ -302,11 +307,14 @@ bool SD_DCR::dir_find_next_appendable_volume()
                 continue;
              }
              Dmsg1(dbglvl, "dir_find_next_appendable_volume return true. vol=%s\n", VolumeName);
-             rtn = true;
+             retval = true;
              goto get_out;
           } else {
              Dmsg1(dbglvl, "Volume %s is in use.\n", VolumeName);
-             /* If volume is not usable, it is in use by someone else */
+
+             /*
+              * If volume is not usable, it is in use by someone else
+              */
              set_found_in_use();
              continue;
           }
@@ -314,13 +322,14 @@ bool SD_DCR::dir_find_next_appendable_volume()
        Dmsg2(dbglvl, "No vol. index %d return false. dev=%s\n", vol_index, dev->print_name());
        break;
     }
-    rtn = false;
+    retval = false;
     VolumeName[0] = 0;
 
 get_out:
     V(vol_info_mutex);
     unlock_volumes();
-    return rtn;
+
+    return retval;
 }
 
 /**
@@ -337,7 +346,9 @@ bool SD_DCR::dir_update_volume_info(bool label, bool update_LastWritten)
    bool ok = false;
    POOL_MEM volume_name;
 
-   /* If system job, do not update catalog */
+   /*
+    * If system job, do not update catalog
+    */
    if (jcr->is_JobType(JT_SYSTEM)) {
       return true;
    }
@@ -348,10 +359,15 @@ bool SD_DCR::dir_update_volume_info(bool label, bool update_LastWritten)
       return false;
    }
 
-   /* Lock during Volume update */
+   /*
+    * Lock during Volume update
+    */
    P(vol_info_mutex);
    Dmsg1(dbglvl, "Update cat VolBytes=%lld\n", vol->VolCatBytes);
-   /* Just labeled or relabeled the tape */
+
+   /*
+    * Just labeled or relabeled the tape
+    */
    if (label) {
       bstrncpy(vol->VolCatStatus, "Append", sizeof(vol->VolCatStatus));
    }
@@ -374,7 +390,9 @@ bool SD_DCR::dir_update_volume_info(bool label, bool update_LastWritten)
               edit_uint64(vol->VolFirstWritten, ed5));
    Dmsg1(dbglvl, ">dird %s", dir->msg);
 
-   /* Do not lock device here because it may be locked from label */
+   /*
+    * Do not lock device here because it may be locked from label
+    */
    if (!jcr->is_canceled()) {
       if (!do_get_volume_info(this)) {
          Jmsg(jcr, M_FATAL, 0, "%s", jcr->errmsg);
@@ -383,7 +401,10 @@ bool SD_DCR::dir_update_volume_info(bool label, bool update_LastWritten)
          goto bail_out;
       }
       Dmsg1(420, "get_volume_info() %s", dir->msg);
-      /* Update dev Volume info in case something changed (e.g. expired) */
+
+      /*
+       * Update dev Volume info in case something changed (e.g. expired)
+       */
       dev->VolCatInfo = VolCatInfo;
       ok = true;
    }
@@ -401,12 +422,16 @@ bool SD_DCR::dir_create_jobmedia_record(bool zero)
    BSOCK *dir = jcr->dir_bsock;
    char ed1[50];
 
-   /* If system job, do not update catalog */
+   /*
+    * If system job, do not update catalog
+    */
    if (jcr->is_JobType(JT_SYSTEM)) {
       return true;
    }
 
-   /* Throw out records where FI is zero -- i.e. nothing done */
+   /*
+    * Throw out records where FI is zero -- i.e. nothing done
+    */
    if (!zero && VolFirstIndex == 0 &&
         (StartBlock != 0 || EndBlock != 0)) {
       Dmsg0(dbglvl, "JobMedia FI=0 StartBlock!=0 record suppressed\n");
@@ -419,7 +444,9 @@ bool SD_DCR::dir_create_jobmedia_record(bool zero)
 
    WroteVol = false;
    if (zero) {
-      /* Send dummy place holder to avoid purging */
+      /*
+       * Send dummy place holder to avoid purging
+       */
       dir->fsend(Create_job_media, jcr->Job,
                  0 , 0, 0, 0, 0, 0, 0, 0, edit_uint64(VolMediaId, ed1));
    } else {
@@ -431,6 +458,7 @@ bool SD_DCR::dir_create_jobmedia_record(bool zero)
                  edit_uint64(VolMediaId, ed1));
    }
    Dmsg1(dbglvl, ">dird %s", dir->msg);
+
    if (dir->recv() <= 0) {
       Dmsg0(dbglvl, "create_jobmedia error bnet_recv\n");
       Jmsg(jcr, M_FATAL, 0, _("Error creating JobMedia record: ERR=%s\n"),
@@ -438,11 +466,13 @@ bool SD_DCR::dir_create_jobmedia_record(bool zero)
       return false;
    }
    Dmsg1(dbglvl, "<dird %s", dir->msg);
+
    if (!bstrcmp(dir->msg, OK_create)) {
       Dmsg1(dbglvl, "Bad response from Dir: %s\n", dir->msg);
       Jmsg(jcr, M_FATAL, 0, _("Error creating JobMedia record: %s\n"), dir->msg);
       return false;
    }
+
    return true;
 }
 
@@ -451,15 +481,17 @@ bool SD_DCR::dir_create_jobmedia_record(bool zero)
  * We do the following:
  *  1. expand the bsock buffer to be large enough
  *  2. Write a "header" into the buffer with serialized data
- *    VolSessionId
- *    VolSeesionTime
- *    FileIndex
- *    Stream
- *    data length that follows
- *    start of raw byte data from the Device record.
+ *     VolSessionId
+ *     VolSeesionTime
+ *     FileIndex
+ *     Stream
+ *     data length that follows
+ *     start of raw byte data from the Device record.
+ *
  * Note, this is primarily for Attribute data, but can
- *   also handle any device record. The Director must know
- *   the raw byte data format that is defined for each Stream.
+ * also handle any device record. The Director must know
+ * the raw byte data format that is defined for each Stream.
+ *
  * Now Restore Objects pass through here STREAM_RESTORE_OBJECT
  */
 bool SD_DCR::dir_update_file_attributes(DEV_RECORD *record)
@@ -484,26 +516,28 @@ bool SD_DCR::dir_update_file_attributes(DEV_RECORD *record)
    ser_bytes(record->data, record->data_len);
    dir->msglen = ser_length(dir->msg);
    Dmsg1(1800, ">dird %s", dir->msg);    /* Attributes */
+
    return dir->send();
 }
 
 /**
- *   Request the sysop to create an appendable volume
+ * Request the sysop to create an appendable volume
  *
- *   Entered with device blocked.
- *   Leaves with device blocked.
+ * Entered with device blocked.
+ * Leaves with device blocked.
  *
- *   Returns: true  on success (operator issues a mount command)
- *            false on failure
- *              Note, must create dev->errmsg on error return.
+ * Returns: true  on success (operator issues a mount command)
+ *          false on failure
  *
- *    On success, dcr->VolumeName and dcr->VolCatInfo contain
- *      information on suggested volume, but this may not be the
- *      same as what is actually mounted.
+ * Note, must create dev->errmsg on error return.
  *
- *    When we return with success, the correct tape may or may not
- *      actually be mounted. The calling routine must read it and
- *      verify the label.
+ * On success, dcr->VolumeName and dcr->VolCatInfo contain
+ * information on suggested volume, but this may not be the
+ * same as what is actually mounted.
+ *
+ * When we return with success, the correct tape may or may not
+ * actually be mounted. The calling routine must read it and
+ * verify the label.
  */
 bool SD_DCR::dir_ask_sysop_to_create_appendable_volume()
 {
@@ -513,6 +547,7 @@ bool SD_DCR::dir_ask_sysop_to_create_appendable_volume()
    if (job_canceled(jcr)) {
       return false;
    }
+
    Dmsg0(dbglvl, "enter dir_ask_sysop_to_create_appendable_volume\n");
    ASSERT(dev->blocked());
    for ( ;; ) {
@@ -529,15 +564,15 @@ bool SD_DCR::dir_ask_sysop_to_create_appendable_volume()
       } else {
          if (status == W_TIMEOUT || status == W_MOUNT) {
             Mmsg(dev->errmsg, _(
-"Job %s is waiting. Cannot find any appendable volumes.\n"
-"Please use the \"label\" command to create a new Volume for:\n"
-"    Storage:      %s\n"
-"    Pool:         %s\n"
-"    Media type:   %s\n"),
-               jcr->Job,
-               dev->print_name(),
-               pool_name,
-               media_type);
+                 "Job %s is waiting. Cannot find any appendable volumes.\n"
+                 "Please use the \"label\" command to create a new Volume for:\n"
+                 "    Storage:      %s\n"
+                 "    Pool:         %s\n"
+                 "    Media type:   %s\n"),
+                 jcr->Job,
+                 dev->print_name(),
+                 pool_name,
+                 media_type);
             Jmsg(jcr, M_MOUNT, 0, "%s", dev->errmsg);
             Dmsg1(dbglvl, "%s", dev->errmsg);
          }
@@ -562,6 +597,7 @@ bool SD_DCR::dir_ask_sysop_to_create_appendable_volume()
          }
          continue;
       }
+
       if (status == W_ERROR) {
          berrno be;
          Mmsg0(dev->errmsg, _("pthread error in mount_next_volume.\n"));
@@ -574,6 +610,7 @@ bool SD_DCR::dir_ask_sysop_to_create_appendable_volume()
 get_out:
    jcr->sendJobStatus(JS_Running);
    Dmsg0(dbglvl, "leave dir_ask_sysop_to_mount_create_appendable_volume\n");
+
    return true;
 }
 
@@ -606,9 +643,8 @@ bool SD_DCR::dir_ask_sysop_to_mount_volume(int mode)
       }
 
       /*
-       * If we are not polling, and the wait timeout or the
-       *   user explicitly did a mount, send him the message.
-       *   Otherwise skip it.
+       * If we are not polling, and the wait timeout or the user explicitly did a mount,
+       * send him the message. Otherwise skip it.
        */
       if (!dev->poll && (status == W_TIMEOUT || status == W_MOUNT)) {
          const char *msg;
@@ -652,6 +688,7 @@ bool SD_DCR::dir_ask_sysop_to_mount_volume(int mode)
          }
          continue;
       }
+
       if (status == W_ERROR) {
          berrno be;
          Mmsg(dev->errmsg, _("pthread error in mount_volume\n"));
