@@ -62,8 +62,6 @@ static char *wbuf;                    /* write buffer address */
 static uint32_t wsize;                /* write size */
 static uint64_t fileAddr = 0;         /* file write address */
 
-#define CONFIG_FILE "bareos-sd.conf"
-
 static void usage()
 {
    fprintf(stderr, _(
@@ -71,7 +69,7 @@ PROG_COPYRIGHT
 "\nVersion: %s (%s)\n\n"
 "Usage: bextract <options> <bareos-archive-device-name> <directory-to-store-files>\n"
 "       -b <file>       specify a bootstrap file\n"
-"       -c <file>       specify a Storage configuration file\n"
+"       -c <path>       specify a Storage configuration file or directory\n"
 "       -D <director>   specify a director name specified in the Storage\n"
 "                       configuration file for the Key Encryption Key selection\n"
 "       -d <nn>         set debug level to <nn>\n"
@@ -196,16 +194,12 @@ int main (int argc, char *argv[])
       usage();
    }
 
-   if (configfile == NULL) {
-      configfile = bstrdup(CONFIG_FILE);
-   }
-
    my_config = new_config_parser();
    parse_sd_config(my_config, configfile, M_ERROR_TERM);
 
    if (DirectorName) {
       foreach_res(director, R_DIRECTOR) {
-         if (bstrcmp(director->hdr.name, DirectorName)) {
+         if (bstrcmp(director->name(), DirectorName)) {
             break;
          }
       }
