@@ -86,12 +86,16 @@
 #ifdef HAVE_CEPHFS
 #include "backends/cephfs_device.h"
 #endif
+#ifdef HAVE_ELASTO
+#include "backends/elasto_device.h"
+#endif
 #include "backends/generic_tape_device.h"
 #ifdef HAVE_WIN32
 #include "backends/win32_tape_device.h"
+#include "backends/win32_fifo_device.h"
 #else
-#include "backends/unix_fifo_device.h"
 #include "backends/unix_tape_device.h"
+#include "backends/unix_fifo_device.h"
 #endif
 #endif /* HAVE_DYNAMIC_SD_BACKENDS */
 
@@ -155,7 +159,7 @@ static inline DEVICE *m_init_dev(JCR *jcr, DEVRES *device, bool new_init)
          device->dev_type = B_TAPE_DEV;
       } else if (S_ISFIFO(statp.st_mode)) {
          device->dev_type = B_FIFO_DEV;
-      } else if (!(device->cap_bits & CAP_REQMOUNT)) {
+      } else if (!bit_is_set(CAP_REQMOUNT, device->cap_bits)) {
          Jmsg2(jcr, M_ERROR, 0,
                _("%s is an unknown device type. Must be tape or directory, st_mode=%x\n"),
                device->device_name, statp.st_mode);
@@ -192,10 +196,18 @@ static inline DEVICE *m_init_dev(JCR *jcr, DEVRES *device, bool new_init)
       dev = New(cephfs_device);
       break;
 #endif
+#ifdef HAVE_ELASTO
+   case B_ELASTO_DEV:
+      dev = New(elasto_device);
+      break;
+#endif
 #ifdef HAVE_WIN32
    case B_TAPE_DEV:
       dev = New(win32_tape_device);
       break;
+   case B_FIFO_DEV:
+      dev = New(win32_fifo_device);
+      break;
 #else
    case B_TAPE_DEV:
       dev = New(unix_tape_device);
@@ -231,16 +243,20 @@ static inline DEVICE *m_init_dev(JCR *jcr, DEVRES *device, bool new_init)
    /*
     * Copy user supplied device parameters from Resource
     */
-   dev->dev_name = get_memory(strlen(device->device_name)+1);
+   dev->dev_name = get_memory(strlen(device->device_name) + 1);
    pm_strcpy(dev->dev_name, device->device_name);
-   dev->prt_name = get_memory(strlen(device->device_name) + strlen(device->hdr.name) + 20);
+   if (device->device_options) {
+      dev->dev_options = get_memory(strlen(device->device_options) + 1);
+      pm_strcpy(dev->dev_options, device->device_options);
+   }
+   dev->prt_name = get_memory(strlen(device->device_name) + strlen(device->name()) + 20);
 
    /*
     * We edit "Resource-name" (physical-name)
     */
-   Mmsg(dev->prt_name, "\"%s\" (%s)", device->hdr.name, device->device_name);
+   Mmsg(dev->prt_name, "\"%s\" (%s)", device->name(), device->device_name);
    Dmsg1(400, "Allocate dev=%s\n", dev->print_name());
-   dev->capabilities = device->cap_bits;
+   copy_bits(CAP_MAX, device->cap_bits, dev->capabilities);
 
    /*
     * current block sizes
@@ -256,6 +272,7 @@ static inline DEVICE *m_init_dev(JCR *jcr, DEVRES *device, bool new_init)
    dev->max_open_vols = device->max_open_vols;
    dev->vol_poll_interval = device->vol_poll_interval;
    dev->max_spool_size = device->max_spool_size;
+   dev->drive = device->drive;
    dev->drive_index = device->drive_index;
    dev->autoselect = device->autoselect;
    dev->norewindonclose = device->norewindonclose;
@@ -271,7 +288,7 @@ static inline DEVICE *m_init_dev(JCR *jcr, DEVRES *device, bool new_init)
    device->dev = dev;
 
    if (dev->is_fifo()) {
-      dev->capabilities |= CAP_STREAM; /* set stream device */
+      dev->set_cap(CAP_STREAM);       /* set stream device */
    }
 
    /*
@@ -516,7 +533,9 @@ void DEVICE::set_label_blocksize(DCR *dcr)
  */
 bool DEVICE::open(DCR *dcr, int omode)
 {
-   int preserve = 0;
+   char preserve[ST_BYTES];
+
+   clear_all_bits(ST_MAX, preserve);
    if (is_open()) {
       if (open_mode == omode) {
          return true;
@@ -524,9 +543,16 @@ bool DEVICE::open(DCR *dcr, int omode)
          d_close(m_fd);
          clear_opened();
          Dmsg0(100, "Close fd for mode change.\n");
-         preserve = state & (ST_LABEL | ST_APPENDREADY | ST_READREADY);
+
+         if (bit_is_set(ST_LABEL, state))
+            set_bit(ST_LABEL, preserve);
+         if (bit_is_set(ST_APPENDREADY, state))
+            set_bit(ST_APPENDREADY, preserve);
+         if (bit_is_set(ST_READREADY, state))
+            set_bit(ST_READREADY, preserve);
       }
    }
+
    if (dcr) {
       dcr->setVolCatName(dcr->VolumeName);
       VolCatInfo = dcr->VolCatInfo;    /* structure assign */
@@ -534,13 +560,32 @@ bool DEVICE::open(DCR *dcr, int omode)
 
    Dmsg4(100, "open dev: type=%d dev_name=%s vol=%s mode=%s\n", dev_type,
          print_name(), getVolCatName(), mode_to_str(omode));
-   state &= ~(ST_LABEL | ST_APPENDREADY | ST_READREADY | ST_EOT | ST_WEOT | ST_EOF);
+
+   clear_bit(ST_LABEL, state);
+   clear_bit(ST_APPENDREADY, state);
+   clear_bit(ST_READREADY, state);
+   clear_bit(ST_EOT, state);
+   clear_bit(ST_WEOT, state);
+   clear_bit(ST_EOF, state);
+
    label_type = B_BAREOS_LABEL;
 
+   /*
+    * We are about to open the device so let any plugin know we are.
+    */
+   if (dcr && generate_plugin_event(dcr->jcr, bsdEventDeviceOpen, dcr) != bRC_OK) {
+      Dmsg0(100, "open_dev: bsdEventDeviceOpen failed\n");
+      return false;
+   }
+
    Dmsg1(100, "call open_device mode=%s\n", mode_to_str(omode));
    open_device(dcr, omode);
 
-   state |= preserve;                 /* reset any important state info */
+   /*
+    * Reset any important state info
+    */
+   clone_bits(ST_MAX, preserve, state);
+
    Dmsg2(100, "preserve=0x%x fd=%d\n", preserve, m_fd);
 
    return m_fd >= 0;
@@ -635,7 +680,14 @@ void DEVICE::open_device(DCR *dcr, int omode)
 bool DEVICE::rewind(DCR *dcr)
 {
    Dmsg3(400, "rewind res=%d fd=%d %s\n", num_reserved(), m_fd, print_name());
-   state &= ~(ST_EOT | ST_EOF | ST_WEOT); /* Remove EOF/EOT flags */
+
+   /*
+    * Remove EOF/EOT flags
+    */
+   clear_bit(ST_EOT, state);
+   clear_bit(ST_EOF, state);
+   clear_bit(ST_WEOT, state);
+
    block_num = file = 0;
    file_size = 0;
    file_addr = 0;
@@ -677,7 +729,9 @@ void DEVICE::set_ateot()
    /*
     * Make volume effectively read-only
     */
-   state |= (ST_EOF | ST_EOT | ST_WEOT);
+   set_bit(ST_EOF, state);
+   set_bit(ST_EOT, state);
+   set_bit(ST_WEOT, state);
    clear_append();
 }
 
@@ -769,21 +823,25 @@ bool DEVICE::update_pos(DCR *dcr)
    return ok;
 }
 
-uint32_t DEVICE::status_dev()
+char *DEVICE::status_dev()
 {
-   uint32_t status = 0;
+   char *status;
 
-   if (state & (ST_EOT | ST_WEOT)) {
-      status |= BMT_EOD;
+   status = (char *)malloc(BMT_BYTES);
+   clear_all_bits(BMT_MAX, status);
+
+   if (bit_is_set(ST_EOT, state) || bit_is_set(ST_WEOT, state)) {
+      set_bit(BMT_EOD, status);
       Pmsg0(-20, " EOD");
    }
 
-   if (state & ST_EOF) {
-      status |= BMT_EOF;
+   if (bit_is_set(ST_EOF, state)) {
+      set_bit(BMT_EOF, status);
       Pmsg0(-20, " EOF");
    }
 
-   status |= BMT_ONLINE | BMT_BOT;
+   set_bit(BMT_ONLINE, status);
+   set_bit(BMT_BOT, status);
 
    return status;
 }
@@ -808,16 +866,20 @@ bool DEVICE::offline_or_rewind()
    }
 }
 
-void DEVICE::set_slot(int32_t slot)
+void DEVICE::set_slot(slot_number_t slot)
 {
    m_slot = slot;
-   if (vol) vol->clear_slot();
+   if (vol) {
+      vol->clear_slot();
+   }
 }
 
 void DEVICE::clear_slot()
 {
    m_slot = -1;
-   if (vol) vol->set_slot(-1);
+   if (vol) {
+      vol->set_slot(-1);
+   }
 }
 
 /*
@@ -877,32 +939,37 @@ void DEVICE::clear_volhdr()
 /*
  * Close the device.
  */
-void DEVICE::close(DCR *dcr)
+bool DEVICE::close(DCR *dcr)
 {
+   bool retval = true;
    int status;
    Dmsg1(100, "close_dev %s\n", print_name());
 
-   if (!norewindonclose) {
-      offline_or_rewind();
-   }
-
    if (!is_open()) {
       Dmsg2(100, "device %s already closed vol=%s\n", print_name(), VolHdr.VolumeName);
-      return;                         /* already closed */
+      goto bail_out;                  /* already closed */
+   }
+
+   if (!norewindonclose) {
+      offline_or_rewind();
    }
 
    switch (dev_type) {
    case B_VTL_DEV:
    case B_TAPE_DEV:
       unlock_door();
-      /* Fall through wanted */
+      /*
+       * Fall through wanted
+       */
    default:
       status = d_close(m_fd);
       if (status < 0) {
          berrno be;
 
-         Mmsg2(errmsg, _("Unable to close device %s. ERR=%s\n"), print_name(), be.bstrerror());
-         Jmsg(dcr->jcr, M_FATAL, 0, errmsg);
+         Mmsg2(errmsg, _("Unable to close device %s. ERR=%s\n"),
+               print_name(), be.bstrerror());
+         dev_errno = errno;
+         retval = false;
       }
       break;
    }
@@ -914,8 +981,16 @@ void DEVICE::close(DCR *dcr)
     */
    clear_opened();
 
-   state &= ~(ST_LABEL | ST_READREADY | ST_APPENDREADY | ST_EOT | ST_WEOT |
-              ST_EOF | ST_MOUNTED | ST_MEDIA | ST_SHORT);
+   clear_bit(ST_LABEL, state);
+   clear_bit(ST_READREADY, state);
+   clear_bit(ST_APPENDREADY, state);
+   clear_bit(ST_EOT, state);
+   clear_bit(ST_WEOT, state);
+   clear_bit(ST_EOF, state);
+   clear_bit(ST_MOUNTED, state);
+   clear_bit(ST_MEDIA, state);
+   clear_bit(ST_SHORT, state);
+
    label_type = B_BAREOS_LABEL;
    file = block_num = 0;
    file_size = 0;
@@ -928,6 +1003,16 @@ void DEVICE::close(DCR *dcr)
       stop_thread_timer(tid);
       tid = 0;
    }
+
+   /*
+    * We closed the device so let any plugin know we did.
+    */
+   if (dcr) {
+      generate_plugin_event(dcr->jcr, bsdEventDeviceClose, dcr);
+   }
+
+bail_out:
+   return retval;
 }
 
 /*
@@ -951,9 +1036,7 @@ bool DEVICE::mount(DCR *dcr, int timeout)
     * bsdEventDeviceMount plugin event so any plugin
     * that want to do something can do things now.
     */
-   if (retval && generate_plugin_event(dcr->jcr,
-                                       bsdEventDeviceMount,
-                                       dcr) != bRC_OK) {
+   if (retval && generate_plugin_event(dcr->jcr, bsdEventDeviceMount, dcr) != bRC_OK) {
       retval = false;
    }
 
@@ -989,9 +1072,7 @@ bool DEVICE::unmount(DCR *dcr, int timeout)
     * bsdEventDeviceUnmount plugin event so any plugin
     * that want to do something can do things now.
     */
-   if (dcr && generate_plugin_event(dcr->jcr,
-                                    bsdEventDeviceUnmount,
-                                    dcr) != bRC_OK) {
+   if (dcr && generate_plugin_event(dcr->jcr, bsdEventDeviceUnmount, dcr) != bRC_OK) {
       retval = false;
       goto bail_out;
    }
@@ -1122,7 +1203,7 @@ ssize_t DEVICE::write(const void *buf, size_t len)
  */
 const char *DEVICE::name() const
 {
-   return device->hdr.name;
+   return device->name();
 }
 
 /*
@@ -1146,6 +1227,10 @@ void DEVICE::term()
       free_memory(dev_name);
       dev_name = NULL;
    }
+   if (dev_options) {
+      free_memory(dev_options);
+      dev_options = NULL;
+   }
    if (prt_name) {
       free_memory(prt_name);
       prt_name = NULL;
@@ -1192,7 +1277,6 @@ void init_device_wait_timers(DCR *dcr)
    jcr->wait_sec = jcr->min_wait;
    jcr->rem_wait_sec = jcr->wait_sec;
    jcr->num_wait = 0;
-
 }
 
 void init_jcr_device_wait_timers(JCR *jcr)
