@@ -3,7 +3,7 @@
 
    Copyright (C) 2001-2012 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2015 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -45,7 +45,6 @@
 /* Exported variables */
 
 /* Imported variables */
-extern BSOCK *filed_chan;
 extern struct s_last_job last_job;
 extern bool init_done;
 
@@ -60,24 +59,24 @@ static char cancelcmd[] =
    "cancel Job=%127s";
 static char relabelcmd[] =
    "relabel %127s OldName=%127s NewName=%127s PoolName=%127s "
-   "MediaType=%127s Slot=%d drive=%d MinBlocksize=%d MaxBlocksize=%d";
+   "MediaType=%127s Slot=%hd drive=%hd MinBlocksize=%d MaxBlocksize=%d";
 static char labelcmd[] =
    "label %127s VolumeName=%127s PoolName=%127s "
-   "MediaType=%127s Slot=%d drive=%d MinBlocksize=%d MaxBlocksize=%d";
+   "MediaType=%127s Slot=%hd drive=%hd MinBlocksize=%d MaxBlocksize=%d";
 static char mountslotcmd[] =
-   "mount %127s drive=%d slot=%d";
+   "mount %127s drive=%hd slot=%hd";
 static char mountcmd[] =
-   "mount %127s drive=%d";
+   "mount %127s drive=%hd";
 static char unmountcmd[] =
-   "unmount %127s drive=%d";
+   "unmount %127s drive=%hd";
 #if 0
 static char actionopcmd[] =
    "action_on_purge %127s vol=%127s action=%d";
 #endif
 static char releasecmd[] =
-   "release %127s drive=%d";
+   "release %127s drive=%hd";
 static char readlabelcmd[] =
-   "readlabel %127s Slot=%d drive=%d";
+   "readlabel %127s Slot=%hd drive=%hd";
 static char replicatecmd[] =
    "replicate Job=%127s address=%s port=%d ssl=%d Authorization=%100s";
 static char passiveclientcmd[] =
@@ -110,6 +109,8 @@ static char OKpassive[] =
    "2000 OK passive client\n";
 static char OKpluginoptions[] =
    "2000 OK plugin options\n";
+static char OKsecureerase[] =
+   "2000 OK SDSecureEraseCmd %s \n";
 
 /* Imported functions */
 extern bool finish_cmd(JCR *jcr);
@@ -142,16 +143,18 @@ static bool relabel_cmd(JCR *jcr);
 static bool release_cmd(JCR *jcr);
 static bool replicate_cmd(JCR *jcr);
 static bool run_cmd(JCR *jcr);
+static bool secureerasereq_cmd(JCR *jcr);
 static bool setbandwidth_cmd(JCR *jcr);
 static bool setdebug_cmd(JCR *jcr);
 static bool unmount_cmd(JCR *jcr);
 
-static DCR *find_device(JCR *jcr, POOL_MEM &dev_name, int drive, BLOCKSIZES *blocksizes);
-static void read_volume_label(JCR *jcr, DCR *dcr, DEVICE *dev, int Slot);
+static DCR *find_device(JCR *jcr, POOL_MEM &dev_name,
+                        drive_number_t drive, BLOCKSIZES *blocksizes);
+static void read_volume_label(JCR *jcr, DCR *dcr, DEVICE *dev, slot_number_t slot);
 static void label_volume_if_ok(DCR *dcr, char *oldname,
                                char *newname, char *poolname,
-                               int Slot, bool relabel);
-static bool try_autoload_device(JCR *jcr, DCR *dcr, int slot, const char *VolName);
+                               slot_number_t Slot, bool relabel);
+static int try_autoload_device(JCR *jcr, DCR *dcr, slot_number_t slot, const char *VolName);
 static void send_dir_busy_message(BSOCK *dir, DEVICE *dev);
 
 struct s_cmds {
@@ -186,6 +189,7 @@ static struct s_cmds cmds[] = {
    { "resolve", resolve_cmd, false },
    { "replicate", replicate_cmd, false },   /* Replicate data to an external SD */
    { "run", run_cmd, false },               /* Start of Job */
+   { "getSecureEraseCmd", secureerasereq_cmd, false },
    { "setbandwidth=", setbandwidth_cmd, false },
    { "setdebug=", setdebug_cmd, false },    /* Set debug level */
    { "stats", stats_cmd, false },
@@ -196,6 +200,22 @@ static struct s_cmds cmds[] = {
    { NULL, NULL, false } /* list terminator */
 };
 
+ /*
+  * Count the number of running jobs.
+  */
+static inline bool count_running_jobs()
+{
+   JCR *jcr;
+   unsigned int cnt = 0;
+
+   foreach_jcr(jcr) {
+      cnt++;
+   }
+   endeach_jcr(jcr);
+
+   return (cnt >= me->MaxConcurrentJobs) ? false : true;
+}
+
 /*
  * Connection request from an director.
  *
@@ -206,13 +226,18 @@ static struct s_cmds cmds[] = {
  *  - We execute the command
  *  - We continue or exit depending on the return status
  */
-static void *handle_director_connection(BSOCK *dir, char *job_name)
+void *handle_director_connection(BSOCK *dir)
 {
    JCR *jcr;
    int i, errstat;
    int bnet_stat = 0;
    bool found, quit;
 
+   if (!count_running_jobs()) {
+      Emsg0(M_ERROR, 0, _("Number of Jobs exhausted, please increase MaximumConcurrentJobs\n"));
+      return NULL;
+   }
+
    /*
     * This is a connection from the Director, so setup a JCR
     */
@@ -310,64 +335,6 @@ bail_out:
    return NULL;
 }
 
-/*
- * Connection request. We accept connections either from the
- * Director, Storage Daemon or a Client (File daemon).
- *
- * Note, we are running as a seperate thread of the Storage daemon.
- *
- * Basic tasks done here:
- *  - If it was a connection from the FD, call handle_filed_connection()
- *  - If it was a connection from an other SD, call handle_stored_connection()
- *  - Otherwise it was a connection from the DIR, call handle_director_connection()
- */
-void *handle_connection_request(void *arg)
-{
-   BSOCK *bs = (BSOCK *)arg;
-   char name[500];
-   char tbuf[100];
-
-   if (bs->recv() <= 0) {
-      Emsg1(M_ERROR, 0, _("Connection request from %s failed.\n"), bs->who());
-      bmicrosleep(5, 0);   /* make user wait 5 seconds */
-      bs->close();
-      return NULL;
-   }
-
-   /*
-    * Do a sanity check on the message received
-    */
-   if (bs->msglen < 25 || bs->msglen > (int)sizeof(name)) {
-      Dmsg1(000, "<filed: %s", bs->msg);
-      Emsg2(M_ERROR, 0, _("Invalid connection from %s. Len=%d\n"), bs->who(), bs->msglen);
-      bmicrosleep(5, 0);   /* make user wait 5 seconds */
-      bs->close();
-      return NULL;
-   }
-
-   Dmsg1(110, "Conn: %s", bs->msg);
-
-   /*
-    * See if this is a File daemon connection. If so call FD handler.
-    */
-   if (sscanf(bs->msg, "Hello Start Job %127s", name) == 1) {
-      Dmsg1(110, "Got a FD connection at %s\n", bstrftimes(tbuf, sizeof(tbuf), (utime_t)time(NULL)));
-      return handle_filed_connection(bs, name);
-   }
-
-   /*
-    * See if this is a Storage daemon connection. If so call SD handler.
-    */
-   if (sscanf(bs->msg, "Hello Start Storage Job %127s", name) == 1) {
-      Dmsg1(110, "Got a SD connection at %s\n", bstrftimes(tbuf, sizeof(tbuf), (utime_t)time(NULL)));
-      return handle_stored_connection(bs, name);
-   }
-
-   Dmsg1(110, "Got a DIR connection at %s\n", bstrftimes(tbuf, sizeof(tbuf), (utime_t)time(NULL)));
-
-   return handle_director_connection(bs, name);
-}
-
 /*
  * Force SD to die, and hopefully dump itself.  Turned on only in development version.
  */
@@ -392,6 +359,20 @@ static bool die_cmd(JCR *jcr)
    return false;
 }
 
+
+/*
+ * Handles the secureerase request
+ * replies the configured secure erase command
+ * or "*None*"
+ */
+static bool secureerasereq_cmd(JCR *jcr) {
+   BSOCK *dir = jcr->dir_bsock;
+
+   Dmsg1(220,"Secure Erase Cmd Request: %s\n", (me->secure_erase_cmdline ? me->secure_erase_cmdline : "*None*"));
+
+   return dir->fsend(OKsecureerase, (me->secure_erase_cmdline ? me->secure_erase_cmdline : "*None*"));
+}
+
 /**
  * Set bandwidth limit as requested by the Director
  */
@@ -607,8 +588,8 @@ static bool do_label(JCR *jcr, bool relabel)
    DEVICE *dev;
    BLOCKSIZES blocksizes;
    bool ok = false;
-   int32_t slot, drive;
-   //, max_block_size, min_block_size;
+   slot_number_t slot;
+   drive_number_t drive;
 
    /*
     * Determine the length of the temporary buffers.
@@ -711,7 +692,7 @@ static bool relabel_cmd(JCR *jcr)
  */
 static void label_volume_if_ok(DCR *dcr, char *oldname,
                                char *newname, char *poolname,
-                               int slot, bool relabel)
+                               slot_number_t slot, bool relabel)
 {
    BSOCK *dir = dcr->jcr->dir_bsock;
    bsteal_lock_t hold;
@@ -725,30 +706,43 @@ static void label_volume_if_ok(DCR *dcr, char *oldname,
    Dmsg1(100, "Stole device %s lock, writing label.\n", dev->print_name());
 
    Dmsg0(90, "try_autoload_device - looking for volume_info\n");
-   if (!try_autoload_device(dcr->jcr, dcr, slot, volname)) {
-      goto bail_out;                  /* error */
+   switch (try_autoload_device(dcr->jcr, dcr, slot, volname)) {
+   case -1:
+      goto cleanup;
+   case -2:
+      goto bail_out;
+   case 0:
+   default:
+      break;
    }
 
-   /* Ensure that the device is open -- autoload_device() closes it */
+   /*
+    * Ensure that the device is open -- autoload_device() closes it
+    */
    if (dev->is_tape()) {
       mode = OPEN_READ_WRITE;
    } else {
       mode = CREATE_READ_WRITE;
    }
 
-   /* Set old volume name for open if relabeling */
+   /*
+    * Set old volume name for open if relabeling
+    */
    dcr->setVolCatName(volname);
 
    if (!dev->open(dcr, mode)) {
-      dir->fsend(_("3910 Unable to open device \"%s\": ERR=%s\n"),
-                 dev->print_name(), dev->bstrerror());
-      goto bail_out;
+      dir->fsend(_("3910 Unable to open device \"%s\": ERR=%s\n"), dev->print_name(), dev->bstrerror());
+      goto cleanup;
    }
 
-   /* See what we have for a Volume */
+   /*
+    * See what we have for a Volume
+    */
    label_status = read_dev_volume_label(dcr);
 
-   /* Set new volume name */
+   /*
+    * Set new volume name
+    */
    dcr->setVolCatName(newname);
    switch(label_status) {
    case VOL_NAME_ERROR:
@@ -756,29 +750,36 @@ static void label_volume_if_ok(DCR *dcr, char *oldname,
    case VOL_LABEL_ERROR:
    case VOL_OK:
       if (!relabel) {
-         dir->fsend(_("3920 Cannot label Volume because it is already labeled: \"%s\"\n"),
-             dev->VolHdr.VolumeName);
-         goto bail_out;
+         dir->fsend(_("3920 Cannot label Volume because it is already labeled: \"%s\"\n"), dev->VolHdr.VolumeName);
+         goto cleanup;
       }
 
-      /* Relabel request. If oldname matches, continue */
+      /*
+       * Relabel request. If oldname matches, continue
+       */
       if (!bstrcmp(oldname, dev->VolHdr.VolumeName)) {
          dir->fsend(_("3921 Wrong volume mounted.\n"));
-         goto bail_out;
+         goto cleanup;
       }
+
       if (dev->label_type != B_BAREOS_LABEL) {
          dir->fsend(_("3922 Cannot relabel an ANSI/IBM labeled Volume.\n"));
-         goto bail_out;
+         goto cleanup;
       }
-      /* Fall through wanted! */
+      /*
+       * Fall through wanted!
+       */
    case VOL_IO_ERROR:
    case VOL_NO_LABEL:
       if (!write_new_volume_label_to_dev(dcr, newname, poolname, relabel)) {
          dir->fsend(_("3912 Failed to label Volume: ERR=%s\n"), dev->bstrerror());
-         goto bail_out;
+         goto cleanup;
       }
       bstrncpy(dcr->VolumeName, newname, sizeof(dcr->VolumeName));
-      /* The following 3000 OK label. string is scanned in ua_label.c */
+
+      /*
+       * The following 3000 OK label. string is scanned in ua_label.c
+       */
       dir->fsend("3000 OK label. VolBytes=%s Volume=\"%s\" Device=%s\n",
                  edit_uint64(dev->VolCatInfo.VolCatBytes, ed1),
                  newname, dev->print_name());
@@ -792,7 +793,7 @@ static void label_volume_if_ok(DCR *dcr, char *oldname,
       break;
    }
 
-bail_out:
+cleanup:
    if (dev->is_open() && !dev->has_cap(CAP_ALWAYSOPEN)) {
       dev->close(dcr);
    }
@@ -800,7 +801,10 @@ bail_out:
       dev->clear_volhdr();
    }
    volume_unused(dcr);                   /* no longer using volume */
+
+bail_out:
    give_back_device_lock(dev, &hold);
+
    return;
 }
 
@@ -840,7 +844,7 @@ static bool read_label(DCR *dcr)
 /*
  * Searches for device by name, and if found, creates a dcr and returns it.
  */
-static DCR *find_device(JCR *jcr, POOL_MEM &devname, int drive, BLOCKSIZES *blocksizes)
+static DCR *find_device(JCR *jcr, POOL_MEM &devname, drive_number_t drive, BLOCKSIZES *blocksizes)
 {
    DEVRES *device;
    AUTOCHANGERRES *changer;
@@ -849,50 +853,57 @@ static DCR *find_device(JCR *jcr, POOL_MEM &devname, int drive, BLOCKSIZES *bloc
 
    unbash_spaces(devname);
    foreach_res(device, R_DEVICE) {
-      /* Find resource, and make sure we were able to open it */
-      if (bstrcmp(device->hdr.name, devname.c_str())) {
+      /*
+       * Find resource, and make sure we were able to open it
+       */
+      if (bstrcmp(device->name(), devname.c_str())) {
          if (!device->dev) {
             device->dev = init_dev(jcr, device);
          }
          if (!device->dev) {
             Jmsg(jcr, M_WARNING, 0, _("\n"
-               "     Device \"%s\" requested by DIR could not be opened or does not exist.\n"),
+                 "     Device \"%s\" requested by DIR could not be opened or does not exist.\n"),
                  devname.c_str());
             continue;
          }
-         Dmsg1(20, "Found device %s\n", device->hdr.name);
+         Dmsg1(20, "Found device %s\n", device->name());
          found = true;
          break;
       }
    }
+
    if (!found) {
       foreach_res(changer, R_AUTOCHANGER) {
-         /* Find resource, and make sure we were able to open it */
-         if (bstrcmp(devname.c_str(), changer->hdr.name)) {
-            /* Try each device in this AutoChanger */
+         /*
+          * Find resource, and make sure we were able to open it
+          */
+         if (bstrcmp(devname.c_str(), changer->name())) {
+            /*
+             * Try each device in this AutoChanger
+             */
             foreach_alist(device, changer->device) {
-               Dmsg1(100, "Try changer device %s\n", device->hdr.name);
+               Dmsg1(100, "Try changer device %s\n", device->name());
                if (!device->dev) {
                   device->dev = init_dev(jcr, device);
                }
                if (!device->dev) {
                   Dmsg1(100, "Device %s could not be opened. Skipped\n", devname.c_str());
                   Jmsg(jcr, M_WARNING, 0, _("\n"
-                     "     Device \"%s\" in changer \"%s\" requested by DIR could not be opened or does not exist.\n"),
-                       device->hdr.name, devname.c_str());
+                       "     Device \"%s\" in changer \"%s\" requested by DIR could not be opened or does not exist.\n"),
+                       device->name(), devname.c_str());
                   continue;
                }
                if (!device->dev->autoselect) {
                   Dmsg1(100, "Device %s not autoselect skipped.\n", devname.c_str());
-                  continue;              /* device is not available */
+                  continue;           /* device is not available */
                }
-               if (drive < 0 || drive == (int)device->dev->drive_index) {
-                  Dmsg1(20, "Found changer device %s\n", device->hdr.name);
+               if (drive < 0 || drive == device->dev->drive) {
+                  Dmsg1(20, "Found changer device %s\n", device->name());
                   found = true;
                   break;
                }
-               Dmsg3(100, "Device %s drive wrong: want=%d got=%d skipping\n",
-                  devname.c_str(), drive, (int)device->dev->drive_index);
+               Dmsg3(100, "Device %s drive wrong: want=%hd got=%hd skipping\n",
+                     devname.c_str(), drive, device->dev->drive);
             }
             break;                    /* we found it but could not open a device */
          }
@@ -900,7 +911,7 @@ static DCR *find_device(JCR *jcr, POOL_MEM &devname, int drive, BLOCKSIZES *bloc
    }
 
    if (found) {
-      Dmsg1(100, "Found device %s\n", device->hdr.name);
+      Dmsg1(100, "Found device %s\n", device->name());
       dcr = New(SD_DCR);
       setup_new_dcr_device(jcr, dcr, device->dev, blocksizes);
       dcr->set_will_write();
@@ -918,8 +929,8 @@ static bool mount_cmd(JCR *jcr)
    BSOCK *dir = jcr->dir_bsock;
    DEVICE *dev;
    DCR *dcr;
-   int32_t drive;
-   int32_t slot = 0;
+   drive_number_t drive;
+   slot_number_t slot = 0;
    bool ok;
 
    ok = sscanf(dir->msg, mountslotcmd, devname.c_str(), &drive, &slot) == 3;
@@ -927,14 +938,14 @@ static bool mount_cmd(JCR *jcr)
       ok = sscanf(dir->msg, mountcmd, devname.c_str(), &drive) == 2;
    }
 
-   Dmsg3(100, "ok=%d drive=%d slot=%d\n", ok, drive, slot);
+   Dmsg3(100, "ok=%d drive=%hd slot=%hd\n", ok, drive, slot);
    if (ok) {
       dcr = find_device(jcr, devname, drive, NULL);
       if (dcr) {
          dev = dcr->dev;
          dev->Lock();                  /* Use P to avoid indefinite block */
-         Dmsg2(100, "mount cmd blocked=%d must_unload=%d\n", dev->blocked(),
-            dev->must_unload());
+         Dmsg2(100, "mount cmd blocked=%d must_unload=%d\n",
+               dev->blocked(), dev->must_unload());
          switch (dev->blocked()) {         /* device blocked? */
          case BST_WAITING_FOR_SYSOP:
             /* Someone is waiting, wake him */
@@ -950,7 +961,7 @@ static bool mount_cmd(JCR *jcr)
          /* In both of these two cases, we (the user) unmounted the Volume */
          case BST_UNMOUNTED_WAITING_FOR_SYSOP:
          case BST_UNMOUNTED:
-            Dmsg2(100, "Unmounted changer=%d slot=%d\n", dev->is_autochanger(), slot);
+            Dmsg2(100, "Unmounted changer=%d slot=%hd\n", dev->is_autochanger(), slot);
             if (dev->is_autochanger() && slot > 0) {
                try_autoload_device(jcr, dcr, slot, "");
             }
@@ -994,7 +1005,7 @@ static bool mount_cmd(JCR *jcr)
             break;
 
          case BST_NOT_BLOCKED:
-            Dmsg2(100, "Not blocked changer=%d slot=%d\n", dev->is_autochanger(), slot);
+            Dmsg2(100, "Not blocked changer=%d slot=%hd\n", dev->is_autochanger(), slot);
             if (dev->is_autochanger() && slot > 0) {
                try_autoload_device(jcr, dcr, slot, "");
             }
@@ -1064,7 +1075,7 @@ static bool unmount_cmd(JCR *jcr)
    BSOCK *dir = jcr->dir_bsock;
    DEVICE *dev;
    DCR *dcr;
-   int32_t drive;
+   drive_number_t drive;
 
    if (sscanf(dir->msg, unmountcmd, devname.c_str(), &drive) == 2) {
       dcr = find_device(jcr, devname, drive, NULL);
@@ -1194,7 +1205,7 @@ static bool release_cmd(JCR *jcr)
    BSOCK *dir = jcr->dir_bsock;
    DEVICE *dev;
    DCR *dcr;
-   int32_t drive;
+   drive_number_t drive;
 
    if (sscanf(dir->msg, releasecmd, devname.c_str(), &drive) == 2) {
       dcr = find_device(jcr, devname, drive, NULL);
@@ -1262,12 +1273,12 @@ static inline bool get_bootstrap_file(JCR *jcr, BSOCK *sock)
    bool ok = false;
 
    if (jcr->RestoreBootstrap) {
-      unlink(jcr->RestoreBootstrap);
+      secure_erase(jcr, jcr->RestoreBootstrap);
       free_pool_memory(jcr->RestoreBootstrap);
    }
    P(bsr_mutex);
    bsr_uniq++;
-   Mmsg(fname, "%s/%s.%s.%d.bootstrap", me->working_directory, me->hdr.name,
+   Mmsg(fname, "%s/%s.%s.%d.bootstrap", me->working_directory, me->name(),
       jcr->Job, bsr_uniq);
    V(bsr_mutex);
    Dmsg1(400, "bootstrap=%s\n", fname);
@@ -1276,7 +1287,7 @@ static inline bool get_bootstrap_file(JCR *jcr, BSOCK *sock)
    if (!bs) {
       berrno be;
       Jmsg(jcr, M_FATAL, 0, _("Could not create bootstrap file %s: ERR=%s\n"),
-         jcr->RestoreBootstrap, be.bstrerror());
+           jcr->RestoreBootstrap, be.bstrerror());
       goto bail_out;
    }
    Dmsg0(10, "=== Bootstrap file ===\n");
@@ -1299,7 +1310,7 @@ static inline bool get_bootstrap_file(JCR *jcr, BSOCK *sock)
    ok = true;
 
 bail_out:
-   unlink(jcr->RestoreBootstrap);
+   secure_erase(jcr, jcr->RestoreBootstrap);
    free_pool_memory(jcr->RestoreBootstrap);
    jcr->RestoreBootstrap = NULL;
    if (!ok) {
@@ -1319,7 +1330,7 @@ static bool bootstrap_cmd(JCR *jcr)
  */
 static bool changer_cmd(JCR *jcr)
 {
-   int src_slot, dst_slot;
+   slot_number_t src_slot, dst_slot;
    POOL_MEM devname;
    BSOCK *dir = jcr->dir_bsock;
    DEVICE *dev;
@@ -1345,7 +1356,7 @@ static bool changer_cmd(JCR *jcr)
    } else if (sscanf(dir->msg, "autochanger drives %127s", devname.c_str()) == 1) {
       cmd = "drives";
       safe_cmd = ok = true;
-   } else if (sscanf(dir->msg, "autochanger transfer %127s %d %d",
+   } else if (sscanf(dir->msg, "autochanger transfer %127s %hd %hd",
                      devname.c_str(), &src_slot, &dst_slot) == 3) {
       cmd = "transfer";
       safe_cmd = ok = true;
@@ -1396,24 +1407,24 @@ static bool readlabel_cmd(JCR *jcr)
    BSOCK *dir = jcr->dir_bsock;
    DEVICE *dev;
    DCR *dcr;
-   int32_t Slot, drive;
+   slot_number_t slot;
+   drive_number_t drive;
 
-   if (sscanf(dir->msg, readlabelcmd, devname.c_str(),
-       &Slot, &drive) == 3) {
+   if (sscanf(dir->msg, readlabelcmd, devname.c_str(), &slot, &drive) == 3) {
       dcr = find_device(jcr, devname, drive, NULL);
       if (dcr) {
          dev = dcr->dev;
          dev->Lock();                 /* Use P to avoid indefinite block */
          if (!dev->is_open()) {
-            read_volume_label(jcr, dcr, dev, Slot);
+            read_volume_label(jcr, dcr, dev, slot);
             dev->close(dcr);
          /* Under certain "safe" conditions, we can steal the lock */
          } else if (dev->can_steal_lock()) {
-            read_volume_label(jcr, dcr, dev, Slot);
+            read_volume_label(jcr, dcr, dev, slot);
          } else if (dev->is_busy() || dev->is_blocked()) {
             send_dir_busy_message(dir, dev);
          } else {                     /* device not being used */
-            read_volume_label(jcr, dcr, dev, Slot);
+            read_volume_label(jcr, dcr, dev, slot);
          }
          dev->Unlock();
          free_dcr(dcr);
@@ -1433,7 +1444,7 @@ static bool readlabel_cmd(JCR *jcr)
  *
  *  Enter with the mutex set
  */
-static void read_volume_label(JCR *jcr, DCR *dcr, DEVICE *dev, int Slot)
+static void read_volume_label(JCR *jcr, DCR *dcr, DEVICE *dev, slot_number_t Slot)
 {
    BSOCK *dir = jcr->dir_bsock;
    bsteal_lock_t hold;
@@ -1441,15 +1452,20 @@ static void read_volume_label(JCR *jcr, DCR *dcr, DEVICE *dev, int Slot)
    dcr->set_dev(dev);
    steal_device_lock(dev, &hold, BST_WRITING_LABEL);
 
-   if (!try_autoload_device(jcr, dcr, Slot, "")) {
-      goto bail_out;                  /* error */
+   switch (try_autoload_device(dcr->jcr, dcr, Slot, "")) {
+   case -1:
+   case -2:
+      goto bail_out;
+   case 0:
+   default:
+      break;
    }
 
    dev->clear_labeled();              /* force read of label */
    switch (read_dev_volume_label(dcr)) {
    case VOL_OK:
       /* DO NOT add quotes around the Volume name. It is scanned in the DIR */
-      dir->fsend(_("3001 Volume=%s Slot=%d\n"), dev->VolHdr.VolumeName, Slot);
+      dir->fsend(_("3001 Volume=%s Slot=%hd\n"), dev->VolHdr.VolumeName, Slot);
       Dmsg1(100, "Volume: %s\n", dev->VolHdr.VolumeName);
       break;
    default:
@@ -1459,20 +1475,27 @@ static void read_volume_label(JCR *jcr, DCR *dcr, DEVICE *dev, int Slot)
 
 bail_out:
    give_back_device_lock(dev, &hold);
+
    return;
 }
 
-static bool try_autoload_device(JCR *jcr, DCR *dcr, int slot, const char *VolName)
+/*
+ * Try autoloading a device.
+ *
+ * Returns: 1 on success
+ *          0 on failure (no changer available)
+ *         -1 on error on autochanger
+ *         -2 on error locking the autochanger
+ */
+static int try_autoload_device(JCR *jcr, DCR *dcr, slot_number_t Slot, const char *VolName)
 {
    BSOCK *dir = jcr->dir_bsock;
 
    bstrncpy(dcr->VolumeName, VolName, sizeof(dcr->VolumeName));
-   dcr->VolCatInfo.Slot = slot;
-   dcr->VolCatInfo.InChanger = slot > 0;
-   if (autoload_device(dcr, 0, dir) < 0) {    /* autoload if possible */
-      return false;
-   }
-   return true;
+   dcr->VolCatInfo.Slot = Slot;
+   dcr->VolCatInfo.InChanger = Slot > 0;
+
+   return autoload_device(dcr, 0, dir);
 }
 
 static void send_dir_busy_message(BSOCK *dir, DEVICE *dev)
