@@ -30,10 +30,11 @@
 
 /*
  * Convert a FileIndex into a printable
- *   ASCII string.  Not reentrant.
+ * ASCII string.  Not reentrant.
+ *
  * If the FileIndex is negative, it flags the
- *   record as a Label, otherwise it is simply
- *   the FileIndex of the current file.
+ * record as a Label, otherwise it is simply
+ * the FileIndex of the current file.
  */
 const char *FI_to_ascii(char *buf, int fi)
 {
@@ -69,14 +70,15 @@ const char *FI_to_ascii(char *buf, int fi)
 
 /*
  * Convert a Stream ID into a printable
- * ASCII string.  Not reentrant.
-
+ * ASCII string. Not reentrant.
+ *
  * A negative stream number represents
- *   stream data that is continued from a
- *   record in the previous block.
+ * stream data that is continued from a
+ * record in the previous block.
+ *
  * If the FileIndex is negative, we are
- *   dealing with a Label, hence the
- *   stream is the JobId.
+ * dealing with a Label, hence the
+ * stream is the JobId.
  */
 const char *stream_to_ascii(char *buf, int stream, int fi)
 {
@@ -85,10 +87,13 @@ const char *stream_to_ascii(char *buf, int stream, int fi)
       sprintf(buf, "%d", stream);
       return buf;
    }
+
    if (stream < 0) {
       stream = -stream;
       stream &= STREAMMASK_TYPE;
-      /* Stream was negative => all are continuation items */
+      /*
+       * Stream was negative => all are continuation items
+       */
       switch (stream) {
       case STREAM_UNIX_ATTRIBUTES:
          return "contUATTR";
@@ -223,6 +228,65 @@ const char *stream_to_ascii(char *buf, int stream, int fi)
    }
 }
 
+static const char *record_state_to_ascii(rec_state state)
+{
+   switch(state) {
+   case st_none:
+      return "st_none";
+   case st_header:
+      return "st_header";
+   case st_header_cont:
+      return "st_header_cont";
+   case st_data:
+      return "st_data";
+   default:
+      return "<unknown>";
+   }
+}
+
+static const char *findex_to_str(int32_t index, char *buf, size_t bufsz)
+{
+   if (index >= 0) {
+      bsnprintf(buf, bufsz, "<User> %d", index);
+      return buf;
+   }
+
+   FI_to_ascii(buf, index);
+
+   return buf;
+}
+
+void dump_record(const char *tag, const DEV_RECORD *rec)
+{
+   char stream[128];
+   char findex[128];
+
+   Dmsg2(100, "%s: rec %p\n", tag, rec);
+
+   Dmsg3(100, "%-14s next %p prev %p\n", "link", rec->link.next, rec->link.prev);
+   Dmsg2(100, "%-14s %u\n", "File", rec->File);
+   Dmsg2(100, "%-14s %u\n", "Block", rec->Block);
+   Dmsg2(100, "%-14s %u\n", "VolSessionId", rec->VolSessionId);
+   Dmsg2(100, "%-14s %u\n", "VolSessionTime", rec->VolSessionTime);
+   Dmsg2(100, "%-14s %s\n", "FileIndex", findex_to_str(rec->FileIndex, findex, sizeof(findex)));
+   Dmsg2(100, "%-14s %s\n", "Stream", stream_to_ascii(stream, rec->Stream, rec->FileIndex));
+   Dmsg2(100, "%-14s %d\n", "maskedStream", rec->maskedStream);
+   Dmsg2(100, "%-14s %u\n", "data_len", rec->data_len);
+   Dmsg2(100, "%-14s %u\n", "remainder", rec->remainder);
+   for (unsigned int i = 0; i < (sizeof(rec->state_bits) / sizeof(rec->state_bits[0])); i++) {
+      Dmsg3(100, "%-11s[%d]        %2.2x\n", "state_bits", i, (uint8_t)rec->state_bits[i]);
+   }
+   Dmsg3(100, "%-14s %u (%s)\n", "state", rec->state, record_state_to_ascii(rec->state));
+   Dmsg2(100, "%-14s %p\n", "bsr", rec->bsr);
+   Dmsg2(100, "%-14s %p\n", "data", rec->data);
+   Dmsg2(100, "%-14s %d\n", "match_stat", rec->match_stat);
+   Dmsg2(100, "%-14s %u\n", "last_VolSessionId", rec->last_VolSessionId);
+   Dmsg2(100, "%-14s %u\n", "last_VolSessionTime", rec->last_VolSessionTime);
+   Dmsg2(100, "%-14s %d\n", "last_FileIndex", rec->last_FileIndex);
+   Dmsg2(100, "%-14s %d\n", "last_Stream", rec->last_Stream);
+   Dmsg2(100, "%-14s %s\n", "own_mempool", rec->own_mempool ? "true" : "false");
+}
+
 /*
  * Return a new record entity
  */
@@ -247,7 +311,12 @@ void empty_record(DEV_RECORD *rec)
    rec->VolSessionId = rec->VolSessionTime = 0;
    rec->FileIndex = rec->Stream = 0;
    rec->data_len = rec->remainder = 0;
-   rec->state_bits &= ~(REC_PARTIAL_RECORD | REC_BLOCK_EMPTY | REC_NO_MATCH | REC_CONTINUATION);
+
+   clear_bit(REC_PARTIAL_RECORD, rec->state_bits);
+   clear_bit(REC_BLOCK_EMPTY, rec->state_bits);
+   clear_bit(REC_NO_MATCH, rec->state_bits);
+   clear_bit(REC_CONTINUATION, rec->state_bits);
+
    rec->state = st_none;
 }
 
@@ -290,70 +359,18 @@ void free_record(DEV_RECORD *rec)
    Dmsg0(950, "Leave free_record.\n");
 }
 
-static inline bool write_header_to_block(DEV_BLOCK *block, DEV_RECORD *rec)
+static inline ssize_t write_header_to_block(DEV_BLOCK *block, const DEV_RECORD *rec, int32_t Stream)
 {
    ser_declare;
 
-   rec->remlen = block->buf_len - block->binbuf;
-
    /*
     * Require enough room to write a full header
     */
-   if (rec->remlen >= WRITE_RECHDR_LENGTH) {
-      ser_begin(block->bufp, WRITE_RECHDR_LENGTH);
-      if (BLOCK_VER == 1) {
-         ser_uint32(rec->VolSessionId);
-         ser_uint32(rec->VolSessionTime);
-      } else {
-         block->VolSessionId = rec->VolSessionId;
-         block->VolSessionTime = rec->VolSessionTime;
-      }
-      ser_int32(rec->FileIndex);
-      ser_int32(rec->Stream);
-      ser_uint32(rec->data_len);
+   if (block_write_navail(block) < WRITE_RECHDR_LENGTH)
+      return -1;
 
-      block->bufp += WRITE_RECHDR_LENGTH;
-      block->binbuf += WRITE_RECHDR_LENGTH;
-      rec->remlen -= WRITE_RECHDR_LENGTH;
-      rec->remainder = rec->data_len;
-      if (rec->FileIndex > 0) {
-         /*
-          * If data record, update what we have in this block
-          */
-         if (block->FirstIndex == 0) {
-            block->FirstIndex = rec->FileIndex;
-         }
-         block->LastIndex = rec->FileIndex;
-      }
-   } else {
-      rec->remainder = rec->data_len + WRITE_RECHDR_LENGTH;
-      return false;
-   }
-
-   return true;
-}
-
-static inline void write_continue_header_to_block(DEV_BLOCK *block, DEV_RECORD *rec)
-{
-   ser_declare;
-
-   rec->remlen = block->buf_len - block->binbuf;
-
-   /*
-    * We have unwritten bytes from a previous
-    * time. Presumably we have a new buffer (possibly
-    * containing a volume label), so the new header
-    * should be able to fit in the block -- otherwise we have
-    * an error.  Note, we have to continue splitting the
-    * data record if it is longer than the block.
-    *
-    * First, write the header.
-    *
-    * Every time we write a header and it is a continuation
-    * of a previous partially written record, we store the
-    * Stream as -Stream in the record header.
-    */
    ser_begin(block->bufp, WRITE_RECHDR_LENGTH);
+
    if (BLOCK_VER == 1) {
       ser_uint32(rec->VolSessionId);
       ser_uint32(rec->VolSessionTime);
@@ -361,24 +378,15 @@ static inline void write_continue_header_to_block(DEV_BLOCK *block, DEV_RECORD *
       block->VolSessionId = rec->VolSessionId;
       block->VolSessionTime = rec->VolSessionTime;
    }
-   ser_int32(rec->FileIndex);
-   if (rec->remainder > rec->data_len) {
-      ser_int32(rec->Stream);      /* normal full header */
-      ser_uint32(rec->data_len);
-      rec->remainder = rec->data_len; /* must still do data record */
-   } else {
-      ser_int32(-rec->Stream);     /* mark this as a continuation record */
-      ser_uint32(rec->remainder);  /* bytes to do */
-   }
 
-   /*
-    * Require enough room to write a full header
-    */
-   ASSERT(rec->remlen >= WRITE_RECHDR_LENGTH);
+   ser_int32(rec->FileIndex);
+   ser_int32(Stream);
+
+   ser_uint32(rec->remainder);   /* each header tracks remaining user bytes to write */
 
    block->bufp += WRITE_RECHDR_LENGTH;
    block->binbuf += WRITE_RECHDR_LENGTH;
-   rec->remlen -= WRITE_RECHDR_LENGTH;
+
    if (rec->FileIndex > 0) {
       /*
        * If data record, update what we have in this block
@@ -388,48 +396,42 @@ static inline void write_continue_header_to_block(DEV_BLOCK *block, DEV_RECORD *
       }
       block->LastIndex = rec->FileIndex;
    }
+
+   return WRITE_RECHDR_LENGTH;
 }
 
-static inline bool write_data_to_block(DEV_BLOCK *block, DEV_RECORD *rec)
+static inline ssize_t write_data_to_block(DEV_BLOCK *block, const DEV_RECORD *rec)
 {
-   rec->remlen = block->buf_len - block->binbuf;
+   uint32_t len;
+
+   len = MIN(rec->remainder, block_write_navail(block));
+   memcpy(block->bufp,
+          ((unsigned char *)rec->data) + (rec->data_len - rec->remainder),
+          len);
+   block->bufp += len;
+   block->binbuf += len;
 
-   /*
-    * Write as much of data as possible
-    */
-   if (rec->remlen >= rec->remainder) {
-      memcpy(block->bufp, rec->data + (rec->data_len - rec->remainder), rec->remainder);
-      block->bufp += rec->remainder;
-      block->binbuf += rec->remainder;
-   } else {
-      memcpy(block->bufp, rec->data + (rec->data_len - rec->remainder), rec->remlen);
 #ifdef xxxxxSMCHECK
-      if (!sm_check_rtn(__FILE__, __LINE__, False)) {
-         /*
-          * We damaged a buffer
-          */
-         Dmsg6(0, "Damaged block FI=%s SessId=%d Strm=%s len=%d\n"
-            "rem=%d remainder=%d\n",
+   if (!sm_check_rtn(__FILE__, __LINE__, False)) {
+      /*
+       * We damaged a buffer
+       */
+      Dmsg7(0, "Damaged block FI=%s SessId=%d Strm=%s datalen=%d\n"
+            "len=%d rem=%d remainder=%d\n",
             FI_to_ascii(buf1, rec->FileIndex), rec->VolSessionId,
-            stream_to_ascii(buf2, rec->Stream, rec->FileIndex), rec->data_len,
-            rec->remlen, rec->remainder);
-         Dmsg5(0, "Damaged block: bufp=%x binbuf=%d buf_len=%d rem=%d moved=%d\n",
-            block->bufp, block->binbuf, block->buf_len, block->buf_len-block->binbuf,
-            rec->remlen);
-         Dmsg2(0, "Damaged block: buf=%x binbuffrombuf=%d \n",
+            stream_to_ascii(buf2, rec->Stream, rec->FileIndex),
+            rec->data_len, len, block_write_navail(block),
+            rec->remainder);
+      Dmsg5(0, "Damaged block: bufp=%x binbuf=%d buf_len=%d rem=%d moved=%d\n",
+            block->bufp, block->binbuf, block->buf_len,
+            block_write_navail(block), len);
+      Dmsg2(0, "Damaged block: buf=%x binbuffrombuf=%d \n",
             block->buf, block->bufp-block->buf);
-         Emsg0(M_ABORT, 0, _("Damaged buffer\n"));
-      }
+      Emsg0(M_ABORT, 0, _("Damaged buffer\n"));
+   }
 #endif
 
-      block->bufp += rec->remlen;
-      block->binbuf += rec->remlen;
-      rec->remainder -= rec->remlen;
-
-      return false;                /* did partial transfer */
-   }
-
-   return true;
+   return len;
 }
 
 /*
@@ -512,6 +514,7 @@ bail_out:
  */
 bool write_record_to_block(DCR *dcr, DEV_RECORD *rec)
 {
+   ssize_t n;
    bool retval = false;
    char buf1[100], buf2[100];
    DEV_BLOCK *block = dcr->block;
@@ -525,11 +528,12 @@ bool write_record_to_block(DCR *dcr, DEV_RECORD *rec)
       ASSERT(block->binbuf == (uint32_t)(block->bufp - block->buf));
       ASSERT(block->buf_len >= block->binbuf);
 
-      Dmsg6(890, "write_record_to_block() FI=%s SessId=%d Strm=%s len=%d\n"
-            "rem=%d remainder=%d\n",
+      Dmsg9(890, "%s() state=%d (%s) FI=%s SessId=%d Strm=%s len=%d "
+            "block_navail=%d remainder=%d\n",
+            __PRETTY_FUNCTION__, rec->state, record_state_to_ascii(rec->state),
             FI_to_ascii(buf1, rec->FileIndex), rec->VolSessionId,
             stream_to_ascii(buf2, rec->Stream, rec->FileIndex), rec->data_len,
-            rec->remlen, rec->remainder);
+            block_write_navail(block), rec->remainder);
 
       switch (rec->state) {
       case st_none:
@@ -537,56 +541,107 @@ bool write_record_to_block(DCR *dcr, DEV_RECORD *rec)
           * Figure out what to do
           */
          rec->state = st_header;
-         continue;              /* go to next state */
+         rec->remainder = rec->data_len;   /* length of data remaining to write */
+         continue;                         /* goto st_header */
 
       case st_header:
          /*
           * Write header
-          *
-          * If rec->remainder is non-zero, we have been called a
-          * second (or subsequent) time to finish writing a record
-          * that did not previously fit into the block.
           */
-          if (!write_header_to_block(block, rec)) {
-             goto bail_out;      /* write block then come back here */
-          }
-          rec->state = st_data;  /* after header, now write data */
-          continue;
-
-      /*
-       * Write continuation header
-       */
-      case st_header_cont:
-         write_continue_header_to_block(block, rec);
-         rec->state = st_data;
-         if (rec->remlen == 0) {
-            goto bail_out;       /* Partial transfer */
+         n = write_header_to_block(block, rec, rec->Stream);
+         if (n < 0) {
+            /*
+             * the header did not fit into the block, so flush the current
+             * block and come back to st_header and try again on the next block.
+             */
+            goto bail_out;
          }
+
+         if (block_write_navail(block) == 0) {
+            /*
+             * The header fit, but no bytes of data will fit,
+             * so flush this block and start the next block with a
+             * continuation header.
+             */
+            rec->state = st_header_cont;
+            goto bail_out;
+         }
+
+         /*
+          * The header fit, and at least one byte of data will fit,
+          * so move to the st_data state and start filling the block
+          * with data bytes
+          */
+         rec->state = st_data;
+         continue;
+
+      case st_header_cont:
+         /*
+          * Write continuation header
+          */
+         n = write_header_to_block(block, rec, -rec->Stream);
+         if (n < 0) {
+            /*
+             * The continuation header wouldn't fit, which is impossible
+             * unless something is broken
+             */
+            Emsg0(M_ABORT, 0, _("couldn't write continuation header\n"));
+         }
+
+         /*
+          * After successfully writing a continuation header, we always start writing
+          * data, even if none will fit into this block.
+          */
+         rec->state = st_data;
+
+         if (block_write_navail(block) == 0) {
+            /*
+             * The header fit, but no bytes of data will fit,
+             * so flush the block and start the next block with
+             * data bytes
+             */
+            goto bail_out;       /* Partial transfer */
+         }
+
          continue;
 
-      /*
-       * Write normal data
-       */
       case st_data:
          /*
-          * Write data
+          * Write normal data
           *
           * Part of it may have already been transferred, and we
           * may not have enough room to transfer the whole this time.
           */
          if (rec->remainder > 0) {
-            if (!write_data_to_block(block, rec)) {
+            n = write_data_to_block(block, rec);
+            if (n < 0) {
+               /*
+                * error appending data to block should be impossible
+                * unless something is broken
+                */
+               Emsg0(M_ABORT, 0, _("data write error\n"));
+            }
+
+            rec->remainder -= n;
+
+            if (rec->remainder > 0) {
+               /*
+                * Could not fit all of the data bytes into this block, so
+                * flush the current block, and start the next block with a
+                * continuation header
+                */
                rec->state = st_header_cont;
                goto bail_out;
             }
          }
+
          rec->remainder = 0;               /* did whole transfer */
          rec->state = st_none;
          retval = true;
          goto bail_out;
 
       default:
-         Dmsg0(000, "Something went wrong. Default state.\n");
+         Emsg1(M_ABORT, 0, _("Something went wrong. Unknown state %d.\n"), rec->state);
          rec->state = st_none;
          retval = true;
          goto bail_out;
@@ -603,28 +658,12 @@ bail_out:
  *  Returns: false on failure
  *           true  on success (all bytes can be written)
  */
-bool can_write_record_to_block(DEV_BLOCK *block, DEV_RECORD *rec)
+bool can_write_record_to_block(DEV_BLOCK *block, const DEV_RECORD *rec)
 {
-   uint32_t remlen;
-
-   remlen = block->buf_len - block->binbuf;
-   if (rec->remainder == 0) {
-      if (remlen >= WRITE_RECHDR_LENGTH) {
-         remlen -= WRITE_RECHDR_LENGTH;
-         rec->remainder = rec->data_len;
-      } else {
-         return false;
-      }
-   } else {
-      return false;
-   }
-   if (rec->remainder > 0 && remlen < rec->remainder) {
-      return false;
-   }
-   return true;
+   return block_write_navail(block) >= WRITE_RECHDR_LENGTH + rec->remainder;
 }
 
-uint64_t get_record_address(DEV_RECORD *rec)
+uint64_t get_record_address(const DEV_RECORD *rec)
 {
    return ((uint64_t)rec->File)<<32 | rec->Block;
 }
@@ -655,9 +694,9 @@ bool read_record_from_block(DCR *dcr, DEV_RECORD *rec)
    /*
     * Clear state flags
     */
-   rec->state_bits = 0;
+   clear_all_bits(REC_STATE_MAX, rec->state_bits);
    if (dcr->block->dev->is_tape()) {
-      rec->state_bits |= REC_ISTAPE;
+      set_bit(REC_ISTAPE, rec->state_bits);
    }
    rec->Block = ((DEVICE *)(dcr->block->dev))->EndBlock;
    rec->File = ((DEVICE *)(dcr->block->dev))->EndFile;
@@ -698,7 +737,7 @@ bool read_record_from_block(DCR *dcr, DEV_RECORD *rec)
        */
       if (rec->remainder && (rec->VolSessionId != VolSessionId ||
                              rec->VolSessionTime != VolSessionTime)) {
-         rec->state_bits |= REC_NO_MATCH;
+         set_bit(REC_NO_MATCH, rec->state_bits);
          Dmsg0(450, "remainder and VolSession doesn't match\n");
          return false;             /* This is from some other Session */
       }
@@ -708,13 +747,12 @@ bool read_record_from_block(DCR *dcr, DEV_RECORD *rec)
        * of a previous partially written record.
        */
       if (Stream < 0) {               /* continuation record? */
-         Dmsg1(500, "Got negative Stream => continuation. remainder=%d\n",
-            rec->remainder);
-         rec->state_bits |= REC_CONTINUATION;
+         Dmsg1(500, "Got negative Stream => continuation. remainder=%d\n", rec->remainder);
+         set_bit(REC_CONTINUATION, rec->state_bits);
          if (!rec->remainder) {       /* if we didn't read previously */
             rec->data_len = 0;        /* return data as if no continuation */
          } else if (rec->Stream != -Stream) {
-            rec->state_bits |= REC_NO_MATCH;
+            set_bit(REC_NO_MATCH, rec->state_bits);
             return false;             /* This is from some other Session */
          }
          rec->Stream = -Stream;       /* set correct Stream */
@@ -749,7 +787,8 @@ bool read_record_from_block(DCR *dcr, DEV_RECORD *rec)
        * then reread.
        */
       Dmsg0(450, "read_record_block: nothing\n");
-      rec->state_bits |= (REC_NO_HEADER | REC_BLOCK_EMPTY);
+      set_bit(REC_NO_HEADER, rec->state_bits);
+      set_bit(REC_BLOCK_EMPTY, rec->state_bits);
       empty_block(dcr->block);                 /* mark block empty */
       return false;
    }
@@ -760,7 +799,8 @@ bool read_record_from_block(DCR *dcr, DEV_RECORD *rec)
        * Something is wrong, force read of next block, abort
        *   continuing with this block.
        */
-      rec->state_bits |= (REC_NO_HEADER | REC_BLOCK_EMPTY);
+      set_bit(REC_NO_HEADER, rec->state_bits);
+      set_bit(REC_BLOCK_EMPTY, rec->state_bits);
       empty_block(dcr->block);
       Jmsg2(dcr->jcr, M_WARNING, 0, _("Sanity check failed. maxlen=%d datalen=%d. Block discarded.\n"),
          MAX_BLOCK_LENGTH, data_bytes);
@@ -795,7 +835,8 @@ bool read_record_from_block(DCR *dcr, DEV_RECORD *rec)
       rec->data_len += remlen;
       rec->remainder = 1;             /* partial record transferred */
       Dmsg1(450, "read_record_block: partial xfered=%d\n", rec->data_len);
-      rec->state_bits |= (REC_PARTIAL_RECORD | REC_BLOCK_EMPTY);
+      set_bit(REC_PARTIAL_RECORD, rec->state_bits);
+      set_bit(REC_BLOCK_EMPTY, rec->state_bits);
       return true;
    }
    rec->remainder = 0;
