@@ -3,7 +3,7 @@
 
    Copyright (C) 2007-2011 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2014 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -31,18 +31,26 @@
 #include "lib/crypto_cache.h"
 
 const int dbglvl = 250;
+#ifdef HAVE_WIN32
+const char *plugin_type = "-sd.dll";
+#else
 const char *plugin_type = "-sd.so";
+#endif
 static alist *sd_plugin_list = NULL;
 
+static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+
 /* Forward referenced functions */
 static bRC bareosGetValue(bpContext *ctx, bsdrVariable var, void *value);
 static bRC bareosSetValue(bpContext *ctx, bsdwVariable var, void *value);
 static bRC bareosRegisterEvents(bpContext *ctx, int nr_events, ...);
+static bRC bareosUnRegisterEvents(bpContext *ctx, int nr_events, ...);
+static bRC bareosGetInstanceCount(bpContext *ctx, int *ret);
 static bRC bareosJobMsg(bpContext *ctx, const char *file, int line,
                         int type, utime_t mtime, const char *fmt, ...);
 static bRC bareosDebugMsg(bpContext *ctx, const char *file, int line,
                           int level, const char *fmt, ...);
-static char *bareosEditDeviceCodes(DCR *dcr, char *omsg,
+static char *bareosEditDeviceCodes(DCR *dcr, POOLMEM *&omsg,
                                    const char *imsg, const char *cmd);
 static char *bareosLookupCryptoKey(const char *VolumeName);
 static bool bareosUpdateVolumeInfo(DCR *dcr);
@@ -63,6 +71,8 @@ static bsdFuncs bfuncs = {
    sizeof(bsdFuncs),
    SD_PLUGIN_INTERFACE_VERSION,
    bareosRegisterEvents,
+   bareosUnRegisterEvents,
+   bareosGetInstanceCount,
    bareosGetValue,
    bareosSetValue,
    bareosJobMsg,
@@ -84,18 +94,20 @@ struct b_plugin_ctx {
    bRC  rc;                                        /* last return code */
    bool disabled;                                  /* set if plugin disabled */
    char events[nbytes_for_bits(SD_NR_EVENTS + 1)]; /* enabled events bitmask */
+   Plugin *plugin;                                 /* pointer to plugin of which this is an instance off */
 };
 
 static inline bool is_event_enabled(bpContext *ctx, bsdEventType eventType)
 {
    b_plugin_ctx *b_ctx;
    if (!ctx) {
-      return true;
+      return false;
    }
    b_ctx = (b_plugin_ctx *)ctx->bContext;
    if (!b_ctx) {
-      return true;
+      return false;
    }
+
    return bit_is_set(eventType, b_ctx->events);
 }
 
@@ -106,6 +118,9 @@ static inline bool is_plugin_disabled(bpContext *ctx)
       return true;
    }
    b_ctx = (b_plugin_ctx *)ctx->bContext;
+   if (!b_ctx) {
+      return true;
+   }
    return b_ctx->disabled;
 }
 
@@ -116,6 +131,25 @@ static inline bool is_plugin_disabled(JCR *jcr)
 }
 #endif
 
+static bool is_ctx_good(bpContext *ctx, JCR *&jcr, b_plugin_ctx *&bctx)
+{
+   if (!ctx) {
+      return false;
+   }
+
+   bctx = (b_plugin_ctx *)ctx->bContext;
+   if (!bctx) {
+      return false;
+   }
+
+   jcr = bctx->jcr;
+   if (!jcr) {
+      return false;
+   }
+
+   return true;
+}
+
 /*
  * Edit codes into ChangerCommand
  *  %% = %
@@ -134,9 +168,8 @@ static inline bool is_plugin_disabled(JCR *jcr)
  *  omsg = edited output message
  *  imsg = input string containing edit codes (%x)
  *  cmd = command string (load, unload, ...)
- *
  */
-char *edit_device_codes(DCR *dcr, char *omsg, const char *imsg, const char *cmd)
+char *edit_device_codes(DCR *dcr, POOLMEM *&omsg, const char *imsg, const char *cmd)
 {
    const char *p;
    const char *str;
@@ -201,32 +234,81 @@ char *edit_device_codes(DCR *dcr, char *omsg, const char *imsg, const char *cmd)
          str = ed1;
       }
       Dmsg1(1900, "add_str %s\n", str);
-      pm_strcat(&omsg, (char *)str);
+      pm_strcat(omsg, (char *)str);
       Dmsg1(1800, "omsg=%s\n", omsg);
    }
    Dmsg1(800, "omsg=%s\n", omsg);
+
    return omsg;
 }
 
-static inline bRC trigger_plugin_event(JCR *jcr, bsdEventType eventType, bsdEvent *event, bpContext *ctx, void *value)
+static inline bool trigger_plugin_event(JCR *jcr, bsdEventType eventType,
+                                        bsdEvent *event, bpContext *ctx,
+                                        void *value, alist *plugin_ctx_list,
+                                        int *index, bRC *rc)
 {
+   bool stop = false;
+
    if (!is_event_enabled(ctx, eventType)) {
       Dmsg1(dbglvl, "Event %d disabled for this plugin.\n", eventType);
-      return bRC_OK;
+      goto bail_out;
    }
 
    if (is_plugin_disabled(ctx)) {
       Dmsg0(dbglvl, "Plugin disabled.\n");
-      return bRC_OK;
+      goto bail_out;
    }
 
-   return sdplug_func(ctx->plugin)->handlePluginEvent(ctx, event, value);
+   /*
+    * See if we should care about the return code.
+    */
+   if (rc) {
+      *rc = sdplug_func(ctx->plugin)->handlePluginEvent(ctx, event, value);
+      switch (*rc) {
+      case bRC_OK:
+         break;
+      case bRC_Stop:
+      case bRC_Error:
+         stop = true;
+         break;
+      case bRC_More:
+         break;
+      case bRC_Term:
+         /*
+          * Request to unload this plugin.
+          * As we remove the plugin from the list of plugins we decrement
+          * the running index value so the next plugin gets triggered as
+          * that moved back a position in the alist.
+          */
+         if (index) {
+            unload_plugin(plugin_ctx_list, ctx->plugin, *index);
+            *index = ((*index) - 1);
+         }
+         break;
+      case bRC_Seen:
+         break;
+      case bRC_Core:
+         break;
+      case bRC_Skip:
+         stop = true;
+         break;
+      case bRC_Cancel:
+         break;
+      default:
+         break;
+      }
+   } else {
+      sdplug_func(ctx->plugin)->handlePluginEvent(ctx, event, value);
+   }
+
+bail_out:
+   return stop;
 }
 
 /*
  * Create a plugin event
  */
-int generate_plugin_event(JCR *jcr, bsdEventType eventType, void *value, bool reverse)
+bRC generate_plugin_event(JCR *jcr, bsdEventType eventType, void *value, bool reverse)
 {
    int i;
    bsdEvent event;
@@ -235,22 +317,20 @@ int generate_plugin_event(JCR *jcr, bsdEventType eventType, void *value, bool re
 
    if (!sd_plugin_list) {
       Dmsg0(dbglvl, "No bplugin_list: generate_plugin_event ignored.\n");
-      return bRC_OK;
+      goto bail_out;
    }
 
    if (!jcr) {
       Dmsg0(dbglvl, "No jcr: generate_plugin_event ignored.\n");
-      return bRC_OK;
+      goto bail_out;
    }
 
+   /*
+    * Return if no plugins loaded
+    */
    if (!jcr->plugin_ctx_list) {
       Dmsg0(dbglvl, "No plugin_ctx_list: generate_plugin_event ignored.\n");
-      return bRC_OK;                  /* Return if no plugins loaded */
-   }
-
-   if (jcr->is_job_canceled()) {
-      Dmsg0(dbglvl, "Cancel return from generate_plugin_event\n");
-      return bRC_Cancel;
+      goto bail_out;
    }
 
    plugin_ctx_list = jcr->plugin_ctx_list;
@@ -265,8 +345,7 @@ int generate_plugin_event(JCR *jcr, bsdEventType eventType, void *value, bool re
       bpContext *ctx;
 
       foreach_alist_rindex(i, ctx, plugin_ctx_list) {
-         rc = trigger_plugin_event(jcr, eventType, &event, ctx, value);
-         if (rc != bRC_OK) {
+         if (trigger_plugin_event(jcr, eventType, &event, ctx, value, plugin_ctx_list, &i, &rc)) {
             break;
          }
       }
@@ -274,13 +353,18 @@ int generate_plugin_event(JCR *jcr, bsdEventType eventType, void *value, bool re
       bpContext *ctx;
 
       foreach_alist_index(i, ctx, plugin_ctx_list) {
-         rc = trigger_plugin_event(jcr, eventType, &event, ctx, value);
-         if (rc != bRC_OK) {
+         if (trigger_plugin_event(jcr, eventType, &event, ctx, value, plugin_ctx_list, &i, &rc)) {
             break;
          }
       }
    }
 
+   if (jcr->is_job_canceled()) {
+      Dmsg0(dbglvl, "Cancel return from generate_plugin_event\n");
+      rc = bRC_Cancel;
+   }
+
+bail_out:
    return rc;
 }
 
@@ -416,6 +500,7 @@ static inline bpContext *instantiate_plugin(JCR *jcr, Plugin *plugin, uint32_t i
    b_ctx = (b_plugin_ctx *)malloc(sizeof(b_plugin_ctx));
    memset(b_ctx, 0, sizeof(b_plugin_ctx));
    b_ctx->jcr = jcr;
+   b_ctx->plugin = plugin;
 
    Dmsg2(dbglvl, "Instantiate dir-plugin_ctx_list=%p JobId=%d\n", jcr->plugin_ctx_list, jcr->JobId);
 
@@ -524,7 +609,7 @@ void dispatch_new_plugin_options(JCR *jcr)
          }
 
          if (ctx) {
-            trigger_plugin_event(jcr, eventType, &event, ctx, (void *)plugin_options);
+            trigger_plugin_event(jcr, eventType, &event, ctx, (void *)plugin_options, NULL, NULL, NULL);
          }
       }
    }
@@ -601,7 +686,7 @@ void free_plugins(JCR *jcr)
 static bRC bareosGetValue(bpContext *ctx, bsdrVariable var, void *value)
 {
    JCR *jcr = NULL;
-   bRC ret = bRC_OK;
+   bRC retval = bRC_OK;
 
    if (!value) {
       return bRC_Error;
@@ -655,7 +740,7 @@ static bRC bareosGetValue(bpContext *ctx, bsdrVariable var, void *value)
             *((char **)value) = jcr->dcr->pool_name;
             Dmsg1(dbglvl, "sd-plugin: return bsdVarPool=%s\n", NPRT(*((char **)value)));
          } else {
-            ret = bRC_Error;
+            retval = bRC_Error;
          }
          break;
       case bsdVarPoolType:
@@ -663,15 +748,15 @@ static bRC bareosGetValue(bpContext *ctx, bsdrVariable var, void *value)
             *((char **)value) = jcr->dcr->pool_type;
             Dmsg1(dbglvl, "sd-plugin: return bsdVarPoolType=%s\n", NPRT(*((char **)value)));
          } else {
-            ret = bRC_Error;
+            retval = bRC_Error;
          }
          break;
       case bsdVarStorage:
          if (jcr->dcr && jcr->dcr->device) {
-            *((char **)value) = jcr->dcr->device->hdr.name;
+            *((char **)value) = jcr->dcr->device->name();
             Dmsg1(dbglvl, "sd-plugin: return bsdVarStorage=%s\n", NPRT(*((char **)value)));
          } else {
-            ret = bRC_Error;
+            retval = bRC_Error;
          }
          break;
       case bsdVarMediaType:
@@ -679,7 +764,7 @@ static bRC bareosGetValue(bpContext *ctx, bsdrVariable var, void *value)
             *((char **)value) = jcr->dcr->media_type;
             Dmsg1(dbglvl, "sd-plugin: return bsdVarMediaType=%s\n", NPRT(*((char **)value)));
          } else {
-            ret = bRC_Error;
+            retval = bRC_Error;
          }
          break;
       case bsdVarJobName:
@@ -695,7 +780,7 @@ static bRC bareosGetValue(bpContext *ctx, bsdrVariable var, void *value)
             *((char **)value) = jcr->dcr->VolumeName;
             Dmsg1(dbglvl, "sd-plugin: return bsdVarVolumeName=%s\n", NPRT(*((char **)value)));
          } else {
-            ret = bRC_Error;
+            retval = bRC_Error;
          }
          Dmsg1(dbglvl, "sd-plugin: return bsdVarVolumeName=%s\n", jcr->VolumeName);
          break;
@@ -716,7 +801,7 @@ static bRC bareosGetValue(bpContext *ctx, bsdrVariable var, void *value)
       }
    }
 
-   return ret;
+   return retval;
 }
 
 static bRC bareosSetValue(bpContext *ctx, bsdwVariable var, void *value)
@@ -763,13 +848,69 @@ static bRC bareosRegisterEvents(bpContext *ctx, int nr_events, ...)
    va_start(args, nr_events);
    for (i = 0; i < nr_events; i++) {
       event = va_arg(args, uint32_t);
-      Dmsg1(dbglvl, "sd-Plugin wants event=%u\n", event);
+      Dmsg1(dbglvl, "sd-plugin: Plugin registered event=%u\n", event);
       set_bit(event, b_ctx->events);
    }
    va_end(args);
    return bRC_OK;
 }
 
+static bRC bareosUnRegisterEvents(bpContext *ctx, int nr_events, ...)
+{
+   int i;
+   va_list args;
+   uint32_t event;
+   b_plugin_ctx *b_ctx;
+
+   if (!ctx) {
+      return bRC_Error;
+   }
+   b_ctx = (b_plugin_ctx *)ctx->bContext;
+   va_start(args, nr_events);
+   for (i = 0; i < nr_events; i++) {
+      event = va_arg(args, uint32_t);
+      Dmsg1(dbglvl, "sd-plugin: Plugin unregistered event=%u\n", event);
+      clear_bit(event, b_ctx->events);
+   }
+   va_end(args);
+   return bRC_OK;
+}
+
+static bRC bareosGetInstanceCount(bpContext *ctx, int *ret)
+{
+   int cnt;
+   JCR *jcr, *njcr;
+   bpContext *nctx;
+   b_plugin_ctx *bctx;
+   bRC retval = bRC_Error;
+
+   if (!is_ctx_good(ctx, jcr, bctx)) {
+      goto bail_out;
+   }
+
+   P(mutex);
+
+   cnt = 0;
+   foreach_jcr(njcr) {
+      if (jcr->plugin_ctx_list) {
+         foreach_alist(nctx, jcr->plugin_ctx_list) {
+            if (nctx->plugin == bctx->plugin) {
+               cnt++;
+            }
+         }
+      }
+   }
+   endeach_jcr(njcr);
+
+   V(mutex);
+
+   *ret = cnt;
+   retval = bRC_OK;
+
+bail_out:
+   return retval;
+}
+
 static bRC bareosJobMsg(bpContext *ctx, const char *file, int line,
                         int type, utime_t mtime, const char *fmt, ...)
 {
@@ -805,7 +946,7 @@ static bRC bareosDebugMsg(bpContext *ctx, const char *file, int line,
    return bRC_OK;
 }
 
-static char *bareosEditDeviceCodes(DCR *dcr, char *omsg,
+static char *bareosEditDeviceCodes(DCR *dcr, POOLMEM *&omsg,
                                    const char *imsg, const char *cmd)
 {
    return edit_device_codes(dcr, omsg, imsg, cmd);
@@ -826,7 +967,7 @@ static void bareosUpdateTapeAlert(DCR *dcr, uint64_t flags)
    utime_t now;
    now = (utime_t)time(NULL);
 
-   update_device_tapealert(dcr->device->hdr.name, flags, now);
+   update_device_tapealert(dcr->device->name(), flags, now);
 }
 
 static DEV_RECORD *bareosNewRecord(bool with_data)
