@@ -33,6 +33,9 @@
 
 #include "bareos.h"
 #include "jcr.h"
+#include "findlib/find.h"
+#define FILE_DAEMON 1
+#include "fd_plugins.h"
 
 #undef setlocale
 
@@ -71,9 +74,6 @@ using namespace std;
 #include <strsafe.h>
 #endif
 
-bool VSSPathConvert(const char *szFilePath, char *szShadowPath, int nBuflen);
-bool VSSPathConvertW(const wchar_t *szFilePath, wchar_t *szShadowPath, int nBuflen);
-
 #ifdef HAVE_MINGW
 class IXMLDOMDocument;
 #endif
@@ -107,22 +107,16 @@ class IXMLDOMDocument;
 
 #define VSS_ERROR_OBJECT_ALREADY_EXISTS 0x8004230D
 
-
-/* In VSSAPI.DLL */
-typedef HRESULT (STDAPICALLTYPE* t_CreateVssBackupComponents)(OUT IVssBackupComponents **);
-typedef void (APIENTRY* t_VssFreeSnapshotProperties)(IN VSS_SNAPSHOT_PROP*);
-
-static t_CreateVssBackupComponents p_CreateVssBackupComponents = NULL;
-static t_VssFreeSnapshotProperties p_VssFreeSnapshotProperties = NULL;
-
 #include "vss.h"
 
 static void JmsgVssApiStatus(JCR *jcr, int msg_status, HRESULT hr, const char *apiName)
 {
    const char *errmsg;
+
    if (hr == S_OK || hr == VSS_S_ASYNC_FINISHED) {
       return;
    }
+
    switch (hr) {
    case E_INVALIDARG:
       errmsg = "One of the parameter values is not valid.";
@@ -328,7 +322,7 @@ static inline POOLMEM *GetMountedVolumeForMountPointPath(POOLMEM *volumepath, PO
    Dmsg3(200, "%s%s mounts volume %s\n", volumepath, mountpoint, buf);
 
    vol = get_pool_memory(PM_FNAME);
-   UTF8_2_wchar(&vol, buf);
+   UTF8_2_wchar(vol, buf);
 
    free_pool_memory(fullPath);
    free_pool_memory(buf);
@@ -338,8 +332,8 @@ static inline POOLMEM *GetMountedVolumeForMountPointPath(POOLMEM *volumepath, PO
 
 static inline bool HandleVolumeMountPoint(VSSClientGeneric *pVssClient,
                                           IVssBackupComponents *pVssObj,
-                                          POOLMEM *volumepath,
-                                          POOLMEM *mountpoint)
+                                          POOLMEM *&volumepath,
+                                          POOLMEM *&mountpoint)
 {
    bool retval = false;
    HRESULT hr;
@@ -351,13 +345,13 @@ static inline bool HandleVolumeMountPoint(VSSClientGeneric *pVssClient,
    hr = pVssObj->AddToSnapshotSet((LPWSTR)vol, GUID_NULL, &pid);
 
    pvol = get_pool_memory(PM_FNAME);
-   wchar_2_UTF8(&pvol, (wchar_t *)vol);
+   wchar_2_UTF8(pvol, (wchar_t *)vol);
 
    if (SUCCEEDED(hr)) {
       pVssClient->AddVolumeMountPointSnapshots(pVssObj, (wchar_t *)vol);
       Dmsg1(200, "%s added to snapshotset \n", pvol);
       retval = true;
-   } else if((unsigned)hr == VSS_ERROR_OBJECT_ALREADY_EXISTS) {
+   } else if ((unsigned)hr == VSS_ERROR_OBJECT_ALREADY_EXISTS) {
       Dmsg1(200, "%s already in snapshotset, skipping.\n" ,pvol);
    } else {
       Dmsg3(200, "%s with vmp %s could not be added to snapshotset, COM ERROR: 0x%X\n", vol, mountpoint, hr);
@@ -420,8 +414,8 @@ VSSClientGeneric::VSSClientGeneric()
 {
    m_hLib = LoadLibraryA("VSSAPI.DLL");
    if (m_hLib) {
-      p_CreateVssBackupComponents = (t_CreateVssBackupComponents) GetProcAddress(m_hLib, VSSVBACK_ENTRY);
-      p_VssFreeSnapshotProperties = (t_VssFreeSnapshotProperties) GetProcAddress(m_hLib, "VssFreeSnapshotProperties");
+      m_CreateVssBackupComponents = (t_CreateVssBackupComponents) GetProcAddress(m_hLib, VSSVBACK_ENTRY);
+      m_VssFreeSnapshotProperties = (t_VssFreeSnapshotProperties) GetProcAddress(m_hLib, "VssFreeSnapshotProperties");
    }
 }
 
@@ -442,13 +436,13 @@ bool VSSClientGeneric::Initialize(DWORD dwContext, bool bDuringRestore)
 {
    VMPs = 0;
    VMP_snapshots = 0;
-   HRESULT hr;
+   HRESULT hr = S_OK;
    CComPtr<IVssAsync> pAsync1;
-   VSS_BACKUP_TYPE backup_type;
    IVssBackupComponents *pVssObj = (IVssBackupComponents *)m_pVssObject;
 
-   if (!(p_CreateVssBackupComponents && p_VssFreeSnapshotProperties)) {
-      Dmsg2(0, "VSSClientGeneric::Initialize: p_CreateVssBackupComponents=0x%08X, p_VssFreeSnapshotProperties=0x%08X\n", p_CreateVssBackupComponents, p_VssFreeSnapshotProperties);
+   if (!(m_CreateVssBackupComponents && m_VssFreeSnapshotProperties)) {
+      Dmsg2(0, "VSSClientGeneric::Initialize: m_CreateVssBackupComponents=0x%08X, m_VssFreeSnapshotProperties=0x%08X\n",
+            m_CreateVssBackupComponents, m_VssFreeSnapshotProperties);
       Jmsg(m_jcr, M_FATAL, 0, "Entry point CreateVssBackupComponents or VssFreeSnapshotProperties missing.\n");
       return false;
    }
@@ -470,24 +464,9 @@ bool VSSClientGeneric::Initialize(DWORD dwContext, bool bDuringRestore)
    /*
     * Initialize COM security
     */
-   if (!m_bCoInitializeSecurityCalled) {
-      hr = CoInitializeSecurity(NULL,                          /*  Allow *all* VSS writers to communicate back! */
-                                -1,                            /*  Default COM authentication service */
-                                NULL,                          /*  Default COM authorization service */
-                                NULL,                          /*  reserved parameter */
-                                RPC_C_AUTHN_LEVEL_PKT_PRIVACY, /*  Strongest COM authentication level */
-                                RPC_C_IMP_LEVEL_IDENTIFY,      /*  Minimal impersonation abilities */
-                                NULL,                          /*  Default COM authentication settings */
-                                EOAC_NONE,                     /*  No special options */
-                                NULL);                         /*  Reserved parameter */
-
-      if (FAILED(hr)) {
-         Dmsg1(0, "VSSClientGeneric::Initialize: CoInitializeSecurity returned 0x%08X\n", hr);
-         JmsgVssApiStatus(m_jcr, M_FATAL, hr, "CoInitializeSecurity");
-         errno = b_errno_win32;
-         return false;
-      }
-      m_bCoInitializeSecurityCalled = true;
+   if (!initialize_com_security()) {
+      JmsgVssApiStatus(m_jcr, M_FATAL, hr, "CoInitializeSecurity");
+      return false;
    }
 
    /*
@@ -501,7 +480,7 @@ bool VSSClientGeneric::Initialize(DWORD dwContext, bool bDuringRestore)
    /*
     * Create new internal backup components object
     */
-   hr = p_CreateVssBackupComponents((IVssBackupComponents**)&m_pVssObject);
+   hr = m_CreateVssBackupComponents((IVssBackupComponents**)&m_pVssObject);
    if (FAILED(hr)) {
       berrno be;
       Dmsg2(0, "VSSClientGeneric::Initialize: CreateVssBackupComponents returned 0x%08X. ERR=%s\n",
@@ -516,7 +495,6 @@ bool VSSClientGeneric::Initialize(DWORD dwContext, bool bDuringRestore)
     */
    pVssObj = (IVssBackupComponents *)m_pVssObject;
 
-
    if (!bDuringRestore) {
 #if defined(B_VSS_W2K3) || defined(B_VSS_VISTA)
       if (dwContext != VSS_CTX_BACKUP) {
@@ -543,32 +521,40 @@ bool VSSClientGeneric::Initialize(DWORD dwContext, bool bDuringRestore)
 
       /*
        * 2. SetBackupState
+       *
+       * Generate a bEventVssSetBackupState event and if none of the plugins
+       * give back a bRC_Skip it means this will not be performed by any plugin
+       * and we should do the generic handling ourself in the core.
        */
-      switch (m_jcr->getJobLevel()) {
-      case L_FULL:
-         backup_type = VSS_BT_FULL;
-         break;
-      case L_DIFFERENTIAL:
-         backup_type = VSS_BT_DIFFERENTIAL;
-         break;
-      case L_INCREMENTAL:
-         backup_type = VSS_BT_INCREMENTAL;
-         break;
-      default:
-         Dmsg1(0, "VSSClientGeneric::Initialize: unknown backup level %d\n", m_jcr->getJobLevel());
-         backup_type = VSS_BT_FULL;
-         break;
-      }
+      if (generate_plugin_event(m_jcr, bEventVssSetBackupState) != bRC_Skip) {
+         VSS_BACKUP_TYPE backup_type;
 
-      /*
-       * FIXME: need to support partial files - make last parameter true when done
-       */
-      hr = pVssObj->SetBackupState(true, true, backup_type, false);
-      if (FAILED(hr)) {
-         Dmsg1(0, "VSSClientGeneric::Initialize: IVssBackupComponents->SetBackupState returned 0x%08X\n", hr);
-         JmsgVssApiStatus(m_jcr, M_FATAL, hr, "SetBackupState");
-         errno = b_errno_win32;
-         return false;
+         switch (m_jcr->getJobLevel()) {
+         case L_FULL:
+            backup_type = VSS_BT_FULL;
+            break;
+         case L_DIFFERENTIAL:
+            backup_type = VSS_BT_DIFFERENTIAL;
+            break;
+         case L_INCREMENTAL:
+            backup_type = VSS_BT_INCREMENTAL;
+            break;
+         default:
+            Dmsg1(0, "VSSClientGeneric::Initialize: unknown backup level %d\n", m_jcr->getJobLevel());
+            backup_type = VSS_BT_FULL;
+            break;
+         }
+
+         /*
+          * FIXME: need to support partial files - make last parameter true when done
+          */
+         hr = pVssObj->SetBackupState(false, false, backup_type, false);
+         if (FAILED(hr)) {
+            Dmsg1(0, "VSSClientGeneric::Initialize: IVssBackupComponents->SetBackupState returned 0x%08X\n", hr);
+            JmsgVssApiStatus(m_jcr, M_FATAL, hr, "SetBackupState");
+            errno = b_errno_win32;
+            return false;
+         }
       }
 
       /*
@@ -673,7 +659,7 @@ void VSSClientGeneric::AddDriveSnapshots(IVssBackupComponents *pVssObj, char *sz
 
             POOLMEM *szBuf = get_pool_memory(PM_FNAME);
 
-            wchar_2_UTF8(&szBuf, volume.c_str());
+            wchar_2_UTF8(szBuf, volume.c_str());
             Dmsg2(200, "%s added to snapshotset (Drive %s:\\)\n", szBuf, szDrive);
             free_pool_memory(szBuf);
          }
@@ -705,7 +691,7 @@ void VSSClientGeneric::AddVolumeMountPointSnapshots(IVssBackupComponents *pVssOb
    mp = get_pool_memory(PM_FNAME);
    path = get_pool_memory(PM_FNAME);
 
-   wchar_2_UTF8(&path, volume);
+   wchar_2_UTF8(path, volume);
 
    len = wcslen(volume) + 1;
 
@@ -767,12 +753,18 @@ bool VSSClientGeneric::CreateSnapshots(char *szDriveLetters, bool onefs_disabled
 
    m_uidCurrentSnapshotSet = GUID_NULL;
 
-   pVssObj = (IVssBackupComponents*)m_pVssObject;
+   pVssObj = (IVssBackupComponents *)m_pVssObject;
 
    /*
     * startSnapshotSet
     */
    hr = pVssObj->StartSnapshotSet(&m_uidCurrentSnapshotSet);
+   while ((unsigned)hr == VSS_E_SNAPSHOT_SET_IN_PROGRESS) {
+      bmicrosleep(5,0);
+      Jmsg(m_jcr, M_INFO, 0, "VSS_E_SNAPSHOT_SET_IN_PROGRESS, retrying ...\n");
+      hr = pVssObj->StartSnapshotSet(&m_uidCurrentSnapshotSet);
+   }
+
    if (FAILED(hr)) {
       JmsgVssApiStatus(m_jcr, M_FATAL, hr, "StartSnapshotSet");
       errno = ENOSYS;
@@ -789,6 +781,7 @@ bool VSSClientGeneric::CreateSnapshots(char *szDriveLetters, bool onefs_disabled
    /*
     * PrepareForBackup
     */
+   generate_plugin_event(m_jcr, bEventVssPrepareSnapshot);
    hr = pVssObj->PrepareForBackup(&pAsync1.p);
    if (FAILED(hr)) {
       JmsgVssApiStatus(m_jcr, M_FATAL, hr, "PrepareForBackup");
@@ -843,8 +836,6 @@ bool VSSClientGeneric::CreateSnapshots(char *szDriveLetters, bool onefs_disabled
     */
    QuerySnapshotSet(m_uidCurrentSnapshotSet);
 
-   SetVSSPathConvert(VSSPathConvert, VSSPathConvertW);
-
    m_bBackupIsInitialized = true;
 
    return true;
@@ -855,16 +846,14 @@ bool VSSClientGeneric::CloseBackup()
    bool bRet = false;
    HRESULT hr;
    BSTR xml;
-   IVssBackupComponents* pVssObj = (IVssBackupComponents*)m_pVssObject;
+   IVssBackupComponents *pVssObj = (IVssBackupComponents *)m_pVssObject;
 
    if (!m_pVssObject) {
       Jmsg(m_jcr, M_FATAL, 0, "VssOject is NULL.\n");
       errno = ENOSYS;
       return bRet;
    }
-   CComPtr<IVssAsync>  pAsync;
-
-   SetVSSPathConvert(NULL, NULL);
+   CComPtr<IVssAsync> pAsync;
 
    m_bBackupIsInitialized = false;
 
@@ -928,14 +917,14 @@ bool VSSClientGeneric::CloseBackup()
    return bRet;
 }
 
-WCHAR *VSSClientGeneric::GetMetadata()
+wchar_t *VSSClientGeneric::GetMetadata()
 {
    return m_metadata;
 }
 
 bool VSSClientGeneric::CloseRestore()
 {
-   IVssBackupComponents* pVssObj = (IVssBackupComponents*)m_pVssObject;
+   IVssBackupComponents *pVssObj = (IVssBackupComponents *)m_pVssObject;
    CComPtr<IVssAsync> pAsync;
 
    if (!pVssObj) {
@@ -950,7 +939,7 @@ bool VSSClientGeneric::CloseRestore()
  */
 void VSSClientGeneric::QuerySnapshotSet(GUID snapshotSetID)
 {
-   if (!(p_CreateVssBackupComponents && p_VssFreeSnapshotProperties)) {
+   if (!(m_CreateVssBackupComponents && m_VssFreeSnapshotProperties)) {
       Jmsg(m_jcr, M_FATAL, 0, "CreateVssBackupComponents or VssFreeSnapshotProperties API is NULL.\n");
       errno = ENOSYS;
       return;
@@ -964,7 +953,7 @@ void VSSClientGeneric::QuerySnapshotSet(GUID snapshotSetID)
       return;
    }
 
-   IVssBackupComponents* pVssObj = (IVssBackupComponents*) m_pVssObject;
+   IVssBackupComponents *pVssObj = (IVssBackupComponents *)m_pVssObject;
 
    /*
     * Get list all shadow copies.
@@ -1014,7 +1003,7 @@ void VSSClientGeneric::QuerySnapshotSet(GUID snapshotSetID)
             }
          }
       }
-      p_VssFreeSnapshotProperties(&Snap);
+      m_VssFreeSnapshotProperties(&Snap);
    }
    errno = 0;
 }
@@ -1024,126 +1013,127 @@ void VSSClientGeneric::QuerySnapshotSet(GUID snapshotSetID)
  */
 bool VSSClientGeneric::CheckWriterStatus()
 {
-    /*
-     * See http://msdn.microsoft.com/en-us/library/windows/desktop/aa382870%28v=vs.85%29.aspx
-     */
-    IVssBackupComponents* pVssObj = (IVssBackupComponents*)m_pVssObject;
-    if (!pVssObj) {
-       Jmsg(m_jcr, M_FATAL, 0, "Cannot get IVssBackupComponents pointer.\n");
-       errno = ENOSYS;
-       return false;
-    }
-    DestroyWriterInfo();
+   /*
+    * See http://msdn.microsoft.com/en-us/library/windows/desktop/aa382870%28v=vs.85%29.aspx
+    */
+   IVssBackupComponents *pVssObj = (IVssBackupComponents *)m_pVssObject;
+   if (!pVssObj) {
+      Jmsg(m_jcr, M_FATAL, 0, "Cannot get IVssBackupComponents pointer.\n");
+      errno = ENOSYS;
+      return false;
+   }
+   DestroyWriterInfo();
 
-    if (m_bWriterStatusCurrent) {
-       m_bWriterStatusCurrent = false;
-       pVssObj->FreeWriterStatus();
-    }
+   if (m_bWriterStatusCurrent) {
+      m_bWriterStatusCurrent = false;
+      pVssObj->FreeWriterStatus();
+   }
 
-    /*
-     * Gather writer status to detect potential errors
-     */
-    CComPtr<IVssAsync>  pAsync;
+   /*
+    * Gather writer status to detect potential errors
+    */
+   CComPtr<IVssAsync>  pAsync;
 
-    HRESULT hr = pVssObj->GatherWriterStatus(&pAsync.p);
-    if (FAILED(hr)) {
-       JmsgVssApiStatus(m_jcr, M_FATAL, hr, "GatherWriterStatus");
-       errno = b_errno_win32;
-       return false;
-    }
+   HRESULT hr = pVssObj->GatherWriterStatus(&pAsync.p);
+   if (FAILED(hr)) {
+      JmsgVssApiStatus(m_jcr, M_FATAL, hr, "GatherWriterStatus");
+      errno = b_errno_win32;
+      return false;
+   }
 
-    /*
-     * Waits for the async operation to finish and checks the result
-     */
-    if (!WaitAndCheckForAsyncOperation(pAsync.p)) {
-       errno = b_errno_win32;
-       return false;
-    }
+   /*
+    * Waits for the async operation to finish and checks the result
+    */
+   if (!WaitAndCheckForAsyncOperation(pAsync.p)) {
+      errno = b_errno_win32;
+      return false;
+   }
 
-    m_bWriterStatusCurrent = true;
+   m_bWriterStatusCurrent = true;
 
-    unsigned cWriters = 0;
+   unsigned cWriters = 0;
 
-    hr = pVssObj->GetWriterStatusCount(&cWriters);
-    if (FAILED(hr)) {
-       JmsgVssApiStatus(m_jcr, M_FATAL, hr, "GatherWriterStatusCount");
-       errno = b_errno_win32;
-       return false;
-    }
+   hr = pVssObj->GetWriterStatusCount(&cWriters);
+   if (FAILED(hr)) {
+      JmsgVssApiStatus(m_jcr, M_FATAL, hr, "GatherWriterStatusCount");
+      errno = b_errno_win32;
+      return false;
+   }
 
     int nState;
     POOLMEM *szBuf = get_pool_memory(PM_FNAME);
 
-    /*
-     * Enumerate each writer
-     */
-    for (unsigned iWriter = 0; iWriter < cWriters; iWriter++) {
-        VSS_ID idInstance = GUID_NULL;
-        VSS_ID idWriter= GUID_NULL;
-        VSS_WRITER_STATE eWriterStatus = VSS_WS_UNKNOWN;
-        CComBSTR bstrWriterName;
-        HRESULT hrWriterFailure = S_OK;
+   /*
+    * Enumerate each writer
+    */
+   for (unsigned iWriter = 0; iWriter < cWriters; iWriter++) {
+      VSS_ID idInstance = GUID_NULL;
+      VSS_ID idWriter= GUID_NULL;
+      VSS_WRITER_STATE eWriterStatus = VSS_WS_UNKNOWN;
+      CComBSTR bstrWriterName;
+      HRESULT hrWriterFailure = S_OK;
 
-        /*
-         * Get writer status
-         */
-        hr = pVssObj->GetWriterStatus(iWriter,
-                                      &idInstance,
-                                      &idWriter,
-                                      &bstrWriterName,
-                                      &eWriterStatus,
-                                      &hrWriterFailure);
-        if (FAILED(hr)) {
-            /*
-             * Api failed
-             */
-            JmsgVssApiStatus(m_jcr, M_WARNING, hr, "GetWriterStatus");
-            nState = 0;         /* Unknown writer state -- API failed */
-        } else {
-            switch(eWriterStatus) {
-            case VSS_WS_FAILED_AT_IDENTIFY:
-            case VSS_WS_FAILED_AT_PREPARE_BACKUP:
-            case VSS_WS_FAILED_AT_PREPARE_SNAPSHOT:
-            case VSS_WS_FAILED_AT_FREEZE:
-            case VSS_WS_FAILED_AT_THAW:
-            case VSS_WS_FAILED_AT_POST_SNAPSHOT:
-            case VSS_WS_FAILED_AT_BACKUP_COMPLETE:
-            case VSS_WS_FAILED_AT_PRE_RESTORE:
-            case VSS_WS_FAILED_AT_POST_RESTORE:
+      /*
+       * Get writer status
+       */
+      hr = pVssObj->GetWriterStatus(iWriter,
+                                    &idInstance,
+                                    &idWriter,
+                                    &bstrWriterName,
+                                    &eWriterStatus,
+                                    &hrWriterFailure);
+      if (FAILED(hr)) {
+          /*
+           * Api failed
+           */
+          JmsgVssApiStatus(m_jcr, M_WARNING, hr, "GetWriterStatus");
+          nState = 0;         /* Unknown writer state -- API failed */
+      } else {
+          switch(eWriterStatus) {
+          case VSS_WS_FAILED_AT_IDENTIFY:
+          case VSS_WS_FAILED_AT_PREPARE_BACKUP:
+          case VSS_WS_FAILED_AT_PREPARE_SNAPSHOT:
+          case VSS_WS_FAILED_AT_FREEZE:
+          case VSS_WS_FAILED_AT_THAW:
+          case VSS_WS_FAILED_AT_POST_SNAPSHOT:
+          case VSS_WS_FAILED_AT_BACKUP_COMPLETE:
+          case VSS_WS_FAILED_AT_PRE_RESTORE:
+          case VSS_WS_FAILED_AT_POST_RESTORE:
 #if defined(B_VSS_W2K3) || defined(B_VSS_VISTA)
-            case VSS_WS_FAILED_AT_BACKUPSHUTDOWN:
+          case VSS_WS_FAILED_AT_BACKUPSHUTDOWN:
 #endif
-               /*
-                * Writer status problem
-                */
-               wchar_2_UTF8(&szBuf, bstrWriterName.p);
-               JmsgVssWriterStatus(m_jcr, M_WARNING, eWriterStatus, szBuf);
-               nState = -1;       /* bad writer state */
-               break;
-            default:
-               nState = 1;        /* Writer state OK */
-            }
-        }
+             /*
+              * Writer status problem
+              */
+             wchar_2_UTF8(szBuf, bstrWriterName.p);
+             JmsgVssWriterStatus(m_jcr, M_WARNING, eWriterStatus, szBuf);
+             nState = -1;       /* bad writer state */
+             break;
+          default:
+             nState = 1;        /* Writer state OK */
+          }
+       }
 
-        /*
-         * store text info
-         */
-        char str[1000];
-        bstrncpy(str, "\"", sizeof(str));
-        wchar_2_UTF8(&szBuf, bstrWriterName.p);
-        bstrncat(str, szBuf, sizeof(str));
-        bstrncat(str, "\", State: 0x", sizeof(str));
-        itoa(eWriterStatus, szBuf, sizeof_pool_memory(szBuf));
-        bstrncat(str, szBuf, sizeof(str));
-        bstrncat(str, " (", sizeof(str));
-        wchar_2_UTF8(&szBuf, GetStringFromWriterStatus(eWriterStatus));
-        bstrncat(str, szBuf, sizeof(str));
-        bstrncat(str, ")", sizeof(str));
-        AppendWriterInfo(nState, (const char *)str);
-    }
+       /*
+        * store text info
+        */
+       char str[1000];
+       bstrncpy(str, "\"", sizeof(str));
+       wchar_2_UTF8(szBuf, bstrWriterName.p);
+       bstrncat(str, szBuf, sizeof(str));
+       bstrncat(str, "\", State: 0x", sizeof(str));
+       itoa(eWriterStatus, szBuf, sizeof_pool_memory(szBuf));
+       bstrncat(str, szBuf, sizeof(str));
+       bstrncat(str, " (", sizeof(str));
+       wchar_2_UTF8(szBuf, GetStringFromWriterStatus(eWriterStatus));
+       bstrncat(str, szBuf, sizeof(str));
+       bstrncat(str, ")", sizeof(str));
+       AppendWriterInfo(nState, (const char *)str);
+   }
 
-    free_pool_memory(szBuf);
-    errno = 0;
-    return true;
+   free_pool_memory(szBuf);
+   errno = 0;
+
+   return true;
 }
 #endif /* WIN32_VSS */
